@@ -1,5 +1,7 @@
 extern crate cargo;
 extern crate env_logger;
+#[macro_use]
+extern crate failure;
 extern crate git2_curl;
 extern crate toml;
 #[macro_use]
@@ -15,8 +17,8 @@ use std::fs;
 use std::path::{Path, PathBuf};
 
 use cargo::core::shell::{Shell, Verbosity};
-use cargo::util::{self, CliResult, lev_distance, Config, CargoResult, CargoError, CargoErrorKind};
-use cargo::util::CliError;
+use cargo::util::{self, CliResult, lev_distance, Config, CargoResult};
+use cargo::util::{CliError, ProcessError};
 
 #[derive(Deserialize)]
 pub struct Flags {
@@ -73,9 +75,9 @@ See 'cargo help <command>' for more information on a specific command.
 ";
 
 fn main() {
-    env_logger::init().unwrap();
+    env_logger::init();
 
-    let config = match Config::default() {
+    let mut config = match Config::default() {
         Ok(cfg) => cfg,
         Err(e) => {
             let mut shell = Shell::new();
@@ -87,12 +89,12 @@ fn main() {
         let args: Vec<_> = try!(env::args_os()
             .map(|s| {
                 s.into_string().map_err(|s| {
-                    CargoError::from(format!("invalid unicode in argument: {:?}", s))
+                    format_err!("invalid unicode in argument: {:?}", s)
                 })
             })
             .collect());
         let rest = &args;
-        cargo::call_main_without_stdin(execute, &config, USAGE, rest, true)
+        cargo::call_main_without_stdin(execute, &mut config, USAGE, rest, true)
     })();
 
     match result {
@@ -146,7 +148,7 @@ each_subcommand!(declare_mod);
   because they are fundamental (and intertwined). Other commands can rely
   on this top-level information.
 */
-fn execute(flags: Flags, config: &Config) -> CliResult {
+fn execute(flags: Flags, config: &mut Config) -> CliResult {
     config.configure(flags.flag_verbose,
                    flags.flag_quiet,
                    &flags.flag_color,
@@ -225,11 +227,11 @@ fn execute(flags: Flags, config: &Config) -> CliResult {
         }
     };
 
-    if let Some(r) = try_execute_builtin_command(&config, &args) {
+    if let Some(r) = try_execute_builtin_command(config, &args) {
         return r;
     }
 
-    let alias_list = aliased_command(&config, &args[1])?;
+    let alias_list = aliased_command(config, &args[1])?;
     let args = match alias_list {
         Some(alias_command) => {
             let chain = args.iter()
@@ -238,7 +240,7 @@ fn execute(flags: Flags, config: &Config) -> CliResult {
                 .chain(args.iter().skip(2))
                 .map(|s| s.to_string())
                 .collect::<Vec<_>>();
-            if let Some(r) = try_execute_builtin_command(&config, &chain) {
+            if let Some(r) = try_execute_builtin_command(config, &chain) {
                 return r;
             } else {
                 chain
@@ -250,11 +252,12 @@ fn execute(flags: Flags, config: &Config) -> CliResult {
     execute_external_subcommand(config, &args[1], &args)
 }
 
-fn try_execute_builtin_command(config: &Config, args: &[String]) -> Option<CliResult> {
+fn try_execute_builtin_command(config: &mut Config, args: &[String]) -> Option<CliResult> {
     macro_rules! cmd {
         ($name:ident) => (if args[1] == stringify!($name).replace("_", "-") {
             config.shell().set_verbosity(Verbosity::Verbose);
-            let r = cargo::call_main_without_stdin($name::execute, config,
+            let r = cargo::call_main_without_stdin($name::execute,
+                                                   config,
                                                    $name::USAGE,
                                                    &args,
                                                    false);
@@ -266,7 +269,7 @@ fn try_execute_builtin_command(config: &Config, args: &[String]) -> Option<CliRe
     None
 }
 
-fn aliased_command(config: &Config, command: &String) -> CargoResult<Option<Vec<String>>> {
+fn aliased_command(config: &Config, command: &str) -> CargoResult<Option<Vec<String>>> {
     let alias_name = format!("alias.{}", command);
     let mut result = Ok(None);
     match config.get_string(&alias_name) {
@@ -298,7 +301,7 @@ fn find_closest(config: &Config, cmd: &str) -> Option<String> {
     // Only consider candidates with a lev_distance of 3 or less so we don't
     // suggest out-of-the-blue options.
     let mut filtered = cmds.iter()
-        .map(|c| (lev_distance(&c, cmd), c))
+        .map(|c| (lev_distance(c, cmd), c))
         .filter(|&(d, _)| d < 4)
         .collect::<Vec<_>>();
     filtered.sort_by(|a, b| a.0.cmp(&b.0));
@@ -314,15 +317,15 @@ fn execute_external_subcommand(config: &Config, cmd: &str, args: &[String]) -> C
     let command = match path {
         Some(command) => command,
         None => {
-            return Err(CargoError::from(match find_closest(config, cmd) {
-                    Some(closest) => {
-                        format!("no such subcommand: `{}`\n\n\tDid you mean `{}`?\n",
+            let err = match find_closest(config, cmd) {
+                Some(closest) => {
+                    format_err!("no such subcommand: `{}`\n\n\tDid you mean `{}`?\n",
                                 cmd,
                                 closest)
-                    }
-                    None => format!("no such subcommand: `{}`", cmd),
-                })
-                .into())
+                }
+                None => format_err!("no such subcommand: `{}`", cmd),
+            };
+            return Err(CliError::new(err, 101))
         }
     };
 
@@ -335,7 +338,7 @@ fn execute_external_subcommand(config: &Config, cmd: &str, args: &[String]) -> C
         Err(e) => e,
     };
 
-    if let &CargoErrorKind::ProcessErrorKind(ref perr) = err.kind() {
+    if let Some(perr) = err.downcast_ref::<ProcessError>() {
         if let Some(code) = perr.exit.as_ref().and_then(|c| c.code()) {
             return Err(CliError::code(code));
         }
@@ -397,10 +400,11 @@ fn search_directories(config: &Config) -> Vec<PathBuf> {
 }
 
 fn init_git_transports(config: &Config) {
-    // Only use a custom transport if a proxy is configured, right now libgit2
-    // doesn't support proxies and we have to use a custom transport in this
-    // case. The custom transport, however, is not as well battle-tested.
-    match cargo::ops::http_proxy_exists(config) {
+    // Only use a custom transport if any HTTP options are specified,
+    // such as proxies or custom certificate authorities. The custom
+    // transport, however, is not as well battle-tested.
+
+    match cargo::ops::needs_custom_http_transport(config) {
         Ok(true) => {}
         _ => return,
     }
