@@ -14,9 +14,10 @@ Options:
     -h, --help          Print this message
 ";
 
-pub fn execute(_: Options, _: &Config) -> CliResult {
+pub fn execute(_: Options, _: &mut Config) -> CliResult {
     // This is a dummy command just so that `cargo help help` works.
     // The actual delegation of help flag to subcommands is handled by the
     // cargo command.
-    Err(CliError::new("help command should not be executed directly".into(), 101))
+    Err(CliError::new(format_err!("help command should not be executed directly"),
+                      101))
 }
