@@ -9,7 +9,7 @@ use serde::ser;
 use url::Url;
 
 use core::{Dependency, PackageId, Summary, SourceId, PackageIdSpec};
-use core::{WorkspaceConfig, Features, Feature};
+use core::{WorkspaceConfig, Epoch, Features, Feature};
 use util::Config;
 use util::toml::TomlManifest;
 use util::errors::*;
@@ -36,6 +36,7 @@ pub struct Manifest {
     workspace: WorkspaceConfig,
     original: Rc<TomlManifest>,
     features: Features,
+    epoch: Epoch,
     im_a_teapot: Option<bool>,
 }
 
@@ -156,7 +157,7 @@ impl ser::Serialize for TargetKind {
 pub struct Profile {
     pub opt_level: String,
     #[serde(skip_serializing)]
-    pub lto: bool,
+    pub lto: Lto,
     #[serde(skip_serializing)]
     pub codegen_units: Option<u32>,    // None = use rustc default
     #[serde(skip_serializing)]
@@ -181,6 +182,12 @@ pub struct Profile {
     pub incremental: bool,
 }
 
+#[derive(Clone, PartialEq, Eq, Debug, Hash)]
+pub enum Lto {
+    Bool(bool),
+    Named(String),
+}
+
 #[derive(Default, Clone, Debug, PartialEq, Eq)]
 pub struct Profiles {
     pub release: Profile,
@@ -233,7 +240,7 @@ struct SerializedTarget<'a> {
     /// Serialized as a list of strings for historical reasons.
     kind: &'a TargetKind,
     /// Corresponds to `--crate-type` compiler attribute.
-    /// See https://doc.rust-lang.org/reference.html#linkage
+    /// See https://doc.rust-lang.org/reference/linkage.html
     crate_types: Vec<&'a str>,
     name: &'a str,
     src_path: &'a PathBuf,
@@ -263,6 +270,7 @@ impl Manifest {
                patch: HashMap<Url, Vec<Dependency>>,
                workspace: WorkspaceConfig,
                features: Features,
+               epoch: Epoch,
                im_a_teapot: Option<bool>,
                original: Rc<TomlManifest>) -> Manifest {
         Manifest {
@@ -279,6 +287,7 @@ impl Manifest {
             patch: patch,
             workspace: workspace,
             features: features,
+            epoch: epoch,
             original: original,
             im_a_teapot: im_a_teapot,
         }
@@ -350,6 +359,10 @@ impl Manifest {
             }
         }
     }
+
+    pub fn epoch(&self) -> Epoch {
+        self.epoch
+    }
 }
 
 impl VirtualManifest {
@@ -702,7 +715,7 @@ impl Default for Profile {
     fn default() -> Profile {
         Profile {
             opt_level: "0".to_string(),
-            lto: false,
+            lto: Lto::Bool(false),
             codegen_units: None,
             rustc_args: None,
             rustdoc_args: None,
