@@ -5,7 +5,8 @@ use semver::Version;
 use url::Url;
 
 use core::PackageId;
-use util::{CargoResult, ToUrl, human, ToSemver, ChainError};
+use util::{ToUrl, ToSemver};
+use util::errors::{CargoResult, CargoResultExt};
 
 #[derive(Clone, PartialEq, Eq, Debug)]
 pub struct PackageIdSpec {
@@ -16,7 +17,7 @@ pub struct PackageIdSpec {
 
 impl PackageIdSpec {
     pub fn parse(spec: &str) -> CargoResult<PackageIdSpec> {
-        if spec.contains("/") {
+        if spec.contains('/') {
             if let Ok(url) = spec.to_url() {
                 return PackageIdSpec::from_url(url);
             }
@@ -29,7 +30,7 @@ impl PackageIdSpec {
         let mut parts = spec.splitn(2, ':');
         let name = parts.next().unwrap();
         let version = match parts.next() {
-            Some(version) => Some(Version::parse(version).map_err(human)?),
+            Some(version) => Some(Version::parse(version)?),
             None => None,
         };
         for ch in name.chars() {
@@ -47,8 +48,8 @@ impl PackageIdSpec {
     pub fn query_str<'a, I>(spec: &str, i: I) -> CargoResult<&'a PackageId>
         where I: IntoIterator<Item=&'a PackageId>
     {
-        let spec = PackageIdSpec::parse(spec).chain_error(|| {
-            human(format!("invalid package id specification: `{}`", spec))
+        let spec = PackageIdSpec::parse(spec).chain_err(|| {
+            format_err!("invalid package id specification: `{}`", spec)
         })?;
         spec.query(i)
     }
@@ -68,12 +69,12 @@ impl PackageIdSpec {
         let frag = url.fragment().map(|s| s.to_owned());
         url.set_fragment(None);
         let (name, version) = {
-            let mut path = url.path_segments().chain_error(|| {
-                human(format!("pkgid urls must have a path: {}", url))
+            let mut path = url.path_segments().ok_or_else(|| {
+                format_err!("pkgid urls must have a path: {}", url)
             })?;
-            let path_name = path.next_back().chain_error(|| {
-                human(format!("pkgid urls must have at least one path \
-                               component: {}", url))
+            let path_name = path.next_back().ok_or_else(|| {
+                format_err!("pkgid urls must have at least one path \
+                             component: {}", url)
             })?;
             match frag {
                 Some(fragment) => {
@@ -81,7 +82,7 @@ impl PackageIdSpec {
                     let name_or_version = parts.next().unwrap();
                     match parts.next() {
                         Some(part) => {
-                            let version = part.to_semver().map_err(human)?;
+                            let version = part.to_semver()?;
                             (name_or_version.to_string(), Some(version))
                         }
                         None => {
@@ -89,8 +90,7 @@ impl PackageIdSpec {
                                               .is_alphabetic() {
                                 (name_or_version.to_string(), None)
                             } else {
-                                let version = name_or_version.to_semver()
-                                                                  .map_err(human)?;
+                                let version = name_or_version.to_semver()?;
                                 (path_name.to_string(), Some(version))
                             }
                         }
@@ -117,9 +117,10 @@ impl PackageIdSpec {
     pub fn matches(&self, package_id: &PackageId) -> bool {
         if self.name() != package_id.name() { return false }
 
-        match self.version {
-            Some(ref v) => if v != package_id.version() { return false },
-            None => {}
+        if let Some(ref v) = self.version {
+            if v != package_id.version() {
+                return false;
+            }
         }
 
         match self.url {
@@ -148,20 +149,20 @@ impl PackageIdSpec {
                                       self.name(), self);
                 let mut vec = vec![ret, other];
                 vec.extend(ids);
-                minimize(&mut msg, vec, self);
-                Err(human(msg))
+                minimize(&mut msg, &vec, self);
+                Err(format_err!("{}", msg))
             }
             None => Ok(ret)
         };
 
         fn minimize(msg: &mut String,
-                    ids: Vec<&PackageId>,
+                    ids: &[&PackageId],
                     spec: &PackageIdSpec) {
             let mut version_cnt = HashMap::new();
-            for id in ids.iter() {
+            for id in ids {
                 *version_cnt.entry(id.version()).or_insert(0) += 1;
             }
-            for id in ids.iter() {
+            for id in ids {
                 if version_cnt[id.version()] == 1 {
                     msg.push_str(&format!("\n  {}:{}", spec.name(),
                                           id.version()));
@@ -184,18 +185,15 @@ impl fmt::Display for PackageIdSpec {
                 } else {
                     write!(f, "{}", url)?;
                 }
-                if url.path_segments().unwrap().next_back().unwrap() != &self.name {
+                if url.path_segments().unwrap().next_back().unwrap() != self.name {
                     printed_name = true;
                     write!(f, "#{}", self.name)?;
                 }
             }
             None => { printed_name = true; write!(f, "{}", self.name)? }
         }
-        match self.version {
-            Some(ref v) => {
-                write!(f, "{}{}", if printed_name {":"} else {"#"}, v)?;
-            }
-            None => {}
+        if let Some(ref v) = self.version {
+            write!(f, "{}{}", if printed_name {":"} else {"#"}, v)?;
         }
         Ok(())
     }
@@ -270,7 +268,7 @@ mod tests {
     #[test]
     fn matching() {
         let url = Url::parse("http://example.com").unwrap();
-        let sid = SourceId::for_registry(&url);
+        let sid = SourceId::for_registry(&url).unwrap();
         let foo = PackageId::new("foo", "1.2.3", &sid).unwrap();
         let bar = PackageId::new("bar", "1.2.3", &sid).unwrap();
 
