@@ -47,10 +47,11 @@
 
 use std::cmp::Ordering;
 use std::collections::{HashSet, HashMap, BinaryHeap, BTreeMap};
-use std::iter::FromIterator;
 use std::fmt;
+use std::iter::FromIterator;
 use std::ops::Range;
 use std::rc::Rc;
+use std::time::{Instant, Duration};
 
 use semver;
 use url::Url;
@@ -73,6 +74,7 @@ mod encode;
 ///
 /// Each instance of `Resolve` also understands the full set of features used
 /// for each package.
+#[derive(PartialEq)]
 pub struct Resolve {
     graph: Graph<PackageId>,
     replacements: HashMap<PackageId, PackageId>,
@@ -114,6 +116,11 @@ struct Candidate {
 }
 
 impl Resolve {
+    /// Resolves one of the paths from the given dependent package up to
+    /// the root.
+    pub fn path_to_top<'a>(&'a self, pkg: &'a PackageId) -> Vec<&'a PackageId> {
+        self.graph.path_to_top(pkg)
+    }
     pub fn register_used_patches(&mut self,
                                  patches: &HashMap<Url, Vec<Summary>>) {
         for summary in patches.values().flat_map(|v| v) {
@@ -342,7 +349,8 @@ type Activations = HashMap<String, HashMap<SourceId, Vec<Summary>>>;
 pub fn resolve(summaries: &[(Summary, Method)],
                replacements: &[(PackageIdSpec, Dependency)],
                registry: &mut Registry,
-               config: Option<&Config>) -> CargoResult<Resolve> {
+               config: Option<&Config>,
+               print_warnings: bool) -> CargoResult<Resolve> {
     let cx = Context {
         resolve_graph: RcList::new(),
         resolve_features: HashMap::new(),
@@ -351,8 +359,8 @@ pub fn resolve(summaries: &[(Summary, Method)],
         replacements: replacements,
         warnings: RcList::new(),
     };
-    let _p = profile::start(format!("resolving"));
-    let cx = activate_deps_loop(cx, registry, summaries)?;
+    let _p = profile::start("resolving");
+    let cx = activate_deps_loop(cx, registry, summaries, config)?;
 
     let mut resolve = Resolve {
         graph: cx.graph(),
@@ -378,11 +386,13 @@ pub fn resolve(summaries: &[(Summary, Method)],
 
     // If we have a shell, emit warnings about required deps used as feature.
     if let Some(config) = config {
-        let mut shell = config.shell();
-        let mut warnings = &cx.warnings;
-        while let Some(ref head) = warnings.head {
-            shell.warn(&head.0)?;
-            warnings = &head.1;
+        if print_warnings {
+            let mut shell = config.shell();
+            let mut warnings = &cx.warnings;
+            while let Some(ref head) = warnings.head {
+                shell.warn(&head.0)?;
+                warnings = &head.1;
+            }
         }
     }
 
@@ -400,7 +410,7 @@ fn activate(cx: &mut Context,
             parent: Option<&Summary>,
             candidate: Candidate,
             method: &Method)
-            -> CargoResult<Option<DepsFrame>> {
+            -> CargoResult<Option<(DepsFrame, Duration)>> {
     if let Some(parent) = parent {
         cx.resolve_graph.push(GraphNode::Link(parent.package_id().clone(),
                                            candidate.summary.package_id().clone()));
@@ -428,12 +438,13 @@ fn activate(cx: &mut Context,
         }
     };
 
+    let now = Instant::now();
     let deps = cx.build_deps(registry, &candidate, method)?;
-
-    Ok(Some(DepsFrame {
+    let frame = DepsFrame {
         parent: candidate,
         remaining_siblings: RcVecIter::new(Rc::new(deps)),
-    }))
+    };
+    Ok(Some((frame, now.elapsed())))
 }
 
 struct RcVecIter<T> {
@@ -448,10 +459,6 @@ impl<T> RcVecIter<T> {
             vec: vec,
         }
     }
-
-    fn cur_index(&self) -> usize {
-        self.rest.start - 1
-    }
 }
 
 // Not derived to avoid `T: Clone`
@@ -522,6 +529,7 @@ impl Ord for DepsFrame {
 }
 
 struct BacktrackFrame<'a> {
+    cur: usize,
     context_backup: Context<'a>,
     deps_backup: BinaryHeap<DepsFrame>,
     remaining_candidates: RemainingCandidates,
@@ -544,12 +552,12 @@ impl RemainingCandidates {
         // define "compatible" here in terms of the semver sense where if
         // the left-most nonzero digit is the same they're considered
         // compatible.
-        self.remaining.by_ref().map(|p| p.1).filter(|b| {
+        self.remaining.by_ref().map(|p| p.1).find(|b| {
             prev_active.iter().any(|a| *a == b.summary) ||
                 prev_active.iter().all(|a| {
                     !compatible(a.version(), b.summary.version())
                 })
-        }).next()
+        })
     }
 }
 
@@ -560,7 +568,8 @@ impl RemainingCandidates {
 /// dependency graph, cx.resolve is returned.
 fn activate_deps_loop<'a>(mut cx: Context<'a>,
                           registry: &mut Registry,
-                          summaries: &[(Summary, Method)])
+                          summaries: &[(Summary, Method)],
+                          config: Option<&Config>)
                           -> CargoResult<Context<'a>> {
     // Note that a `BinaryHeap` is used for the remaining dependencies that need
     // activation. This heap is sorted such that the "largest value" is the most
@@ -574,16 +583,24 @@ fn activate_deps_loop<'a>(mut cx: Context<'a>,
     for &(ref summary, ref method) in summaries {
         debug!("initial activation: {}", summary.package_id());
         let candidate = Candidate { summary: summary.clone(), replace: None };
-        remaining_deps.extend(activate(&mut cx, registry, None, candidate,
-                                       method)?);
+        let res = activate(&mut cx, registry, None, candidate, method)?;
+        if let Some((frame, _)) = res {
+            remaining_deps.push(frame);
+        }
     }
 
+    let mut ticks = 0;
+    let start = Instant::now();
+    let time_to_print = Duration::from_millis(500);
+    let mut printed = false;
+    let mut deps_time = Duration::new(0, 0);
+
     // Main resolution loop, this is the workhorse of the resolution algorithm.
     //
     // You'll note that a few stacks are maintained on the side, which might
     // seem odd when this algorithm looks like it could be implemented
     // recursively. While correct, this is implemented iteratively to avoid
-    // blowing the stack (the recusion depth is proportional to the size of the
+    // blowing the stack (the recursion depth is proportional to the size of the
     // input).
     //
     // The general sketch of this loop is to run until there are no dependencies
@@ -592,6 +609,28 @@ fn activate_deps_loop<'a>(mut cx: Context<'a>,
     // backtracking states where if we hit an error we can return to in order to
     // attempt to continue resolving.
     while let Some(mut deps_frame) = remaining_deps.pop() {
+
+        // If we spend a lot of time here (we shouldn't in most cases) then give
+        // a bit of a visual indicator as to what we're doing. Only enable this
+        // when stderr is a tty (a human is likely to be watching) to ensure we
+        // get deterministic output otherwise when observed by tools.
+        //
+        // Also note that we hit this loop a lot, so it's fairly performance
+        // sensitive. As a result try to defer a possibly expensive operation
+        // like `Instant::now` by only checking every N iterations of this loop
+        // to amortize the cost of the current time lookup.
+        ticks += 1;
+        if let Some(config) = config {
+            if config.shell().is_err_tty() &&
+                !printed &&
+                ticks % 1000 == 0 &&
+                start.elapsed() - deps_time > time_to_print
+            {
+                printed = true;
+                config.shell().status("Resolving", "dependency graph...")?;
+            }
+        }
+
         let frame = match deps_frame.remaining_siblings.next() {
             Some(sibling) => {
                 let parent = Summary::clone(&deps_frame.parent);
@@ -635,6 +674,7 @@ fn activate_deps_loop<'a>(mut cx: Context<'a>,
                 // we can try the next one if this one fails.
                 if has_another {
                     backtrack_stack.push(BacktrackFrame {
+                        cur,
                         context_backup: Context::clone(&cx),
                         deps_backup: <BinaryHeap<DepsFrame>>::clone(&remaining_deps),
                         remaining_candidates: remaining_candidates,
@@ -661,8 +701,8 @@ fn activate_deps_loop<'a>(mut cx: Context<'a>,
                                      &mut features) {
                     None => return Err(activation_error(&cx, registry, &parent,
                                                         &dep,
-                                                        &cx.prev_active(&dep),
-                                                        &candidates)),
+                                                        cx.prev_active(&dep),
+                                                        &candidates, config)),
                     Some(candidate) => candidate,
                 }
             }
@@ -675,16 +715,26 @@ fn activate_deps_loop<'a>(mut cx: Context<'a>,
         };
         trace!("{}[{}]>{} trying {}", parent.name(), cur, dep.name(),
                candidate.summary.version());
-        remaining_deps.extend(activate(&mut cx, registry, Some(&parent),
-                              candidate, &method)?);
+        let res = activate(&mut cx, registry, Some(&parent), candidate, &method)?;
+        if let Some((frame, dur)) = res {
+            remaining_deps.push(frame);
+            deps_time += dur;
+        }
     }
 
     Ok(cx)
 }
 
-// Searches up `backtrack_stack` until it finds a dependency with remaining
-// candidates. Resets `cx` and `remaining_deps` to that level and returns the
+// Looks through the states in `backtrack_stack` for dependencies with
+// remaining candidates. For each one, also checks if rolling back
+// could change the outcome of the failed resolution that caused backtracking
+// in the first place - namely, if we've backtracked past the parent of the
+// failed dep, or the previous number of activations of the failed dep has
+// changed (possibly relaxing version constraints). If the outcome could differ,
+// resets `cx` and `remaining_deps` to that level and returns the
 // next candidate. If all candidates have been exhausted, returns None.
+// Read https://github.com/rust-lang/cargo/pull/4834#issuecomment-362871537 for
+// a more detailed explanation of the logic here.
 fn find_candidate<'a>(backtrack_stack: &mut Vec<BacktrackFrame<'a>>,
                       cx: &mut Context<'a>,
                       remaining_deps: &mut BinaryHeap<DepsFrame>,
@@ -692,19 +742,29 @@ fn find_candidate<'a>(backtrack_stack: &mut Vec<BacktrackFrame<'a>>,
                       cur: &mut usize,
                       dep: &mut Dependency,
                       features: &mut Rc<Vec<String>>) -> Option<Candidate> {
+    let num_dep_prev_active = cx.prev_active(dep).len();
     while let Some(mut frame) = backtrack_stack.pop() {
         let (next, has_another) = {
             let prev_active = frame.context_backup.prev_active(&frame.dep);
             (frame.remaining_candidates.next(prev_active),
              frame.remaining_candidates.clone().next(prev_active).is_some())
         };
+        let cur_num_dep_prev_active = frame.context_backup.prev_active(dep).len();
+        // Activations should monotonically decrease during backtracking
+        assert!(cur_num_dep_prev_active <= num_dep_prev_active);
+        let maychange = !frame.context_backup.is_active(parent) ||
+            cur_num_dep_prev_active != num_dep_prev_active;
+        if !maychange {
+            continue
+        }
         if let Some(candidate) = next {
+            *cur = frame.cur;
             if has_another {
                 *cx = frame.context_backup.clone();
                 *remaining_deps = frame.deps_backup.clone();
                 *parent = frame.parent.clone();
                 *dep = frame.dep.clone();
-                *features = frame.features.clone();
+                *features = Rc::clone(&frame.features);
                 backtrack_stack.push(frame);
             } else {
                 *cx = frame.context_backup;
@@ -713,7 +773,6 @@ fn find_candidate<'a>(backtrack_stack: &mut Vec<BacktrackFrame<'a>>,
                 *dep = frame.dep;
                 *features = frame.features;
             }
-            *cur = remaining_deps.peek().unwrap().remaining_siblings.cur_index();
             return Some(candidate)
         }
     }
@@ -725,31 +784,30 @@ fn activation_error(cx: &Context,
                     parent: &Summary,
                     dep: &Dependency,
                     prev_active: &[Summary],
-                    candidates: &[Candidate]) -> CargoError {
-    if candidates.len() > 0 {
-        let mut msg = format!("failed to select a version for `{}` \
-                               (required by `{}`):\n\
+                    candidates: &[Candidate],
+                    config: Option<&Config>) -> CargoError {
+    let graph = cx.graph();
+    let describe_path = |pkgid: &PackageId| -> String {
+        use std::fmt::Write;
+        let dep_path = graph.path_to_top(pkgid);
+        let mut dep_path_desc = format!("package `{}`", dep_path[0]);
+        for dep in dep_path.iter().skip(1) {
+            write!(dep_path_desc,
+                   "\n    ... which is depended on by `{}`",
+                   dep).unwrap();
+        }
+        dep_path_desc
+    };
+    if !candidates.is_empty() {
+        let mut msg = format!("failed to select a version for `{0}`\n\
                                all possible versions conflict with \
-                               previously selected versions of `{}`",
-                              dep.name(), parent.name(),
+                               previously selected versions of `{0}`\n",
                               dep.name());
-        let graph = cx.graph();
-        'outer: for v in prev_active.iter() {
-            for node in graph.iter() {
-                let edges = match graph.edges(node) {
-                    Some(edges) => edges,
-                    None => continue,
-                };
-                for edge in edges {
-                    if edge != v.package_id() { continue }
-
-                    msg.push_str(&format!("\n  version {} in use by {}",
-                                          v.version(), edge));
-                    continue 'outer;
-                }
-            }
-            msg.push_str(&format!("\n  version {} in use by ??",
-                                  v.version()));
+        msg.push_str("required by ");
+        msg.push_str(&describe_path(parent.package_id()));
+        for v in prev_active.iter() {
+            msg.push_str("\n  previously selected ");
+            msg.push_str(&describe_path(v.package_id()));
         }
 
         msg.push_str(&format!("\n  possible versions to select: {}",
@@ -759,7 +817,7 @@ fn activation_error(cx: &Context,
                                         .collect::<Vec<_>>()
                                         .join(", ")));
 
-        return msg.into()
+        return format_err!("{}", msg)
     }
 
     // Once we're all the way down here, we're definitely lost in the
@@ -780,7 +838,7 @@ fn activation_error(cx: &Context,
         b.version().cmp(a.version())
     });
 
-    let msg = if !candidates.is_empty() {
+    let mut msg = if !candidates.is_empty() {
         let versions = {
             let mut versions = candidates.iter().take(3).map(|cand| {
                 cand.version().to_string()
@@ -793,37 +851,46 @@ fn activation_error(cx: &Context,
             versions.join(", ")
         };
 
-        let mut msg = format!("no matching version `{}` found for package `{}` \
-                               (required by `{}`)\n\
+        let mut msg = format!("no matching version `{}` found for package `{}`\n\
                                location searched: {}\n\
-                               versions found: {}",
+                               versions found: {}\n",
                               dep.version_req(),
                               dep.name(),
-                              parent.name(),
                               dep.source_id(),
                               versions);
+        msg.push_str("required by ");
+        msg.push_str(&describe_path(parent.package_id()));
 
         // If we have a path dependency with a locked version, then this may
         // indicate that we updated a sub-package and forgot to run `cargo
         // update`. In this case try to print a helpful error!
         if dep.source_id().is_path()
-           && dep.version_req().to_string().starts_with("=") {
+           && dep.version_req().to_string().starts_with('=') {
             msg.push_str("\nconsider running `cargo update` to update \
                           a path dependency's locked version");
         }
 
         msg
     } else {
-        format!("no matching package named `{}` found \
-                 (required by `{}`)\n\
-                 location searched: {}\n\
-                 version required: {}",
-                dep.name(), parent.name(),
-                dep.source_id(),
-                dep.version_req())
+        let mut msg = format!("no matching package named `{}` found\n\
+                 location searched: {}\n",
+                dep.name(), dep.source_id());
+        msg.push_str("required by ");
+        msg.push_str(&describe_path(parent.package_id()));
+
+        msg
     };
 
-    msg.into()
+    if let Some(config) = config {
+        if config.cli_unstable().offline {
+            msg.push_str("\nAs a reminder, you're using offline mode (-Z offline) \
+            which can sometimes cause surprising resolution failures, \
+            if this error is too confusing you may with to retry \
+            without the offline flag.");
+        }
+    }
+
+    format_err!("{}", msg)
 }
 
 // Returns if `a` and `b` are compatible in the semver sense. This is a
@@ -839,55 +906,68 @@ fn compatible(a: &semver::Version, b: &semver::Version) -> bool {
     a.patch == b.patch
 }
 
-// Returns a pair of (feature dependencies, all used features)
-//
-// The feature dependencies map is a mapping of package name to list of features
-// enabled. Each package should be enabled, and each package should have the
-// specified set of features enabled.  The boolean indicates whether this
-// package was specifically requested (rather than just requesting features
-// *within* this package).
-//
-// The all used features set is the set of features which this local package had
-// enabled, which is later used when compiling to instruct the code what
-// features were enabled.
-fn build_features<'a>(s: &'a Summary, method: &'a Method)
-                      -> CargoResult<(HashMap<&'a str, (bool, Vec<String>)>, HashSet<&'a str>)> {
-    let mut deps = HashMap::new();
-    let mut used = HashSet::new();
-    let mut visited = HashSet::new();
-    match *method {
-        Method::Everything => {
-            for key in s.features().keys() {
-                add_feature(s, key, &mut deps, &mut used, &mut visited)?;
-            }
-            for dep in s.dependencies().iter().filter(|d| d.is_optional()) {
-                add_feature(s, dep.name(), &mut deps, &mut used,
-                            &mut visited)?;
-            }
-        }
-        Method::Required { features: requested_features, .. } =>  {
-            for feat in requested_features.iter() {
-                add_feature(s, feat, &mut deps, &mut used, &mut visited)?;
-            }
-        }
-    }
-    match *method {
-        Method::Everything |
-        Method::Required { uses_default_features: true, .. } => {
-            if s.features().get("default").is_some() {
-                add_feature(s, "default", &mut deps, &mut used,
-                            &mut visited)?;
-            }
-        }
-        Method::Required { uses_default_features: false, .. } => {}
-    }
-    return Ok((deps, used));
+struct Requirements<'a> {
+    summary: &'a Summary,
+    // The deps map is a mapping of package name to list of features enabled.
+    // Each package should be enabled, and each package should have the
+    // specified set of features enabled. The boolean indicates whether this
+    // package was specifically requested (rather than just requesting features
+    // *within* this package).
+    deps: HashMap<&'a str, (bool, Vec<String>)>,
+    // The used features set is the set of features which this local package had
+    // enabled, which is later used when compiling to instruct the code what
+    // features were enabled.
+    used: HashSet<&'a str>,
+    visited: HashSet<&'a str>,
+}
 
-    fn add_feature<'a>(s: &'a Summary,
-                       feat: &'a str,
-                       deps: &mut HashMap<&'a str, (bool, Vec<String>)>,
-                       used: &mut HashSet<&'a str>,
-                       visited: &mut HashSet<&'a str>) -> CargoResult<()> {
+impl<'r> Requirements<'r> {
+    fn new<'a>(summary: &'a Summary) -> Requirements<'a> {
+        Requirements {
+            summary,
+            deps: HashMap::new(),
+            used: HashSet::new(),
+            visited: HashSet::new(),
+        }
+    }
+
+    fn require_crate_feature(&mut self, package: &'r str, feat: &'r str) {
+        self.used.insert(package);
+        self.deps.entry(package)
+            .or_insert((false, Vec::new()))
+            .1.push(feat.to_string());
+    }
+
+    fn seen(&mut self, feat: &'r str) -> bool {
+        if self.visited.insert(feat) {
+            self.used.insert(feat);
+            false
+        } else {
+            true
+        }
+    }
+
+    fn require_dependency(&mut self, pkg: &'r str) {
+        if self.seen(pkg) {
+            return;
+        }
+        self.deps.entry(pkg).or_insert((false, Vec::new())).0 = true;
+    }
+
+    fn require_feature(&mut self, feat: &'r str) -> CargoResult<()> {
+        if self.seen(feat) {
+            return Ok(());
+        }
+        for f in self.summary.features().get(feat).expect("must be a valid feature") {
+            if f == &feat {
+                bail!("Cyclic feature dependency: feature `{}` depends on itself", feat);
+            }
+            self.add_feature(f)?;
+        }
+        Ok(())
+    }
+
+    fn add_feature(&mut self, feat: &'r str) -> CargoResult<()> {
         if feat.is_empty() { return Ok(()) }
 
         // If this feature is of the form `foo/bar`, then we just lookup package
@@ -899,38 +979,53 @@ fn build_features<'a>(s: &'a Summary, method: &'a Method)
         let feat_or_package = parts.next().unwrap();
         match parts.next() {
             Some(feat) => {
-                let package = feat_or_package;
-                used.insert(package);
-                deps.entry(package)
-                    .or_insert((false, Vec::new()))
-                    .1.push(feat.to_string());
+                self.require_crate_feature(feat_or_package, feat);
             }
             None => {
-                let feat = feat_or_package;
-                if !visited.insert(feat) {
-                    bail!("Cyclic feature dependency: feature `{}` depends \
-                           on itself", feat)
+                if self.summary.features().contains_key(feat_or_package) {
+                    self.require_feature(feat_or_package)?;
+                } else {
+                    self.require_dependency(feat_or_package);
                 }
-                used.insert(feat);
-                match s.features().get(feat) {
-                    Some(recursive) => {
-                        // This is a feature, add it recursively.
-                        for f in recursive {
-                            add_feature(s, f, deps, used, visited)?;
-                        }
-                    }
-                    None => {
-                        // This is a dependency, mark it as explicitly requested.
-                        deps.entry(feat).or_insert((false, Vec::new())).0 = true;
-                    }
-                }
-                visited.remove(feat);
             }
         }
         Ok(())
     }
 }
 
+// Takes requested features for a single package from the input Method and
+// recurses to find all requested features, dependencies and requested
+// dependency features in a Requirements object, returning it to the resolver.
+fn build_requirements<'a, 'b: 'a>(s: &'a Summary, method: &'b Method)
+                                  -> CargoResult<Requirements<'a>> {
+    let mut reqs = Requirements::new(s);
+    match *method {
+        Method::Everything => {
+            for key in s.features().keys() {
+                reqs.require_feature(key)?;
+            }
+            for dep in s.dependencies().iter().filter(|d| d.is_optional()) {
+                reqs.require_dependency(dep.name());
+            }
+        }
+        Method::Required { features: requested_features, .. } =>  {
+            for feat in requested_features.iter() {
+                reqs.add_feature(feat)?;
+            }
+        }
+    }
+    match *method {
+        Method::Everything |
+        Method::Required { uses_default_features: true, .. } => {
+            if s.features().get("default").is_some() {
+                reqs.require_feature("default")?;
+            }
+        }
+        Method::Required { uses_default_features: false, .. } => {}
+    }
+    return Ok(reqs);
+}
+
 impl<'a> Context<'a> {
     // Activate this summary by inserting it into our list of known activations.
     //
@@ -972,7 +1067,7 @@ impl<'a> Context<'a> {
                   registry: &mut Registry,
                   candidate: &Summary,
                   method: &Method) -> CargoResult<Vec<DepInfo>> {
-        // First, figure out our set of dependencies based on the requsted set
+        // First, figure out our set of dependencies based on the requested set
         // of features. This also calculates what features we're going to enable
         // for our own dependencies.
         let deps = self.resolve_features(candidate, method)?;
@@ -1025,13 +1120,13 @@ impl<'a> Context<'a> {
 
             let mut summaries = registry.query_vec(dep)?.into_iter();
             let s = summaries.next().ok_or_else(|| {
-                format!("no matching package for override `{}` found\n\
-                         location searched: {}\n\
-                         version required: {}",
-                         spec, dep.source_id(), dep.version_req())
+                format_err!("no matching package for override `{}` found\n\
+                             location searched: {}\n\
+                             version required: {}",
+                            spec, dep.source_id(), dep.version_req())
             })?;
             let summaries = summaries.collect::<Vec<_>>();
-            if summaries.len() > 0 {
+            if !summaries.is_empty() {
                 let bullets = summaries.iter().map(|s| {
                     format!("  * {}", s.package_id())
                 }).collect::<Vec<_>>();
@@ -1077,6 +1172,14 @@ impl<'a> Context<'a> {
             .unwrap_or(&[])
     }
 
+    fn is_active(&mut self, summary: &Summary) -> bool {
+        let id = summary.package_id();
+        self.activations.get(id.name())
+            .and_then(|v| v.get(id.source_id()))
+            .map(|v| v.iter().any(|s| s == summary))
+            .unwrap_or(false)
+    }
+
     /// Return all dependencies and the features we want from them.
     fn resolve_features<'b>(&mut self,
                             s: &'b Summary,
@@ -1091,18 +1194,18 @@ impl<'a> Context<'a> {
         let deps = s.dependencies();
         let deps = deps.iter().filter(|d| d.is_transitive() || dev_deps);
 
-        let (mut feature_deps, used_features) = build_features(s, method)?;
+        let mut reqs = build_requirements(s, method)?;
         let mut ret = Vec::new();
 
         // Next, collect all actually enabled dependencies and their features.
         for dep in deps {
             // Skip optional dependencies, but not those enabled through a feature
-            if dep.is_optional() && !feature_deps.contains_key(dep.name()) {
+            if dep.is_optional() && !reqs.deps.contains_key(dep.name()) {
                 continue
             }
             // So we want this dependency.  Move the features we want from `feature_deps`
             // to `ret`.
-            let base = feature_deps.remove(dep.name()).unwrap_or((false, vec![]));
+            let base = reqs.deps.remove(dep.name()).unwrap_or((false, vec![]));
             if !dep.is_optional() && base.0 {
                 self.warnings.push(
                     format!("Package `{}` does not have feature `{}`. It has a required dependency \
@@ -1125,21 +1228,22 @@ impl<'a> Context<'a> {
         // Any remaining entries in feature_deps are bugs in that the package does not actually
         // have those dependencies.  We classified them as dependencies in the first place
         // because there is no such feature, either.
-        if !feature_deps.is_empty() {
-            let unknown = feature_deps.keys().map(|s| &s[..])
-                                      .collect::<Vec<&str>>();
+        if !reqs.deps.is_empty() {
+            let unknown = reqs.deps.keys()
+                                   .map(|s| &s[..])
+                                   .collect::<Vec<&str>>();
             let features = unknown.join(", ");
             bail!("Package `{}` does not have these features: `{}`",
                     s.package_id(), features)
         }
 
         // Record what list of features is active for this package.
-        if !used_features.is_empty() {
+        if !reqs.used.is_empty() {
             let pkgid = s.package_id();
 
             let set = self.resolve_features.entry(pkgid.clone())
                               .or_insert_with(HashSet::new);
-            for feature in used_features {
+            for feature in reqs.used {
                 if !set.contains(feature) {
                     set.insert(feature.to_string());
                 }
@@ -1157,7 +1261,7 @@ impl<'a> Context<'a> {
             replacements.insert(k, v);
             cur = &node.1;
         }
-        return replacements
+        replacements
     }
 
     fn graph(&self) -> Graph<PackageId> {
@@ -1170,7 +1274,7 @@ impl<'a> Context<'a> {
             }
             cur = &node.1;
         }
-        return graph
+        graph
     }
 }
 
