@@ -3,7 +3,8 @@ use std::fs;
 use std::path::Path;
 
 use core::{Profiles, Workspace};
-use util::{CargoResult, human, ChainError, Config};
+use util::Config;
+use util::errors::{CargoResult, CargoResultExt};
 use ops::{self, Context, BuildConfig, Kind, Unit};
 
 pub struct CleanOptions<'a> {
@@ -33,9 +34,10 @@ pub fn clean(ws: &Workspace, opts: &CleanOptions) -> CargoResult<()> {
     let host_triple = opts.config.rustc()?.host.clone();
     let mut cx = Context::new(ws, &resolve, &packages, opts.config,
                                    BuildConfig {
-                                       host_triple: host_triple,
+                                       host_triple,
                                        requested_target: opts.target.map(|s| s.to_owned()),
                                        release: opts.release,
+                                       jobs: 1,
                                        ..BuildConfig::default()
                                    },
                                    profiles)?;
@@ -44,7 +46,7 @@ pub fn clean(ws: &Workspace, opts: &CleanOptions) -> CargoResult<()> {
     for spec in opts.spec {
         // Translate the spec to a Package
         let pkgid = resolve.query(spec)?;
-        let pkg = packages.get(&pkgid)?;
+        let pkg = packages.get(pkgid)?;
 
         // Generate all relevant `Unit` targets for this package
         for target in pkg.targets() {
@@ -52,15 +54,15 @@ pub fn clean(ws: &Workspace, opts: &CleanOptions) -> CargoResult<()> {
                 let Profiles {
                     ref release, ref dev, ref test, ref bench, ref doc,
                     ref custom_build, ref test_deps, ref bench_deps, ref check,
-                    ref doctest,
+                    ref check_test, ref doctest,
                 } = *profiles;
                 let profiles = [release, dev, test, bench, doc, custom_build,
-                                test_deps, bench_deps, check, doctest];
+                                test_deps, bench_deps, check, check_test, doctest];
                 for profile in profiles.iter() {
                     units.push(Unit {
-                        pkg: &pkg,
-                        target: target,
-                        profile: profile,
+                        pkg,
+                        target,
+                        profile,
                         kind: *kind,
                     });
                 }
@@ -81,10 +83,10 @@ pub fn clean(ws: &Workspace, opts: &CleanOptions) -> CargoResult<()> {
             continue
         }
 
-        for (src, link_dst, _) in cx.target_filenames(unit)? {
-            rm_rf(&src)?;
-            if let Some(dst) = link_dst {
-                rm_rf(&dst)?;
+        for &(ref src, ref link_dst, _) in cx.target_filenames(unit)?.iter() {
+            rm_rf(src)?;
+            if let Some(ref dst) = *link_dst {
+                rm_rf(dst)?;
             }
         }
     }
@@ -95,12 +97,12 @@ pub fn clean(ws: &Workspace, opts: &CleanOptions) -> CargoResult<()> {
 fn rm_rf(path: &Path) -> CargoResult<()> {
     let m = fs::metadata(path);
     if m.as_ref().map(|s| s.is_dir()).unwrap_or(false) {
-        fs::remove_dir_all(path).chain_error(|| {
-            human("could not remove build directory")
+        fs::remove_dir_all(path).chain_err(|| {
+            format_err!("could not remove build directory")
         })?;
     } else if m.is_ok() {
-        fs::remove_file(path).chain_error(|| {
-            human("failed to remove build artifact")
+        fs::remove_file(path).chain_err(|| {
+            format_err!("failed to remove build artifact")
         })?;
     }
     Ok(())
