@@ -1,9 +1,10 @@
 use std::collections::{BTreeMap, HashSet};
-use std::path::Path;
+
+use termcolor::Color::{self, Cyan, Green, Red};
 
 use core::PackageId;
 use core::registry::PackageRegistry;
-use core::{Resolve, SourceId, Package};
+use core::{Resolve, SourceId, Workspace};
 use core::resolver::Method;
 use ops;
 use util::config::Config;
@@ -16,33 +17,40 @@ pub struct UpdateOptions<'a> {
     pub aggressive: bool,
 }
 
-pub fn generate_lockfile(manifest_path: &Path, config: &Config)
-                         -> CargoResult<()> {
-    let package = try!(Package::for_path(manifest_path, config));
-    let mut registry = PackageRegistry::new(config);
-    let resolve = try!(ops::resolve_with_previous(&mut registry, &package,
-                                                  Method::Everything,
-                                                  None, None));
-    try!(ops::write_pkg_lockfile(&package, &resolve, config));
+pub fn generate_lockfile(ws: &Workspace) -> CargoResult<()> {
+    let mut registry = PackageRegistry::new(ws.config())?;
+    let resolve = ops::resolve_with_previous(&mut registry,
+                                             ws,
+                                             Method::Everything,
+                                             None,
+                                             None,
+                                             &[],
+                                             true,
+                                             true)?;
+    ops::write_pkg_lockfile(ws, &resolve)?;
     Ok(())
 }
 
-pub fn update_lockfile(manifest_path: &Path,
-                       opts: &UpdateOptions) -> CargoResult<()> {
+pub fn update_lockfile(ws: &Workspace, opts: &UpdateOptions)
+                       -> CargoResult<()> {
 
     if opts.aggressive && opts.precise.is_some() {
         bail!("cannot specify both aggressive and precise simultaneously")
     }
 
-    let package = try!(Package::for_path(manifest_path, opts.config));
+    if ws.members().is_empty() {
+        bail!("you can't generate a lockfile for an empty workspace.")
+    }
 
-    let previous_resolve = match try!(ops::load_pkg_lockfile(&package, opts.config)) {
-    	Some(resolve) => resolve,
-	None => {
-	     return generate_lockfile(manifest_path, opts.config);
-	}
+    if opts.config.cli_unstable().offline {
+        bail!("you can't update in the offline mode");
+    }
+
+    let previous_resolve = match ops::load_pkg_lockfile(ws)? {
+        Some(resolve) => resolve,
+        None => return generate_lockfile(ws),
     };
-    let mut registry = PackageRegistry::new(opts.config);
+    let mut registry = PackageRegistry::new(opts.config)?;
     let mut to_avoid = HashSet::new();
 
     if opts.to_update.is_empty() {
@@ -50,7 +58,7 @@ pub fn update_lockfile(manifest_path: &Path,
     } else {
         let mut sources = Vec::new();
         for name in opts.to_update {
-            let dep = try!(previous_resolve.query(name));
+            let dep = previous_resolve.query(name)?;
             if opts.aggressive {
                 fill_with_deps(&previous_resolve, dep, &mut to_avoid,
                                &mut HashSet::new());
@@ -74,18 +82,21 @@ pub fn update_lockfile(manifest_path: &Path,
                 });
             }
         }
-        try!(registry.add_sources(&sources));
+        registry.add_sources(&sources)?;
     }
 
-    let resolve = try!(ops::resolve_with_previous(&mut registry,
-                                                  &package,
-                                                  Method::Everything,
-                                                  Some(&previous_resolve),
-                                                  Some(&to_avoid)));
+    let resolve = ops::resolve_with_previous(&mut registry,
+                                             ws,
+                                             Method::Everything,
+                                             Some(&previous_resolve),
+                                             Some(&to_avoid),
+                                             &[],
+                                             true,
+                                             true)?;
 
     // Summarize what is changing for the user.
-    let print_change = |status: &str, msg: String| {
-        opts.config.shell().status(status, msg)
+    let print_change = |status: &str, msg: String, color: Color| {
+        opts.config.shell().status_with_color(status, msg, color)
     };
     for (removed, added) in compare_dependency_graphs(&previous_resolve, &resolve) {
         if removed.len() == 1 && added.len() == 1 {
@@ -95,18 +106,18 @@ pub fn update_lockfile(manifest_path: &Path,
             } else {
                 format!("{} -> v{}", removed[0], added[0].version())
             };
-            try!(print_change("Updating", msg));
+            print_change("Updating", msg, Green)?;
         } else {
             for package in removed.iter() {
-                try!(print_change("Removing", format!("{}", package)));
+                print_change("Removing", format!("{}", package), Red)?;
             }
             for package in added.iter() {
-                try!(print_change("Adding", format!("{}", package)));
+                print_change("Adding", format!("{}", package), Cyan)?;
             }
         }
     }
 
-    try!(ops::write_pkg_lockfile(&package, &resolve, opts.config));
+    ops::write_pkg_lockfile(ws, &resolve)?;
     return Ok(());
 
     fn fill_with_deps<'a>(resolve: &'a Resolve, dep: &'a PackageId,
@@ -148,7 +159,7 @@ pub fn update_lockfile(manifest_path: &Path,
                 // we'll let it through.
                 //
                 // Note that we only check this for non-registry sources,
-                // however, as registries countain enough version information in
+                // however, as registries contain enough version information in
                 // the package id to disambiguate
                 if a.source_id().is_registry() {
                     return false
@@ -169,7 +180,7 @@ pub fn update_lockfile(manifest_path: &Path,
             changes.entry(key(dep)).or_insert(empty.clone()).1.push(dep);
         }
 
-        for (_, v) in changes.iter_mut() {
+        for v in changes.values_mut() {
             let (ref mut old, ref mut new) = *v;
             old.sort();
             new.sort();
