@@ -37,7 +37,9 @@ fn metadata_no_deps(ws: &Workspace,
         packages: ws.members().cloned().collect(),
         workspace_members: ws.members().map(|pkg| pkg.package_id().clone()).collect(),
         resolve: None,
+        target_directory: ws.target_dir().display().to_string(),
         version: VERSION,
+        workspace_root: ws.root().display().to_string(),
     })
 }
 
@@ -63,7 +65,9 @@ fn metadata_full(ws: &Workspace,
             resolve: resolve,
             root: ws.current_opt().map(|pkg| pkg.package_id().clone()),
         }),
+        target_directory: ws.target_dir().display().to_string(),
         version: VERSION,
+        workspace_root: ws.root().display().to_string(),
     })
 }
 
@@ -72,7 +76,9 @@ pub struct ExportInfo {
     packages: Vec<Package>,
     workspace_members: Vec<PackageId>,
     resolve: Option<MetadataResolve>,
+    target_directory: String,
     version: u32,
+    workspace_root: String,
 }
 
 /// Newtype wrapper to provide a custom `Serialize` implementation.
