@@ -7,11 +7,12 @@ use std::sync::Arc;
 use flate2::read::GzDecoder;
 use flate2::{GzBuilder, Compression};
 use git2;
-use tar::{Archive, Builder, Header};
+use tar::{Archive, Builder, Header, EntryType};
 
-use core::{SourceId, Package, PackageId, Workspace, Source};
+use core::{Package, Workspace, Source, SourceId};
 use sources::PathSource;
-use util::{self, CargoResult, human, internal, ChainError, Config, FileLock};
+use util::{self, internal, Config, FileLock};
+use util::errors::{CargoResult, CargoResultExt};
 use ops::{self, DefaultExecutor};
 
 pub struct PackageOpts<'cfg> {
@@ -21,12 +22,15 @@ pub struct PackageOpts<'cfg> {
     pub allow_dirty: bool,
     pub verify: bool,
     pub jobs: Option<u32>,
+    pub target: Option<&'cfg str>,
+    pub registry: Option<String>,
 }
 
 pub fn package(ws: &Workspace,
                opts: &PackageOpts) -> CargoResult<Option<FileLock>> {
     let pkg = ws.current()?;
     let config = ws.config();
+
     let mut src = PathSource::new(pkg.root(),
                                   pkg.package_id().source_id(),
                                   config);
@@ -36,12 +40,12 @@ pub fn package(ws: &Workspace,
         check_metadata(pkg, config)?;
     }
 
-    verify_dependencies(&pkg)?;
+    verify_dependencies(pkg)?;
 
     if opts.list {
         let root = pkg.root();
-        let mut list: Vec<_> = src.list_files(&pkg)?.iter().map(|file| {
-            util::without_prefix(&file, &root).unwrap().to_path_buf()
+        let mut list: Vec<_> = src.list_files(pkg)?.iter().map(|file| {
+            util::without_prefix(file, root).unwrap().to_path_buf()
         }).collect();
         list.sort();
         for file in list.iter() {
@@ -51,7 +55,7 @@ pub fn package(ws: &Workspace,
     }
 
     if !opts.allow_dirty {
-        check_not_dirty(&pkg, &src)?;
+        check_not_dirty(pkg, &src)?;
     }
 
     let filename = format!("{}-{}.crate", pkg.name(), pkg.version());
@@ -67,21 +71,21 @@ pub fn package(ws: &Workspace,
     // it exists.
     config.shell().status("Packaging", pkg.package_id().to_string())?;
     dst.file().set_len(0)?;
-    tar(ws, &src, dst.file(), &filename).chain_error(|| {
-        human("failed to prepare local package for uploading")
+    tar(ws, &src, dst.file(), &filename).chain_err(|| {
+        format_err!("failed to prepare local package for uploading")
     })?;
     if opts.verify {
         dst.seek(SeekFrom::Start(0))?;
-        run_verify(ws, dst.file(), opts).chain_error(|| {
-            human("failed to verify package tarball")
+        run_verify(ws, &dst, opts).chain_err(|| {
+            "failed to verify package tarball"
         })?
     }
     dst.seek(SeekFrom::Start(0))?;
     {
         let src_path = dst.path();
         let dst_path = dst.parent().join(&filename);
-        fs::rename(&src_path, &dst_path).chain_error(|| {
-            human("failed to move temporary tarball into final location")
+        fs::rename(&src_path, &dst_path).chain_err(|| {
+            "failed to move temporary tarball into final location"
         })?;
     }
     Ok(Some(dst))
@@ -112,7 +116,7 @@ fn check_metadata(pkg: &Package, config: &Config) -> CargoResult<()> {
         if !things.is_empty() {
             things.push_str(" or ");
         }
-        things.push_str(&missing.last().unwrap());
+        things.push_str(missing.last().unwrap());
 
         config.shell().warn(
             &format!("manifest has no {things}.\n\
@@ -125,12 +129,10 @@ fn check_metadata(pkg: &Package, config: &Config) -> CargoResult<()> {
 // check that the package dependencies are safe to deploy.
 fn verify_dependencies(pkg: &Package) -> CargoResult<()> {
     for dep in pkg.dependencies() {
-        if dep.source_id().is_path() {
-            if !dep.specified_req() {
-                bail!("all path dependencies must have a version specified \
-                       when packaging.\ndependency `{}` does not specify \
-                       a version.", dep.name())
-            }
+        if dep.source_id().is_path() && !dep.specified_req() {
+            bail!("all path dependencies must have a version specified \
+                    when packaging.\ndependency `{}` does not specify \
+                    a version.", dep.name())
         }
     }
     Ok(())
@@ -188,7 +190,7 @@ fn tar(ws: &Workspace,
     // Prepare the encoder and its header
     let filename = Path::new(filename);
     let encoder = GzBuilder::new().filename(util::path2bytes(filename)?)
-                                  .write(dst, Compression::Best);
+                                  .write(dst, Compression::best());
 
     // Put all package files into a compressed archive
     let mut ar = Builder::new(encoder);
@@ -196,14 +198,11 @@ fn tar(ws: &Workspace,
     let config = ws.config();
     let root = pkg.root();
     for file in src.list_files(pkg)?.iter() {
-        let relative = util::without_prefix(&file, &root).unwrap();
+        let relative = util::without_prefix(file, root).unwrap();
         check_filename(relative)?;
-        let relative = relative.to_str().chain_error(|| {
-            human(format!("non-utf8 path in source directory: {}",
-                          relative.display()))
-        })?;
-        let mut file = File::open(file).chain_error(|| {
-            human(format!("failed to open for archiving: `{}`", file.display()))
+        let relative = relative.to_str().ok_or_else(|| {
+            format_err!("non-utf8 path in source directory: {}",
+                        relative.display())
         })?;
         config.shell().verbose(|shell| {
             shell.status("Archiving", &relative)
@@ -230,71 +229,77 @@ fn tar(ws: &Workspace,
         // unpack the selectors 0.4.0 crate on crates.io. Either that or take a
         // look at rust-lang/cargo#2326
         let mut header = Header::new_ustar();
-        let metadata = file.metadata().chain_error(|| {
-            human(format!("could not learn metadata for: `{}`", relative))
+        header.set_path(&path).chain_err(|| {
+            format!("failed to add to archive: `{}`", relative)
         })?;
-        header.set_path(&path).chain_error(|| {
-            human(format!("failed to add to archive: `{}`", relative))
+        let mut file = File::open(file).chain_err(|| {
+            format!("failed to open for archiving: `{}`", file.display())
+        })?;
+        let metadata = file.metadata().chain_err(|| {
+            format!("could not learn metadata for: `{}`", relative)
         })?;
         header.set_metadata(&metadata);
-        header.set_cksum();
 
-        ar.append(&header, &mut file).chain_error(|| {
-            internal(format!("could not archive source file `{}`", relative))
-        })?;
+        if relative == "Cargo.toml" {
+            let orig = Path::new(&path).with_file_name("Cargo.toml.orig");
+            header.set_path(&orig)?;
+            header.set_cksum();
+            ar.append(&header, &mut file).chain_err(|| {
+                internal(format!("could not archive source file `{}`", relative))
+            })?;
+
+            let mut header = Header::new_ustar();
+            let toml = pkg.to_registry_toml(ws.config())?;
+            header.set_path(&path)?;
+            header.set_entry_type(EntryType::file());
+            header.set_mode(0o644);
+            header.set_size(toml.len() as u64);
+            header.set_cksum();
+            ar.append(&header, toml.as_bytes()).chain_err(|| {
+                internal(format!("could not archive source file `{}`", relative))
+            })?;
+        } else {
+            header.set_cksum();
+            ar.append(&header, &mut file).chain_err(|| {
+                internal(format!("could not archive source file `{}`", relative))
+            })?;
+        }
     }
     let encoder = ar.into_inner()?;
     encoder.finish()?;
     Ok(())
 }
 
-fn run_verify(ws: &Workspace, tar: &File, opts: &PackageOpts) -> CargoResult<()> {
+fn run_verify(ws: &Workspace, tar: &FileLock, opts: &PackageOpts) -> CargoResult<()> {
     let config = ws.config();
     let pkg = ws.current()?;
 
     config.shell().status("Verifying", pkg)?;
 
-    let f = GzDecoder::new(tar)?;
-    let dst = pkg.root().join(&format!("target/package/{}-{}",
-                                       pkg.name(), pkg.version()));
+    let f = GzDecoder::new(tar.file());
+    let dst = tar.parent().join(&format!("{}-{}", pkg.name(), pkg.version()));
     if fs::metadata(&dst).is_ok() {
         fs::remove_dir_all(&dst)?;
     }
     let mut archive = Archive::new(f);
     archive.unpack(dst.parent().unwrap())?;
-    let manifest_path = dst.join("Cargo.toml");
 
-    // When packages are uploaded to a registry, all path dependencies are
-    // implicitly converted to registry dependencies, so we rewrite those
-    // dependencies here.
-    //
-    // We also make sure to point all paths at `dst` instead of the previous
-    // location that the package was originally read from. In locking the
-    // `SourceId` we're telling it that the corresponding `PathSource` will be
-    // considered updated and we won't actually read any packages.
-    let cratesio = SourceId::crates_io(config)?;
-    let precise = Some("locked".to_string());
-    let new_src = SourceId::for_path(&dst)?.with_precise(precise);
-    let new_pkgid = PackageId::new(pkg.name(), pkg.version(), &new_src)?;
-    let new_summary = pkg.summary().clone().map_dependencies(|d| {
-        if !d.source_id().is_path() { return d }
-        d.clone_inner().set_source_id(cratesio.clone()).into_dependency()
-    });
-    let mut new_manifest = pkg.manifest().clone();
-    new_manifest.set_summary(new_summary.override_id(new_pkgid));
-    let new_pkg = Package::new(new_manifest, &manifest_path);
-
-    // Now that we've rewritten all our path dependencies, compile it!
+    // Manufacture an ephemeral workspace to ensure that even if the top-level
+    // package has a workspace we can still build our new crate.
+    let id = SourceId::for_path(&dst)?;
+    let mut src = PathSource::new(&dst, &id, ws.config());
+    let new_pkg = src.root_package()?;
     let ws = Workspace::ephemeral(new_pkg, config, None, true)?;
+
     ops::compile_ws(&ws, None, &ops::CompileOptions {
         config: config,
         jobs: opts.jobs,
-        target: None,
+        target: opts.target,
         features: &[],
         no_default_features: false,
         all_features: false,
         spec: ops::Packages::Packages(&[]),
-        filter: ops::CompileFilter::Everything { required_features_filterable: true },
+        filter: ops::CompileFilter::Default { required_features_filterable: true },
         release: false,
         message_format: ops::MessageFormat::Human,
         mode: ops::CompileMode::Build,
@@ -324,7 +329,7 @@ fn check_filename(file: &Path) -> CargoResult<()> {
         }
     };
     let bad_chars = ['/', '\\', '<', '>', ':', '"', '|', '?', '*'];
-    for c in bad_chars.iter().filter(|c| name.contains(**c)) {
+    if let Some(c) = bad_chars.iter().find(|c| name.contains(**c)) {
         bail!("cannot package a filename with a special character `{}`: {}",
               c, file.display())
     }
