@@ -4,22 +4,10 @@ use std::io;
 use std::path::{Path, PathBuf};
 
 use core::{Package, SourceId, PackageId, EitherManifest};
-use util::{self, paths, CargoResult, human, Config, ChainError};
+use util::{self, Config};
+use util::errors::{CargoResult, CargoError};
 use util::important_paths::find_project_manifest_exact;
-use util::toml::Layout;
-
-pub fn read_manifest(path: &Path, source_id: &SourceId, config: &Config)
-                     -> CargoResult<(EitherManifest, Vec<PathBuf>)> {
-    trace!("read_package; path={}; source-id={}", path.display(), source_id);
-    let contents = paths::read(path)?;
-
-    let layout = Layout::from_project_path(path.parent().unwrap());
-    let root = layout.root.clone();
-    util::toml::to_manifest(&contents, source_id, layout, config).chain_error(|| {
-        human(format!("failed to parse manifest at `{}`",
-                      root.join("Cargo.toml").display()))
-    })
-}
+use util::toml::read_manifest;
 
 pub fn read_package(path: &Path, source_id: &SourceId, config: &Config)
                     -> CargoResult<(Package, Vec<PathBuf>)> {
@@ -40,6 +28,7 @@ pub fn read_packages(path: &Path, source_id: &SourceId, config: &Config)
                      -> CargoResult<Vec<Package>> {
     let mut all_packages = HashMap::new();
     let mut visited = HashSet::<PathBuf>::new();
+    let mut errors = Vec::<CargoError>::new();
 
     trace!("looking for root package: {}, source_id={}", path.display(), source_id);
 
@@ -49,7 +38,7 @@ pub fn read_packages(path: &Path, source_id: &SourceId, config: &Config)
         // Don't recurse into hidden/dot directories unless we're at the toplevel
         if dir != path {
             let name = dir.file_name().and_then(|s| s.to_str());
-            if name.map(|s| s.starts_with(".")) == Some(true) {
+            if name.map(|s| s.starts_with('.')) == Some(true) {
                 return Ok(false)
             }
 
@@ -67,13 +56,16 @@ pub fn read_packages(path: &Path, source_id: &SourceId, config: &Config)
 
         if has_manifest(dir) {
             read_nested_packages(dir, &mut all_packages, source_id, config,
-                                      &mut visited)?;
+                                      &mut visited, &mut errors)?;
         }
         Ok(true)
     })?;
 
     if all_packages.is_empty() {
-        Err(human(format!("Could not find Cargo.toml in `{}`", path.display())))
+        match errors.pop() {
+            Some(err) => Err(err),
+            None => Err(format_err!("Could not find Cargo.toml in `{}`", path.display())),
+        }
     } else {
         Ok(all_packages.into_iter().map(|(_, v)| v).collect())
     }
@@ -94,9 +86,9 @@ fn walk(path: &Path, callback: &mut FnMut(&Path) -> CargoResult<bool>)
             return Ok(())
         }
         Err(e) => {
-            return Err(human(e)).chain_error(|| {
-                human(format!("failed to read directory `{}`", path.display()))
-            })
+            let cx = format!("failed to read directory `{}`", path.display());
+            let e = CargoError::from(e);
+            return Err(e.context(cx).into())
         }
     };
     for dir in dirs {
@@ -116,12 +108,29 @@ fn read_nested_packages(path: &Path,
                         all_packages: &mut HashMap<PackageId, Package>,
                         source_id: &SourceId,
                         config: &Config,
-                        visited: &mut HashSet<PathBuf>) -> CargoResult<()> {
+                        visited: &mut HashSet<PathBuf>,
+                        errors: &mut Vec<CargoError>) -> CargoResult<()> {
     if !visited.insert(path.to_path_buf()) { return Ok(()) }
 
     let manifest_path = find_project_manifest_exact(path, "Cargo.toml")?;
 
-    let (manifest, nested) = read_manifest(&manifest_path, source_id, config)?;
+    let (manifest, nested) = match read_manifest(&manifest_path, source_id, config) {
+        Err(err) => {
+            // Ignore malformed manifests found on git repositories
+            //
+            // git source try to find and read all manifests from the repository
+            // but since it's not possible to exclude folders from this search
+            // it's safer to ignore malformed manifests to avoid
+            //
+            // TODO: Add a way to exclude folders?
+            info!("skipping malformed package found at `{}`",
+                  path.to_string_lossy());
+            errors.push(err);
+            return Ok(());
+        }
+        Ok(tuple) => tuple
+    };
+
     let manifest = match manifest {
         EitherManifest::Real(manifest) => manifest,
         EitherManifest::Virtual(..) => return Ok(()),
@@ -148,7 +157,7 @@ fn read_nested_packages(path: &Path,
         for p in nested.iter() {
             let path = util::normalize_path(&path.join(p));
             read_nested_packages(&path, all_packages, source_id,
-                                      config, visited)?;
+                                      config, visited, errors)?;
         }
     }
 
