@@ -1,8 +1,6 @@
 use std::env;
-use std::fs::{self, File, OpenOptions};
+use std::fs;
 use std::hash::{self, Hasher};
-use std::io::prelude::*;
-use std::io::{BufReader, SeekFrom};
 use std::path::{Path, PathBuf};
 use std::sync::{Arc, Mutex};
 
@@ -11,14 +9,14 @@ use serde::ser::{self, Serialize};
 use serde::de::{self, Deserialize};
 use serde_json;
 
-use core::{Package, TargetKind};
+use core::{Epoch, Package, TargetKind};
 use util;
 use util::{Fresh, Dirty, Freshness, internal, profile};
 use util::errors::{CargoResult, CargoResultExt};
 use util::paths;
 
 use super::job::Work;
-use super::context::{Context, Unit};
+use super::context::{Context, Unit, TargetFileType};
 use super::custom_build::BuildDeps;
 
 /// A tuple result of the `prepare_foo` functions in this module.
@@ -87,7 +85,10 @@ pub fn prepare_target<'a, 'cfg>(cx: &mut Context<'a, 'cfg>,
         missing_outputs = !root.join(unit.target.crate_name())
                                .join("index.html").exists();
     } else {
-        for &(ref src, ref link_dst, _) in cx.target_filenames(unit)?.iter() {
+        for &(ref src, ref link_dst, file_type) in cx.target_filenames(unit)?.iter() {
+            if file_type == TargetFileType::DebugInfo {
+                continue;
+            }
             missing_outputs |= !src.exists();
             if let Some(ref link_dst) = *link_dst {
                 missing_outputs |= !link_dst.exists();
@@ -96,8 +97,9 @@ pub fn prepare_target<'a, 'cfg>(cx: &mut Context<'a, 'cfg>,
     }
 
     let allow_failure = unit.profile.rustc_args.is_some();
+    let target_root = cx.target_root().to_path_buf();
     let write_fingerprint = Work::new(move |_| {
-        match fingerprint.update_local() {
+        match fingerprint.update_local(&target_root) {
             Ok(()) => {}
             Err(..) if allow_failure => return Ok(()),
             Err(e) => return Err(e)
@@ -136,15 +138,17 @@ pub struct Fingerprint {
     features: String,
     target: u64,
     profile: u64,
+    path: u64,
     #[serde(serialize_with = "serialize_deps", deserialize_with = "deserialize_deps")]
     deps: Vec<(String, Arc<Fingerprint>)>,
     local: Vec<LocalFingerprint>,
     #[serde(skip_serializing, skip_deserializing)]
     memoized_hash: Mutex<Option<u64>>,
     rustflags: Vec<String>,
+    epoch: Epoch,
 }
 
-fn serialize_deps<S>(deps: &Vec<(String, Arc<Fingerprint>)>, ser: S)
+fn serialize_deps<S>(deps: &[(String, Arc<Fingerprint>)], ser: S)
                      -> Result<S::Ok, S::Error>
     where S: ser::Serializer,
 {
@@ -162,10 +166,12 @@ fn deserialize_deps<'de, D>(d: D) -> Result<Vec<(String, Arc<Fingerprint>)>, D::
             rustc: 0,
             target: 0,
             profile: 0,
+            path: 0,
             local: vec![LocalFingerprint::Precalculated(String::new())],
             features: String::new(),
             deps: Vec::new(),
             memoized_hash: Mutex::new(Some(hash)),
+            epoch: Epoch::Epoch2015,
             rustflags: Vec::new(),
         }))
     }).collect())
@@ -178,15 +184,27 @@ enum LocalFingerprint {
     EnvBased(String, Option<String>),
 }
 
+impl LocalFingerprint {
+    fn mtime(root: &Path, mtime: Option<FileTime>, path: &Path)
+        -> LocalFingerprint
+    {
+        let mtime = MtimeSlot(Mutex::new(mtime));
+        assert!(path.is_absolute());
+        let path = path.strip_prefix(root).unwrap_or(path);
+        LocalFingerprint::MtimeBased(mtime, path.to_path_buf())
+    }
+}
+
 struct MtimeSlot(Mutex<Option<FileTime>>);
 
 impl Fingerprint {
-    fn update_local(&self) -> CargoResult<()> {
+    fn update_local(&self, root: &Path) -> CargoResult<()> {
         let mut hash_busted = false;
         for local in self.local.iter() {
             match *local {
                 LocalFingerprint::MtimeBased(ref slot, ref path) => {
-                    let meta = fs::metadata(path)
+                    let path = root.join(path);
+                    let meta = fs::metadata(&path)
                         .chain_err(|| {
                             internal(format!("failed to stat `{}`", path.display()))
                         })?;
@@ -224,15 +242,21 @@ impl Fingerprint {
         if self.target != old.target {
             bail!("target configuration has changed")
         }
+        if self.path != old.path {
+            bail!("path to the compiler has changed")
+        }
         if self.profile != old.profile {
             bail!("profile configuration has changed")
         }
         if self.rustflags != old.rustflags {
-            return Err(internal("RUSTFLAGS has changed"))
+            bail!("RUSTFLAGS has changed")
         }
         if self.local.len() != old.local.len() {
             bail!("local lens changed");
         }
+        if self.epoch != old.epoch {
+            bail!("epoch changed")
+        }
         for (new, old) in self.local.iter().zip(&old.local) {
             match (new, old) {
                 (&LocalFingerprint::Precalculated(ref a),
@@ -291,13 +315,22 @@ impl hash::Hash for Fingerprint {
             rustc,
             ref features,
             target,
+            path,
             profile,
             ref deps,
             ref local,
             memoized_hash: _,
+            epoch,
             ref rustflags,
         } = *self;
-        (rustc, features, target, profile, deps, local, rustflags).hash(h)
+        (rustc, features, target, path, profile, local, epoch, rustflags).hash(h);
+
+        h.write_usize(deps.len());
+        for &(ref name, ref fingerprint) in deps {
+            name.hash(h);
+            // use memoized dep hashes to avoid exponential blowup
+            h.write_u64(Fingerprint::hash(fingerprint));
+        }
     }
 }
 
@@ -343,7 +376,7 @@ impl<'de> de::Deserialize<'de> for MtimeSlot {
 fn calculate<'a, 'cfg>(cx: &mut Context<'a, 'cfg>, unit: &Unit<'a>)
                        -> CargoResult<Arc<Fingerprint>> {
     if let Some(s) = cx.fingerprints.get(unit) {
-        return Ok(s.clone())
+        return Ok(Arc::clone(s))
     }
 
     // Next, recursively calculate the fingerprint for all of our dependencies.
@@ -365,8 +398,8 @@ fn calculate<'a, 'cfg>(cx: &mut Context<'a, 'cfg>, unit: &Unit<'a>)
     // And finally, calculate what our own local fingerprint is
     let local = if use_dep_info(unit) {
         let dep_info = dep_info_loc(cx, unit);
-        let mtime = dep_info_mtime_if_fresh(&dep_info)?;
-        LocalFingerprint::MtimeBased(MtimeSlot(Mutex::new(mtime)), dep_info)
+        let mtime = dep_info_mtime_if_fresh(unit.pkg, &dep_info)?;
+        LocalFingerprint::mtime(cx.target_root(), mtime, &dep_info)
     } else {
         let fingerprint = pkg_fingerprint(cx, unit.pkg)?;
         LocalFingerprint::Precalculated(fingerprint)
@@ -381,14 +414,18 @@ fn calculate<'a, 'cfg>(cx: &mut Context<'a, 'cfg>, unit: &Unit<'a>)
     let fingerprint = Arc::new(Fingerprint {
         rustc: util::hash_u64(&cx.config.rustc()?.verbose_version),
         target: util::hash_u64(&unit.target),
-        profile: util::hash_u64(&unit.profile),
+        profile: util::hash_u64(&(&unit.profile, cx.incremental_args(unit)?)),
+        // Note that .0 is hashed here, not .1 which is the cwd. That doesn't
+        // actually affect the output artifact so there's no need to hash it.
+        path: util::hash_u64(&super::path_args(cx, unit).0),
         features: format!("{:?}", cx.resolve.features_sorted(unit.pkg.package_id())),
         deps: deps,
         local: vec![local],
         memoized_hash: Mutex::new(None),
+        epoch: unit.pkg.manifest().epoch(),
         rustflags: extra_flags,
     });
-    cx.fingerprints.insert(*unit, fingerprint.clone());
+    cx.fingerprints.insert(*unit, Arc::clone(&fingerprint));
     Ok(fingerprint)
 }
 
@@ -433,10 +470,12 @@ pub fn prepare_build_cmd<'a, 'cfg>(cx: &mut Context<'a, 'cfg>, unit: &Unit<'a>)
         rustc: 0,
         target: 0,
         profile: 0,
+        path: 0,
         features: String::new(),
         deps: Vec::new(),
         local: local,
         memoized_hash: Mutex::new(None),
+        epoch: Epoch::Epoch2015,
         rustflags: Vec::new(),
     };
     let compare = compare_old_fingerprint(&loc, &fingerprint);
@@ -452,9 +491,10 @@ pub fn prepare_build_cmd<'a, 'cfg>(cx: &mut Context<'a, 'cfg>, unit: &Unit<'a>)
     // Hence, if there were some `rerun-if-changed` directives forcibly change
     // the kind of fingerprint by reinterpreting the dependencies output by the
     // build script.
-    let state = cx.build_state.clone();
+    let state = Arc::clone(&cx.build_state);
     let key = (unit.pkg.package_id().clone(), unit.kind);
-    let root = unit.pkg.root().to_path_buf();
+    let pkg_root = unit.pkg.root().to_path_buf();
+    let target_root = cx.target_root().to_path_buf();
     let write_fingerprint = Work::new(move |_| {
         if let Some(output_path) = output_path {
             let outputs = state.outputs.lock().unwrap();
@@ -462,8 +502,8 @@ pub fn prepare_build_cmd<'a, 'cfg>(cx: &mut Context<'a, 'cfg>, unit: &Unit<'a>)
             if !outputs.rerun_if_changed.is_empty() ||
                !outputs.rerun_if_env_changed.is_empty() {
                 let deps = BuildDeps::new(&output_path, Some(outputs));
-                fingerprint.local = local_fingerprints_deps(&deps, &root);
-                fingerprint.update_local()?;
+                fingerprint.local = local_fingerprints_deps(&deps, &target_root, &pkg_root);
+                fingerprint.update_local(&target_root)?;
             }
         }
         write_fingerprint(&loc, &fingerprint)
@@ -506,18 +546,19 @@ fn build_script_local_fingerprints<'a, 'cfg>(cx: &mut Context<'a, 'cfg>,
     // Ok so now we're in "new mode" where we can have files listed as
     // dependencies as well as env vars listed as dependencies. Process them all
     // here.
-    Ok((local_fingerprints_deps(deps, unit.pkg.root()), Some(output)))
+    Ok((local_fingerprints_deps(deps, cx.target_root(), unit.pkg.root()), Some(output)))
 }
 
-fn local_fingerprints_deps(deps: &BuildDeps, root: &Path) -> Vec<LocalFingerprint> {
+fn local_fingerprints_deps(deps: &BuildDeps, target_root: &Path, pkg_root: &Path)
+    -> Vec<LocalFingerprint>
+{
     debug!("new local fingerprints deps");
     let mut local = Vec::new();
     if !deps.rerun_if_changed.is_empty() {
         let output = &deps.build_script_output;
-        let deps = deps.rerun_if_changed.iter().map(|p| root.join(p));
+        let deps = deps.rerun_if_changed.iter().map(|p| pkg_root.join(p));
         let mtime = mtime_if_fresh(output, deps);
-        let mtime = MtimeSlot(Mutex::new(mtime));
-        local.push(LocalFingerprint::MtimeBased(mtime, output.clone()));
+        local.push(LocalFingerprint::mtime(target_root, mtime, output));
     }
 
     for var in deps.rerun_if_env_changed.iter() {
@@ -525,20 +566,20 @@ fn local_fingerprints_deps(deps: &BuildDeps, root: &Path) -> Vec<LocalFingerprin
         local.push(LocalFingerprint::EnvBased(var.clone(), val));
     }
 
-    return local
+    local
 }
 
 fn write_fingerprint(loc: &Path, fingerprint: &Fingerprint) -> CargoResult<()> {
     let hash = fingerprint.hash();
     debug!("write fingerprint: {}", loc.display());
-    paths::write(&loc, util::to_hex(hash).as_bytes())?;
+    paths::write(loc, util::to_hex(hash).as_bytes())?;
     paths::write(&loc.with_extension("json"),
                  &serde_json::to_vec(&fingerprint).unwrap())?;
     Ok(())
 }
 
 /// Prepare for work when a package starts to build
-pub fn prepare_init(cx: &mut Context, unit: &Unit) -> CargoResult<()> {
+pub fn prepare_init<'a, 'cfg>(cx: &mut Context<'a, 'cfg>, unit: &Unit<'a>) -> CargoResult<()> {
     let new1 = cx.fingerprint_dir(unit);
 
     if fs::metadata(&new1).is_err() {
@@ -548,7 +589,7 @@ pub fn prepare_init(cx: &mut Context, unit: &Unit) -> CargoResult<()> {
     Ok(())
 }
 
-pub fn dep_info_loc(cx: &mut Context, unit: &Unit) -> PathBuf {
+pub fn dep_info_loc<'a, 'cfg>(cx: &mut Context<'a, 'cfg>, unit: &Unit<'a>) -> PathBuf {
     cx.fingerprint_dir(unit).join(&format!("dep-{}", filename(cx, unit)))
 }
 
@@ -563,7 +604,7 @@ fn compare_old_fingerprint(loc: &Path, new_fingerprint: &Fingerprint)
 
     let old_fingerprint_json = paths::read(&loc.with_extension("json"))?;
     let old_fingerprint = serde_json::from_str(&old_fingerprint_json)
-        .chain_err(|| internal(format!("failed to deserialize json")))?;
+        .chain_err(|| internal("failed to deserialize json"))?;
     new_fingerprint.compare(&old_fingerprint)
 }
 
@@ -574,51 +615,34 @@ fn log_compare(unit: &Unit, compare: &CargoResult<()>) {
     };
     info!("fingerprint error for {}: {}", unit.pkg, ce);
 
-    for cause in ce.iter() {
+    for cause in ce.causes().skip(1) {
         info!("  cause: {}", cause);
     }
 }
 
 // Parse the dep-info into a list of paths
-pub fn parse_dep_info(dep_info: &Path) -> CargoResult<Option<Vec<PathBuf>>> {
-    macro_rules! fs_try {
-        ($e:expr) => (match $e { Ok(e) => e, Err(..) => return Ok(None) })
-    }
-    let mut f = BufReader::new(fs_try!(File::open(dep_info)));
-    // see comments in append_current_dir for where this cwd is manifested from.
-    let mut cwd = Vec::new();
-    if fs_try!(f.read_until(0, &mut cwd)) == 0 {
-        return Ok(None)
-    }
-    let cwd = util::bytes2path(&cwd[..cwd.len()-1])?;
-    let line = match f.lines().next() {
-        Some(Ok(line)) => line,
-        _ => return Ok(None),
+pub fn parse_dep_info(pkg: &Package, dep_info: &Path)
+    -> CargoResult<Option<Vec<PathBuf>>>
+{
+    let data = match paths::read_bytes(dep_info) {
+        Ok(data) => data,
+        Err(_) => return Ok(None),
     };
-    let pos = line.find(": ").ok_or_else(|| {
-        internal(format!("dep-info not in an understood format: {}",
-                         dep_info.display()))
-    })?;
-    let deps = &line[pos + 2..];
-
-    let mut paths = Vec::new();
-    let mut deps = deps.split(' ').map(|s| s.trim()).filter(|s| !s.is_empty());
-    while let Some(s) = deps.next() {
-        let mut file = s.to_string();
-        while file.ends_with('\\') {
-            file.pop();
-            file.push(' ');
-            file.push_str(deps.next().ok_or_else(|| {
-                internal("malformed dep-info format, trailing \\".to_string())
-            })?);
-        }
-        paths.push(cwd.join(&file));
+    let paths = data.split(|&x| x == 0)
+        .filter(|x| !x.is_empty())
+        .map(|p| util::bytes2path(p).map(|p| pkg.root().join(p)))
+        .collect::<Result<Vec<_>, _>>()?;
+    if paths.len() == 0 {
+        Ok(None)
+    } else {
+        Ok(Some(paths))
     }
-    Ok(Some(paths))
 }
 
-fn dep_info_mtime_if_fresh(dep_info: &Path) -> CargoResult<Option<FileTime>> {
-    if let Some(paths) = parse_dep_info(dep_info)? {
+fn dep_info_mtime_if_fresh(pkg: &Package, dep_info: &Path)
+    -> CargoResult<Option<FileTime>>
+{
+    if let Some(paths) = parse_dep_info(pkg, dep_info)? {
         Ok(mtime_if_fresh(dep_info, paths.iter()))
     } else {
         Ok(None)
@@ -670,7 +694,7 @@ fn mtime_if_fresh<I>(output: &Path, paths: I) -> Option<FileTime>
     }
 }
 
-fn filename(cx: &mut Context, unit: &Unit) -> String {
+fn filename<'a, 'cfg>(cx: &mut Context<'a, 'cfg>, unit: &Unit<'a>) -> String {
     // file_stem includes metadata hash. Thus we have a different
     // fingerprint for every metadata hash version. This works because
     // even if the package is fresh, we'll still link the fresh target
@@ -694,19 +718,55 @@ fn filename(cx: &mut Context, unit: &Unit) -> String {
     format!("{}{}-{}", flavor, kind, file_stem)
 }
 
-// The dep-info files emitted by the compiler all have their listed paths
-// relative to whatever the current directory was at the time that the compiler
-// was invoked. As the current directory may change over time, we need to record
-// what that directory was at the beginning of the file so we can know about it
-// next time.
-pub fn append_current_dir(path: &Path, cwd: &Path) -> CargoResult<()> {
-    debug!("appending {} <- {}", path.display(), cwd.display());
-    let mut f = OpenOptions::new().read(true).write(true).open(path)?;
-    let mut contents = Vec::new();
-    f.read_to_end(&mut contents)?;
-    f.seek(SeekFrom::Start(0))?;
-    f.write_all(util::path2bytes(cwd)?)?;
-    f.write_all(&[0])?;
-    f.write_all(&contents)?;
+/// Parses the dep-info file coming out of rustc into a Cargo-specific format.
+///
+/// This function will parse `rustc_dep_info` as a makefile-style dep info to
+/// learn about the all files which a crate depends on. This is then
+/// re-serialized into the `cargo_dep_info` path in a Cargo-specific format.
+///
+/// The `pkg_root` argument here is the absolute path to the directory
+/// containing `Cargo.toml` for this crate that was compiled. The paths listed
+/// in the rustc dep-info file may or may not be absolute but we'll want to
+/// consider all of them relative to the `root` specified.
+///
+/// The `rustc_cwd` argument is the absolute path to the cwd of the compiler
+/// when it was invoked.
+///
+/// The serialized Cargo format will contain a list of files, all of which are
+/// relative if they're under `root`. or absolute if they're elsewehre.
+pub fn translate_dep_info(rustc_dep_info: &Path,
+                          cargo_dep_info: &Path,
+                          pkg_root: &Path,
+                          rustc_cwd: &Path) -> CargoResult<()> {
+    let contents = paths::read(rustc_dep_info)?;
+    let line = match contents.lines().next() {
+        Some(line) => line,
+        None => return Ok(()),
+    };
+    let pos = line.find(": ").ok_or_else(|| {
+        internal(format!("dep-info not in an understood format: {}",
+                         rustc_dep_info.display()))
+    })?;
+    let deps = &line[pos + 2..];
+
+    let mut new_contents = Vec::new();
+    let mut deps = deps.split(' ').map(|s| s.trim()).filter(|s| !s.is_empty());
+    while let Some(s) = deps.next() {
+        let mut file = s.to_string();
+        while file.ends_with('\\') {
+            file.pop();
+            file.push(' ');
+            file.push_str(deps.next().ok_or_else(|| {
+                internal("malformed dep-info format, trailing \\".to_string())
+            })?);
+        }
+
+        let absolute = rustc_cwd.join(file);
+        let path = absolute.strip_prefix(pkg_root).unwrap_or(&absolute);
+        new_contents.extend(util::path2bytes(path)?);
+        new_contents.push(0);
+    }
+    paths::write(cargo_dep_info, &new_contents)?;
     Ok(())
 }
+
