@@ -1,14 +1,11 @@
-use std::collections::HashMap;
-use std::env;
+use std::{cmp, env};
 use std::fs::{self, File};
 use std::iter::repeat;
-use std::path::PathBuf;
 use std::time::Duration;
 
 use curl::easy::{Easy, SslOpt};
 use git2;
 use registry::{Registry, NewCrate, NewCrateDependency};
-use term::color::BLACK;
 
 use url::percent_encoding::{percent_encode, QUERY_ENCODE_SET};
 
@@ -19,10 +16,10 @@ use core::dependency::Kind;
 use core::manifest::ManifestMetadata;
 use ops;
 use sources::{RegistrySource};
-use util::config;
+use util::config::{self, Config};
 use util::paths;
-use util::{CargoResult, human, ChainError, ToUrl};
-use util::config::{Config, ConfigValue, Location};
+use util::ToUrl;
+use util::errors::{CargoResult, CargoResultExt};
 use util::important_paths::find_root_manifest_for_wd;
 
 pub struct RegistryConfig {
@@ -37,20 +34,38 @@ pub struct PublishOpts<'cfg> {
     pub verify: bool,
     pub allow_dirty: bool,
     pub jobs: Option<u32>,
+    pub target: Option<&'cfg str>,
     pub dry_run: bool,
+    pub registry: Option<String>,
 }
 
 pub fn publish(ws: &Workspace, opts: &PublishOpts) -> CargoResult<()> {
     let pkg = ws.current()?;
 
-    if !pkg.publish() {
-        bail!("some crates cannot be published.\n\
-               `{}` is marked as unpublishable", pkg.name());
+    // Allow publishing if a registry has been provided, or if there are no nightly
+    // features enabled.
+    if opts.registry.is_none() && !pkg.manifest().features().activated().is_empty() {
+        bail!("cannot publish crates which activate nightly-only cargo features to crates.io")
+    }
+
+    if let &Some(ref allowed_registries) = pkg.publish() {
+        if !match opts.registry {
+            Some(ref registry) => allowed_registries.contains(registry),
+            None => false,
+        } {
+            bail!("some crates cannot be published.\n\
+                   `{}` is marked as unpublishable", pkg.name());
+        }
+    }
+
+    if !pkg.manifest().patch().is_empty() {
+        bail!("published crates cannot contain [patch] sections");
     }
 
     let (mut registry, reg_id) = registry(opts.config,
-                                               opts.token.clone(),
-                                               opts.index.clone())?;
+                                          opts.token.clone(),
+                                          opts.index.clone(),
+                                          opts.registry.clone())?;
     verify_dependencies(pkg, &reg_id)?;
 
     // Prepare a tarball, with a non-surpressable warning if metadata
@@ -61,12 +76,14 @@ pub fn publish(ws: &Workspace, opts: &PublishOpts) -> CargoResult<()> {
         list: false,
         check_metadata: true,
         allow_dirty: opts.allow_dirty,
+        target: opts.target,
         jobs: opts.jobs,
+        registry: opts.registry.clone(),
     })?.unwrap();
 
     // Upload said tarball to the specified destination
     opts.config.shell().status("Uploading", pkg.package_id().to_string())?;
-    transmit(opts.config, pkg, tarball.file(), &mut registry, opts.dry_run)?;
+    transmit(opts.config, pkg, tarball.file(), &mut registry, &reg_id, opts.dry_run)?;
 
     Ok(())
 }
@@ -81,11 +98,22 @@ fn verify_dependencies(pkg: &Package, registry_src: &SourceId)
                        a version", dep.name())
             }
         } else if dep.source_id() != registry_src {
-            bail!("crates cannot be published to crates.io with dependencies sourced from \
-                   a repository\neither publish `{}` as its own crate on crates.io and \
-                   specify a crates.io version as a dependency or pull it into this \
-                   repository and specify it with a path and version\n(crate `{}` has \
-                   repository path `{}`)", dep.name(), dep.name(),  dep.source_id());
+            if dep.source_id().is_registry() {
+                // Block requests to send to a registry if it is not an alternative
+                // registry
+                if !registry_src.is_alt_registry() {
+                    bail!("crates cannot be published to crates.io with dependencies sourced from other\n\
+                           registries either publish `{}` on crates.io or pull it into this repository\n\
+                           and specify it with a path and version\n\
+                           (crate `{}` is pulled from {})", dep.name(), dep.name(), dep.source_id());
+                }
+            } else {
+                bail!("crates cannot be published to crates.io with dependencies sourced from \
+                       a repository\neither publish `{}` as its own crate on crates.io and \
+                       specify a crates.io version as a dependency or pull it into this \
+                       repository and specify it with a path and version\n(crate `{}` has \
+                       repository path `{}`)", dep.name(), dep.name(),  dep.source_id());
+            }
         }
     }
     Ok(())
@@ -95,9 +123,24 @@ fn transmit(config: &Config,
             pkg: &Package,
             tarball: &File,
             registry: &mut Registry,
+            registry_id: &SourceId,
             dry_run: bool) -> CargoResult<()> {
+
     let deps = pkg.dependencies().iter().map(|dep| {
-        NewCrateDependency {
+
+        // If the dependency is from a different registry, then include the
+        // registry in the dependency.
+        let dep_registry_id = match dep.registry_id() {
+            Some(id) => id,
+            None => bail!("dependency missing registry ID"),
+        };
+        let dep_registry = if dep_registry_id != registry_id {
+            Some(dep_registry_id.url().to_string())
+        } else {
+            None
+        };
+
+        Ok(NewCrateDependency {
             optional: dep.is_optional(),
             default_features: dep.uses_default_features(),
             name: dep.name().to_string(),
@@ -109,15 +152,16 @@ fn transmit(config: &Config,
                 Kind::Build => "build",
                 Kind::Development => "dev",
             }.to_string(),
-        }
-    }).collect::<Vec<NewCrateDependency>>();
+            registry: dep_registry,
+        })
+    }).collect::<CargoResult<Vec<NewCrateDependency>>>()?;
     let manifest = pkg.manifest();
     let ManifestMetadata {
         ref authors, ref description, ref homepage, ref documentation,
         ref keywords, ref readme, ref repository, ref license, ref license_file,
         ref categories, ref badges,
     } = *manifest.metadata();
-    let readme = match *readme {
+    let readme_content = match *readme {
         Some(ref readme) => Some(paths::read(&pkg.root().join(readme))?),
         None => None,
     };
@@ -144,7 +188,8 @@ fn transmit(config: &Config,
         documentation: documentation.clone(),
         keywords: keywords.clone(),
         categories: categories.clone(),
-        readme: readme,
+        readme: readme_content,
+        readme_file: readme.clone(),
         repository: repository.clone(),
         license: license.clone(),
         license_file: license_file.clone(),
@@ -175,35 +220,52 @@ fn transmit(config: &Config,
 
             Ok(())
         },
-        Err(e) => Err(human(e.to_string())),
+        Err(e) => Err(e),
     }
 }
 
-pub fn registry_configuration(config: &Config) -> CargoResult<RegistryConfig> {
-    let index = config.get_string("registry.index")?.map(|p| p.val);
-    let token = config.get_string("registry.token")?.map(|p| p.val);
-    Ok(RegistryConfig { index: index, token: token })
+pub fn registry_configuration(config: &Config,
+                              registry: Option<String>) -> CargoResult<RegistryConfig> {
+
+    let (index, token) = match registry {
+        Some(registry) => {
+            (Some(config.get_registry_index(&registry)?.to_string()),
+             config.get_string(&format!("registries.{}.token", registry))?.map(|p| p.val))
+        }
+        None => {
+            // Checking out for default index and token
+            (config.get_string("registry.index")?.map(|p| p.val),
+             config.get_string("registry.token")?.map(|p| p.val))
+        }
+    };
+
+    Ok(RegistryConfig {
+        index: index,
+        token: token
+    })
 }
 
 pub fn registry(config: &Config,
                 token: Option<String>,
-                index: Option<String>) -> CargoResult<(Registry, SourceId)> {
+                index: Option<String>,
+                registry: Option<String>) -> CargoResult<(Registry, SourceId)> {
     // Parse all configuration options
     let RegistryConfig {
         token: token_config,
-        index: _index_config,
-    } = registry_configuration(config)?;
+        index: index_config,
+    } = registry_configuration(config, registry.clone())?;
     let token = token.or(token_config);
-    let sid = match index {
-        Some(index) => SourceId::for_registry(&index.to_url()?),
-        None => SourceId::crates_io(config)?,
+    let sid = match (index_config, index, registry) {
+        (_, _, Some(registry)) => SourceId::alt_registry(config, &registry)?,
+        (Some(index), _, _) | (None, Some(index), _) => SourceId::for_registry(&index.to_url()?)?,
+        (None, None, _) => SourceId::crates_io(config)?,
     };
     let api_host = {
         let mut src = RegistrySource::remote(&sid, config);
-        src.update().chain_error(|| {
-            human(format!("failed to update {}", sid))
+        src.update().chain_err(|| {
+            format!("failed to update {}", sid)
         })?;
-        (src.config()?).unwrap().api
+        (src.config()?).unwrap().api.unwrap()
     };
     let handle = http_handle(config)?;
     Ok((Registry::new_handle(api_host, token, handle), sid))
@@ -211,16 +273,38 @@ pub fn registry(config: &Config,
 
 /// Create a new HTTP handle with appropriate global configuration for cargo.
 pub fn http_handle(config: &Config) -> CargoResult<Easy> {
-    if !config.network_allowed() {
+    if config.frozen() {
         bail!("attempting to make an HTTP request, but --frozen was \
                specified")
     }
+    if !config.network_allowed() {
+        bail!("can't make HTTP request in the offline mode")
+    }
 
     // The timeout option for libcurl by default times out the entire transfer,
     // but we probably don't want this. Instead we only set timeouts for the
     // connect phase as well as a "low speed" timeout so if we don't receive
     // many bytes in a large-ish period of time then we time out.
     let mut handle = Easy::new();
+    configure_http_handle(config, &mut handle)?;
+    Ok(handle)
+}
+
+pub fn needs_custom_http_transport(config: &Config) -> CargoResult<bool> {
+    let proxy_exists = http_proxy_exists(config)?;
+    let timeout = http_timeout(config)?;
+    let cainfo = config.get_path("http.cainfo")?;
+    let check_revoke = config.get_bool("http.check-revoke")?;
+
+    Ok(proxy_exists || timeout.is_some() || cainfo.is_some() || check_revoke.is_some())
+}
+
+/// Configure a libcurl http handle with the defaults options for Cargo
+pub fn configure_http_handle(config: &Config, handle: &mut Easy) -> CargoResult<()> {
+    // The timeout option for libcurl by default times out the entire transfer,
+    // but we probably don't want this. Instead we only set timeouts for the
+    // connect phase as well as a "low speed" timeout so if we don't receive
+    // many bytes in a large-ish period of time then we time out.
     handle.connect_timeout(Duration::new(30, 0))?;
     handle.low_speed_limit(10 /* bytes per second */)?;
     handle.low_speed_time(Duration::new(30, 0))?;
@@ -238,7 +322,7 @@ pub fn http_handle(config: &Config) -> CargoResult<Easy> {
         handle.connect_timeout(Duration::new(timeout as u64, 0))?;
         handle.low_speed_time(Duration::new(timeout as u64, 0))?;
     }
-    Ok(handle)
+    Ok(())
 }
 
 /// Find an explicit HTTP proxy if one is available.
@@ -263,11 +347,11 @@ fn http_proxy(config: &Config) -> CargoResult<Option<String>> {
 ///
 /// * cargo's `http.proxy`
 /// * git's `http.proxy`
-/// * http_proxy env var
-/// * HTTP_PROXY env var
-/// * https_proxy env var
-/// * HTTPS_PROXY env var
-pub fn http_proxy_exists(config: &Config) -> CargoResult<bool> {
+/// * `http_proxy` env var
+/// * `HTTP_PROXY` env var
+/// * `https_proxy` env var
+/// * `HTTPS_PROXY` env var
+fn http_proxy_exists(config: &Config) -> CargoResult<bool> {
     if http_proxy(config)?.is_some() {
         Ok(true)
     } else {
@@ -276,24 +360,28 @@ pub fn http_proxy_exists(config: &Config) -> CargoResult<bool> {
     }
 }
 
-pub fn http_timeout(config: &Config) -> CargoResult<Option<i64>> {
+fn http_timeout(config: &Config) -> CargoResult<Option<i64>> {
     if let Some(s) = config.get_i64("http.timeout")? {
         return Ok(Some(s.val))
     }
     Ok(env::var("HTTP_TIMEOUT").ok().and_then(|s| s.parse().ok()))
 }
 
-pub fn registry_login(config: &Config, token: String) -> CargoResult<()> {
-    let RegistryConfig { index, token: _ } = registry_configuration(config)?;
-    let mut map = HashMap::new();
-    let p = config.cwd().to_path_buf();
-    if let Some(index) = index {
-        map.insert("index".to_string(), ConfigValue::String(index, p.clone()));
-    }
-    map.insert("token".to_string(), ConfigValue::String(token, p));
+pub fn registry_login(config: &Config,
+                      token: String,
+                      registry: Option<String>) -> CargoResult<()> {
+    let RegistryConfig {
+        token: old_token,
+        ..
+    } = registry_configuration(config, registry.clone())?;
 
-    config::set_config(config, Location::Global, "registry",
-                       ConfigValue::Table(map, PathBuf::from(".")))
+    if let Some(old_token) = old_token {
+        if old_token == token {
+            return Ok(());
+        }
+    }
+
+    config::save_credentials(config, token, registry)
 }
 
 pub struct OwnersOptions {
@@ -303,6 +391,7 @@ pub struct OwnersOptions {
     pub to_add: Option<Vec<String>>,
     pub to_remove: Option<Vec<String>>,
     pub list: bool,
+    pub registry: Option<String>,
 }
 
 pub fn modify_owners(config: &Config, opts: &OwnersOptions) -> CargoResult<()> {
@@ -315,30 +404,32 @@ pub fn modify_owners(config: &Config, opts: &OwnersOptions) -> CargoResult<()> {
         }
     };
 
-    let (mut registry, _) = registry(config, opts.token.clone(),
-                                          opts.index.clone())?;
+    let (mut registry, _) = registry(config,
+                                     opts.token.clone(),
+                                     opts.index.clone(),
+                                     opts.registry.clone())?;
 
     if let Some(ref v) = opts.to_add {
         let v = v.iter().map(|s| &s[..]).collect::<Vec<_>>();
-        config.shell().status("Owner", format!("adding {:?} to crate {}",
-                                                    v, name))?;
-        registry.add_owners(&name, &v).map_err(|e| {
-            human(format!("failed to add owners to crate {}: {}", name, e))
+        let msg = registry.add_owners(&name, &v).map_err(|e| {
+            format_err!("failed to invite owners to crate {}: {}", name, e)
         })?;
+
+        config.shell().status("Owner", msg)?;
     }
 
     if let Some(ref v) = opts.to_remove {
         let v = v.iter().map(|s| &s[..]).collect::<Vec<_>>();
         config.shell().status("Owner", format!("removing {:?} from crate {}",
                                                     v, name))?;
-        registry.remove_owners(&name, &v).map_err(|e| {
-            human(format!("failed to remove owners from crate {}: {}", name, e))
+        registry.remove_owners(&name, &v).chain_err(|| {
+            format!("failed to remove owners from crate {}", name)
         })?;
     }
 
     if opts.list {
-        let owners = registry.list_owners(&name).map_err(|e| {
-            human(format!("failed to list owners of crate {}: {}", name, e))
+        let owners = registry.list_owners(&name).chain_err(|| {
+            format!("failed to list owners of crate {}", name)
         })?;
         for owner in owners.iter() {
             print!("{}", owner.login);
@@ -346,7 +437,7 @@ pub fn modify_owners(config: &Config, opts: &OwnersOptions) -> CargoResult<()> {
                 (Some(name), Some(email)) => println!(" ({} <{}>)", name, email),
                 (Some(s), None) |
                 (None, Some(s)) => println!(" ({})", s),
-                (None, None) => println!(""),
+                (None, None) => println!(),
             }
         }
     }
@@ -359,7 +450,8 @@ pub fn yank(config: &Config,
             version: Option<String>,
             token: Option<String>,
             index: Option<String>,
-            undo: bool) -> CargoResult<()> {
+            undo: bool,
+            reg: Option<String>) -> CargoResult<()> {
     let name = match krate {
         Some(name) => name,
         None => {
@@ -373,17 +465,17 @@ pub fn yank(config: &Config,
         None => bail!("a version must be specified to yank")
     };
 
-    let (mut registry, _) = registry(config, token, index)?;
+    let (mut registry, _) = registry(config, token, index, reg)?;
 
     if undo {
         config.shell().status("Unyank", format!("{}:{}", name, version))?;
-        registry.unyank(&name, &version).map_err(|e| {
-            human(format!("failed to undo a yank: {}", e))
+        registry.unyank(&name, &version).chain_err(|| {
+            "failed to undo a yank"
         })?;
     } else {
         config.shell().status("Yank", format!("{}:{}", name, version))?;
-        registry.yank(&name, &version).map_err(|e| {
-            human(format!("failed to yank: {}", e))
+        registry.yank(&name, &version).chain_err(|| {
+            "failed to yank"
         })?;
     }
 
@@ -393,33 +485,42 @@ pub fn yank(config: &Config,
 pub fn search(query: &str,
               config: &Config,
               index: Option<String>,
-              limit: u8) -> CargoResult<()> {
-    fn truncate_with_ellipsis(s: &str, max_length: usize) -> String {
-        if s.len() < max_length {
-            s.to_string()
-        } else {
-            format!("{}…", &s[..max_length - 1])
+              limit: u8,
+              reg: Option<String>) -> CargoResult<()> {
+    fn truncate_with_ellipsis(s: &str, max_width: usize) -> String {
+        // We should truncate at grapheme-boundary and compute character-widths,
+        // yet the dependencies on unicode-segmentation and unicode-width are
+        // not worth it.
+        let mut chars = s.chars();
+        let mut prefix = (&mut chars).take(max_width - 1).collect::<String>();
+        if chars.next().is_some() {
+            prefix.push('…');
         }
+        prefix
     }
 
-    let (mut registry, _) = registry(config, None, index)?;
-    let (crates, total_crates) = registry.search(query, limit).map_err(|e| {
-        human(format!("failed to retrieve search results from the registry: {}", e))
+    let (mut registry, _) = registry(config, None, index, reg)?;
+    let (crates, total_crates) = registry.search(query, limit).chain_err(|| {
+        "failed to retrieve search results from the registry"
     })?;
 
-    let list_items = crates.iter()
-        .map(|krate| (
-            format!("{} = \"{}\"", krate.name, krate.max_version),
-            krate.description.as_ref().map(|desc|
-                truncate_with_ellipsis(&desc.replace("\n", " "), 128))
-        ))
-        .collect::<Vec<_>>();
-    let description_margin = list_items.iter()
-        .map(|&(ref left, _)| left.len() + 4)
-        .max()
-        .unwrap_or(0);
+    let names = crates.iter()
+        .map(|krate| format!("{} = \"{}\"", krate.name, krate.max_version))
+        .collect::<Vec<String>>();
 
-    for (name, description) in list_items.into_iter() {
+    let description_margin = names.iter()
+        .map(|s| s.len() + 4)
+        .max()
+        .unwrap_or_default();
+
+    let description_length = cmp::max(80, 128 - description_margin);
+
+    let descriptions = crates.iter()
+        .map(|krate|
+            krate.description.as_ref().map(|desc|
+                truncate_with_ellipsis(&desc.replace("\n", " "), description_length)));
+
+    for (name, description) in names.into_iter().zip(descriptions) {
         let line = match description {
             Some(desc) => {
                 let space = repeat(' ').take(description_margin - name.len())
@@ -428,25 +529,17 @@ pub fn search(query: &str,
             }
             None => name
         };
-        config.shell().say(line, BLACK)?;
+        println!("{}", line);
     }
 
     let search_max_limit = 100;
-    if total_crates > limit as u32 && limit < search_max_limit {
-        config.shell().say(
-            format!("... and {} crates more (use --limit N to see more)",
-                    total_crates - limit as u32),
-            BLACK)
-        ?;
-    } else if total_crates > limit as u32 && limit >= search_max_limit {
-        config.shell().say(
-            format!(
-                "... and {} crates more (go to http://crates.io/search?q={} to see more)",
-                total_crates - limit as u32,
-                percent_encode(query.as_bytes(), QUERY_ENCODE_SET)
-            ),
-            BLACK)
-        ?;
+    if total_crates > u32::from(limit) && limit < search_max_limit {
+        println!("... and {} crates more (use --limit N to see more)",
+                 total_crates - u32::from(limit));
+    } else if total_crates > u32::from(limit) && limit >= search_max_limit {
+        println!("... and {} crates more (go to http://crates.io/search?q={} to see more)",
+                 total_crates - u32::from(limit),
+                 percent_encode(query.as_bytes(), QUERY_ENCODE_SET));
     }
 
     Ok(())
