@@ -4,12 +4,14 @@ use std::fs::File;
 use std::io::Read;
 use std::path::{Path, PathBuf};
 
-use rustc_serialize::hex::ToHex;
+use hex;
+
 use serde_json;
 
 use core::{Package, PackageId, Summary, SourceId, Source, Dependency, Registry};
 use sources::PathSource;
-use util::{CargoResult, human, ChainError, Config, Sha256};
+use util::{Config, Sha256};
+use util::errors::{CargoResult, CargoResultExt};
 use util::paths;
 
 pub struct DirectorySource<'cfg> {
@@ -21,7 +23,7 @@ pub struct DirectorySource<'cfg> {
 
 #[derive(Deserialize)]
 struct Checksum {
-    package: String,
+    package: Option<String>,
     files: HashMap<String, String>,
 }
 
@@ -44,16 +46,24 @@ impl<'cfg> Debug for DirectorySource<'cfg> {
 }
 
 impl<'cfg> Registry for DirectorySource<'cfg> {
-    fn query(&mut self, dep: &Dependency) -> CargoResult<Vec<Summary>> {
+    fn query(&mut self,
+             dep: &Dependency,
+             f: &mut FnMut(Summary)) -> CargoResult<()> {
         let packages = self.packages.values().map(|p| &p.0);
         let matches = packages.filter(|pkg| dep.matches(pkg.summary()));
-        let summaries = matches.map(|pkg| pkg.summary().clone());
-        Ok(summaries.collect())
+        for summary in matches.map(|pkg| pkg.summary().clone()) {
+            f(summary);
+        }
+        Ok(())
     }
 
     fn supports_checksums(&self) -> bool {
         true
     }
+
+    fn requires_precise(&self) -> bool {
+        true
+    }
 }
 
 impl<'cfg> Source for DirectorySource<'cfg> {
@@ -63,9 +73,9 @@ impl<'cfg> Source for DirectorySource<'cfg> {
 
     fn update(&mut self) -> CargoResult<()> {
         self.packages.clear();
-        let entries = self.root.read_dir().chain_error(|| {
-            human(format!("failed to read root of directory source: {}",
-                          self.root.display()))
+        let entries = self.root.read_dir().chain_err(|| {
+            format!("failed to read root of directory source: {}",
+                    self.root.display())
         })?;
 
         for entry in entries {
@@ -81,28 +91,50 @@ impl<'cfg> Source for DirectorySource<'cfg> {
                 }
             }
 
+            // Vendor directories are often checked into a VCS, but throughout
+            // the lifetime of a vendor dir crates are often added and deleted.
+            // Some VCS implementations don't always fully delete the directory
+            // when a dir is removed from a different checkout. Sometimes a
+            // mostly-empty dir is left behind.
+            //
+            // Additionally vendor directories are sometimes accompanied with
+            // readme files and other auxiliary information not too interesting
+            // to Cargo.
+            //
+            // To help handle all this we only try processing folders with a
+            // `Cargo.toml` in them. This has the upside of being pretty
+            // flexible with the contents of vendor directories but has the
+            // downside of accidentally misconfigured vendor directories
+            // silently returning less crates.
+            if !path.join("Cargo.toml").exists() {
+                continue
+            }
+
             let mut src = PathSource::new(&path, &self.source_id, self.config);
             src.update()?;
             let pkg = src.root_package()?;
 
             let cksum_file = path.join(".cargo-checksum.json");
-            let cksum = paths::read(&path.join(cksum_file)).chain_error(|| {
-                human(format!("failed to load checksum `.cargo-checksum.json` \
-                               of {} v{}",
-                              pkg.package_id().name(),
-                              pkg.package_id().version()))
+            let cksum = paths::read(&path.join(cksum_file)).chain_err(|| {
+                format!("failed to load checksum `.cargo-checksum.json` \
+                         of {} v{}",
+                        pkg.package_id().name(),
+                        pkg.package_id().version())
 
             })?;
-            let cksum: Checksum = serde_json::from_str(&cksum).chain_error(|| {
-                human(format!("failed to decode `.cargo-checksum.json` of \
-                               {} v{}",
-                              pkg.package_id().name(),
-                              pkg.package_id().version()))
+            let cksum: Checksum = serde_json::from_str(&cksum).chain_err(|| {
+                format!("failed to decode `.cargo-checksum.json` of \
+                         {} v{}",
+                        pkg.package_id().name(),
+                        pkg.package_id().version())
             })?;
 
             let mut manifest = pkg.manifest().clone();
-            let summary = manifest.summary().clone();
-            manifest.set_summary(summary.set_checksum(cksum.package.clone()));
+            let mut summary = manifest.summary().clone();
+            if let Some(ref package) = cksum.package {
+                summary = summary.set_checksum(package.clone());
+            }
+            manifest.set_summary(summary);
             let pkg = Package::new(manifest, pkg.manifest_path());
             self.packages.insert(pkg.package_id().clone(), (pkg, cksum));
         }
@@ -111,8 +143,8 @@ impl<'cfg> Source for DirectorySource<'cfg> {
     }
 
     fn download(&mut self, id: &PackageId) -> CargoResult<Package> {
-        self.packages.get(id).map(|p| &p.0).cloned().chain_error(|| {
-            human(format!("failed to find package with id: {}", id))
+        self.packages.get(id).map(|p| &p.0).cloned().ok_or_else(|| {
+            format_err!("failed to find package with id: {}", id)
         })
     }
 
@@ -140,12 +172,12 @@ impl<'cfg> Source for DirectorySource<'cfg> {
                         n => h.update(&buf[..n]),
                     }
                 }
-            }).chain_error(|| {
-                human(format!("failed to calculate checksum of: {}",
-                              file.display()))
+            })().chain_err(|| {
+                format!("failed to calculate checksum of: {}",
+                        file.display())
             })?;
 
-            let actual = h.finish().to_hex();
+            let actual = hex::encode(h.finish());
             if &*actual != cksum {
                 bail!("\
                     the listed checksum of `{}` has changed:\n\
