@@ -1,15 +1,18 @@
 use std::env;
 use std::fmt;
 use std::fs::{self, File};
+use std::mem;
 use std::path::{Path, PathBuf};
+use std::process::Command;
 
+use curl::easy::{Easy, List};
 use git2::{self, ObjectType};
 use serde::ser::{self, Serialize};
 use url::Url;
 
 use core::GitReference;
-use util::{ToUrl, internal, Config, network};
-use util::errors::{CargoResult, CargoResultExt, CargoError};
+use util::{ToUrl, internal, Config, network, Progress};
+use util::errors::{CargoResult, CargoResultExt, Internal};
 
 #[derive(PartialEq, Clone, Debug)]
 pub struct GitRevision(git2::Oid);
@@ -41,16 +44,16 @@ impl GitShortID {
     }
 }
 
-/// GitRemote represents a remote repository. It gets cloned into a local
-/// GitDatabase.
+/// `GitRemote` represents a remote repository. It gets cloned into a local
+/// `GitDatabase`.
 #[derive(PartialEq, Clone, Debug, Serialize)]
 pub struct GitRemote {
     #[serde(serialize_with = "serialize_str")]
     url: Url,
 }
 
-/// GitDatabase is a local clone of a remote repository's database. Multiple
-/// GitCheckouts can be cloned from this GitDatabase.
+/// `GitDatabase` is a local clone of a remote repository's database. Multiple
+/// `GitCheckouts` can be cloned from this `GitDatabase`.
 #[derive(Serialize)]
 pub struct GitDatabase {
     remote: GitRemote,
@@ -59,9 +62,9 @@ pub struct GitDatabase {
     repo: git2::Repository,
 }
 
-/// GitCheckout is a local checkout of a particular revision. Calling
+/// `GitCheckout` is a local checkout of a particular revision. Calling
 /// `clone_into` with a reference will resolve the reference into a revision,
-/// and return a CargoError if no revision for that reference was found.
+/// and return a `CargoError` if no revision for that reference was found.
 #[derive(Serialize)]
 pub struct GitCheckout<'a> {
     database: &'a GitDatabase,
@@ -90,8 +93,8 @@ impl GitRemote {
 
     pub fn checkout(&self, into: &Path, cargo_config: &Config) -> CargoResult<GitDatabase> {
         let repo = match git2::Repository::open(into) {
-            Ok(repo) => {
-                self.fetch_into(&repo, cargo_config).chain_err(|| {
+            Ok(mut repo) => {
+                self.fetch_into(&mut repo, cargo_config).chain_err(|| {
                     format!("failed to fetch into {}", into.display())
                 })?;
                 repo
@@ -119,43 +122,37 @@ impl GitRemote {
         })
     }
 
-    fn fetch_into(&self, dst: &git2::Repository, cargo_config: &Config) -> CargoResult<()> {
+    fn fetch_into(&self, dst: &mut git2::Repository, cargo_config: &Config) -> CargoResult<()> {
         // Create a local anonymous remote in the repository to fetch the url
-        let url = self.url.to_string();
         let refspec = "refs/heads/*:refs/heads/*";
-        fetch(dst, &url, refspec, cargo_config)
+        fetch(dst, &self.url, refspec, cargo_config)
     }
 
     fn clone_into(&self, dst: &Path, cargo_config: &Config) -> CargoResult<git2::Repository> {
-        let url = self.url.to_string();
         if fs::metadata(&dst).is_ok() {
             fs::remove_dir_all(dst)?;
         }
         fs::create_dir_all(dst)?;
-        let repo = git2::Repository::init_bare(dst)?;
-        fetch(&repo, &url, "refs/heads/*:refs/heads/*", cargo_config)?;
+        let mut repo = git2::Repository::init_bare(dst)?;
+        fetch(&mut repo, &self.url, "refs/heads/*:refs/heads/*", cargo_config)?;
         Ok(repo)
     }
 }
 
 impl GitDatabase {
-    fn path(&self) -> &Path {
-        &self.path
-    }
-
     pub fn copy_to(&self, rev: GitRevision, dest: &Path, cargo_config: &Config)
                    -> CargoResult<GitCheckout> {
         let checkout = match git2::Repository::open(dest) {
             Ok(repo) => {
-                let checkout = GitCheckout::new(dest, self, rev, repo);
+                let mut checkout = GitCheckout::new(dest, self, rev, repo);
                 if !checkout.is_fresh() {
                     checkout.fetch(cargo_config)?;
-                    checkout.reset()?;
+                    checkout.reset(cargo_config)?;
                     assert!(checkout.is_fresh());
                 }
                 checkout
             }
-            Err(..) => GitCheckout::clone_into(dest, self, rev)?,
+            Err(..) => GitCheckout::clone_into(dest, self, rev, cargo_config)?,
         };
         checkout.update_submodules(cargo_config)?;
         Ok(checkout)
@@ -178,7 +175,7 @@ impl GitDatabase {
                 (|| {
                     let b = self.repo.find_branch(s, git2::BranchType::Local)?;
                     b.get().target().ok_or_else(|| {
-                        CargoError::from(format!("branch `{}` did not have a target", s))
+                        format_err!("branch `{}` did not have a target", s)
                     })
                 })().chain_err(|| {
                     format!("failed to find branch `{}`", s)
@@ -219,59 +216,76 @@ impl<'a> GitCheckout<'a> {
         }
     }
 
-    fn clone_into(into: &Path, database: &'a GitDatabase,
-                  revision: GitRevision)
+    fn clone_into(into: &Path,
+                  database: &'a GitDatabase,
+                  revision: GitRevision,
+                  config: &Config)
                   -> CargoResult<GitCheckout<'a>>
     {
-        let repo = GitCheckout::clone_repo(database.path(), into)?;
-        let checkout = GitCheckout::new(into, database, revision, repo);
-        checkout.reset()?;
-        Ok(checkout)
-    }
-
-    fn clone_repo(source: &Path, into: &Path) -> CargoResult<git2::Repository> {
         let dirname = into.parent().unwrap();
-
         fs::create_dir_all(&dirname).chain_err(|| {
             format!("Couldn't mkdir {}", dirname.display())
         })?;
-
-        if fs::metadata(&into).is_ok() {
+        if into.exists() {
             fs::remove_dir_all(into).chain_err(|| {
                 format!("Couldn't rmdir {}", into.display())
             })?;
         }
 
-        let url = source.to_url()?;
-        let url = url.to_string();
-        let repo = git2::Repository::clone(&url, into)            
-            .chain_err(|| {
-                internal(format!("failed to clone {} into {}", source.display(),
-                             into.display()))
+        // we're doing a local filesystem-to-filesystem clone so there should
+        // be no need to respect global configuration options, so pass in
+        // an empty instance of `git2::Config` below.
+        let git_config = git2::Config::new()?;
+
+        // Clone the repository, but make sure we use the "local" option in
+        // libgit2 which will attempt to use hardlinks to set up the database.
+        // This should speed up the clone operation quite a bit if it works.
+        //
+        // Note that we still use the same fetch options because while we don't
+        // need authentication information we may want progress bars and such.
+        let url = database.path.to_url()?;
+        let mut repo = None;
+        with_fetch_options(&git_config, &url, config, &mut |fopts| {
+            let mut checkout = git2::build::CheckoutBuilder::new();
+            checkout.dry_run(); // we'll do this below during a `reset`
+
+            let r = git2::build::RepoBuilder::new()
+                // use hard links and/or copy the database, we're doing a
+                // filesystem clone so this'll speed things up quite a bit.
+                .clone_local(git2::build::CloneLocal::Local)
+                .with_checkout(checkout)
+                .fetch_options(fopts)
+                // .remote_create(|repo, _name, url| repo.remote_anonymous(url))
+                .clone(url.as_str(), into)?;
+            repo = Some(r);
+            Ok(())
         })?;
-        Ok(repo)
+        let repo = repo.unwrap();
+
+        let checkout = GitCheckout::new(into, database, revision, repo);
+        checkout.reset(config)?;
+        Ok(checkout)
     }
 
     fn is_fresh(&self) -> bool {
         match self.repo.revparse_single("HEAD") {
             Ok(ref head) if head.id() == self.revision.0 => {
                 // See comments in reset() for why we check this
-                fs::metadata(self.location.join(".cargo-ok")).is_ok()
+                self.location.join(".cargo-ok").exists()
             }
             _ => false,
         }
     }
 
-    fn fetch(&self, cargo_config: &Config) -> CargoResult<()> {
+    fn fetch(&mut self, cargo_config: &Config) -> CargoResult<()> {
         info!("fetch {}", self.repo.path().display());
         let url = self.database.path.to_url()?;
-        let url = url.to_string();
         let refspec = "refs/heads/*:refs/heads/*";
-        fetch(&self.repo, &url, refspec, cargo_config)?;
+        fetch(&mut self.repo, &url, refspec, cargo_config)?;
         Ok(())
     }
 
-    fn reset(&self) -> CargoResult<()> {
+    fn reset(&self, config: &Config) -> CargoResult<()> {
         // If we're interrupted while performing this reset (e.g. we die because
         // of a signal) Cargo needs to be sure to try to check out this repo
         // again on the next go-round.
@@ -284,7 +298,7 @@ impl<'a> GitCheckout<'a> {
         let _ = fs::remove_file(&ok_file);
         info!("reset {} to {}", self.repo.path().display(), self.revision);
         let object = self.repo.find_object(self.revision.0, None)?;
-        self.repo.reset(&object, git2::ResetType::Hard, None)?;
+        reset(&self.repo, &object, config)?;
         File::create(ok_file)?;
         Ok(())
     }
@@ -295,13 +309,12 @@ impl<'a> GitCheckout<'a> {
         fn update_submodules(repo: &git2::Repository, cargo_config: &Config) -> CargoResult<()> {
             info!("update submodules for: {:?}", repo.workdir().unwrap());
 
-            for mut child in repo.submodules()?.into_iter() {
+            for mut child in repo.submodules()? {
                 update_submodule(repo, &mut child, cargo_config)
-                    .map_err(CargoError::into_internal)
                     .chain_err(|| {
                         format!("failed to update submodule `{}`",
                                 child.name().unwrap_or(""))
-                })?;
+                    })?;
             }
             Ok(())
         }
@@ -323,35 +336,36 @@ impl<'a> GitCheckout<'a> {
 
             // If the submodule hasn't been checked out yet, we need to
             // clone it. If it has been checked out and the head is the same
-            // as the submodule's head, then we can bail out and go to the
-            // next submodule.
+            // as the submodule's head, then we can skip an update and keep
+            // recursing.
             let head_and_repo = child.open().and_then(|repo| {
                 let target = repo.head()?.target();
                 Ok((target, repo))
             });
-            let repo = match head_and_repo {
+            let mut repo = match head_and_repo {
                 Ok((head, repo)) => {
                     if child.head_id() == head {
-                        return Ok(())
+                        return update_submodules(&repo, cargo_config)
                     }
                     repo
                 }
                 Err(..) => {
                     let path = parent.workdir().unwrap().join(child.path());
                     let _ = fs::remove_dir_all(&path);
-                    git2::Repository::clone(url, &path)?
+                    git2::Repository::init(&path)?
                 }
             };
 
             // Fetch data from origin and reset to the head commit
             let refspec = "refs/heads/*:refs/heads/*";
-            fetch(&repo, url, refspec, cargo_config).chain_err(|| {
+            let url = url.to_url()?;
+            fetch(&mut repo, &url, refspec, cargo_config).chain_err(|| {
                 internal(format!("failed to fetch submodule `{}` from {}",
                                  child.name().unwrap_or(""), url))
             })?;
 
-            repo.find_object(head, None)
-                .and_then(|obj| { repo.reset(&obj, git2::ResetType::Hard, None)})?;
+            let obj = repo.find_object(head, None)?;
+            reset(&repo, &obj, cargo_config)?;
             update_submodules(&repo, cargo_config)
         }
     }
@@ -532,7 +546,7 @@ fn with_authentication<T, F>(url: &str, cfg: &git2::Config, mut f: F)
     // In the case of an authentication failure (where we tried something) then
     // we try to give a more helpful error message about precisely what we
     // tried.
-    res.map_err(CargoError::from).map_err(|e| e.into_internal()).chain_err(|| {
+    let res = res.map_err(Internal::new).chain_err(|| {
         let mut msg = "failed to authenticate when downloading \
                        repository".to_string();
         if !ssh_agent_attempts.is_empty() {
@@ -554,32 +568,211 @@ fn with_authentication<T, F>(url: &str, cfg: &git2::Config, mut f: F)
             }
         }
         msg
-    })
+    })?;
+    Ok(res)
 }
 
-pub fn fetch(repo: &git2::Repository,
-             url: &str,
-             refspec: &str,
-             config: &Config) -> CargoResult<()> {
-    if !config.network_allowed() {
-        bail!("attempting to update a git repository, but --frozen \
-               was specified")
-    }
+fn reset(repo: &git2::Repository,
+         obj: &git2::Object,
+         config: &Config) -> CargoResult<()> {
+    let mut pb = Progress::new("Checkout", config);
+    let mut opts = git2::build::CheckoutBuilder::new();
+    opts.progress(|_, cur, max| {
+        drop(pb.tick(cur, max));
+    });
+    repo.reset(obj, git2::ResetType::Hard, Some(&mut opts))?;
+    Ok(())
+}
 
-    with_authentication(url, &repo.config()?, |f| {
-        let mut cb = git2::RemoteCallbacks::new();
-        cb.credentials(f);
+pub fn with_fetch_options(git_config: &git2::Config,
+                          url: &Url,
+                          config: &Config,
+                          cb: &mut FnMut(git2::FetchOptions) -> CargoResult<()>)
+    -> CargoResult<()>
+{
+    let mut progress = Progress::new("Fetch", config);
+    network::with_retry(config, url, || {
+        with_authentication(url.as_str(), git_config, |f| {
+            let mut rcb = git2::RemoteCallbacks::new();
+            rcb.credentials(f);
 
-        // Create a local anonymous remote in the repository to fetch the url
-        let mut remote = repo.remote_anonymous(url)?;
-        let mut opts = git2::FetchOptions::new();
-        opts.remote_callbacks(cb)
-            .download_tags(git2::AutotagOption::All);
+            rcb.transfer_progress(|stats| {
+                progress.tick(stats.indexed_objects(), stats.total_objects()).is_ok()
+            });
 
-        network::with_retry(config, || {
-            remote.fetch(&[refspec], Some(&mut opts), None)
-                .map_err(CargoError::from)
+            // Create a local anonymous remote in the repository to fetch the
+            // url
+            let mut opts = git2::FetchOptions::new();
+            opts.remote_callbacks(rcb)
+                .download_tags(git2::AutotagOption::All);
+            cb(opts)
         })?;
         Ok(())
     })
 }
+
+pub fn fetch(repo: &mut git2::Repository,
+             url: &Url,
+             refspec: &str,
+             config: &Config) -> CargoResult<()> {
+    if config.frozen() {
+        bail!("attempting to update a git repository, but --frozen \
+               was specified")
+    }
+    if !config.network_allowed() {
+        bail!("can't update a git repository in the offline mode")
+    }
+
+    // If we're fetching from github, attempt github's special fast path for
+    // testing if we've already got an up-to-date copy of the repository
+    if url.host_str() == Some("github.com") {
+        if let Ok(oid) = repo.refname_to_id("refs/remotes/origin/master") {
+            let mut handle = config.http()?.borrow_mut();
+            debug!("attempting github fast path for {}", url);
+            if github_up_to_date(&mut handle, url, &oid) {
+                return Ok(())
+            } else {
+                debug!("fast path failed, falling back to a git fetch");
+            }
+        }
+    }
+
+    // We reuse repositories quite a lot, so before we go through and update the
+    // repo check to see if it's a little too old and could benefit from a gc.
+    // In theory this shouldn't be too too expensive compared to the network
+    // request we're about to issue.
+    maybe_gc_repo(repo)?;
+
+    debug!("doing a fetch for {}", url);
+    with_fetch_options(&repo.config()?, url, config, &mut |mut opts| {
+        debug!("initiating fetch of {} from {}", refspec, url);
+        let mut remote = repo.remote_anonymous(url.as_str())?;
+        remote.fetch(&[refspec], Some(&mut opts), None)?;
+        Ok(())
+    })
+}
+
+/// Cargo has a bunch of long-lived git repositories in its global cache and
+/// some, like the index, are updated very frequently. Right now each update
+/// creates a new "pack file" inside the git database, and over time this can
+/// cause bad performance and bad current behavior in libgit2.
+///
+/// One pathological use case today is where libgit2 opens hundreds of file
+/// descriptors, getting us dangerously close to blowing out the OS limits of
+/// how many fds we can have open. This is detailed in #4403.
+///
+/// To try to combat this problem we attempt a `git gc` here. Note, though, that
+/// we may not even have `git` installed on the system! As a result we
+/// opportunistically try a `git gc` when the pack directory looks too big, and
+/// failing that we just blow away the repository and start over.
+fn maybe_gc_repo(repo: &mut git2::Repository) -> CargoResult<()> {
+    // Here we arbitrarily declare that if you have more than 100 files in your
+    // `pack` folder that we need to do a gc.
+    let entries = match repo.path().join("objects/pack").read_dir() {
+        Ok(e) => e.count(),
+        Err(_) => {
+            debug!("skipping gc as pack dir appears gone");
+            return Ok(())
+        }
+    };
+    let max = env::var("__CARGO_PACKFILE_LIMIT").ok()
+        .and_then(|s| s.parse::<usize>().ok())
+        .unwrap_or(100);
+    if entries < max {
+        debug!("skipping gc as there's only {} pack files", entries);
+        return Ok(())
+    }
+
+    // First up, try a literal `git gc` by shelling out to git. This is pretty
+    // likely to fail though as we may not have `git` installed. Note that
+    // libgit2 doesn't currently implement the gc operation, so there's no
+    // equivalent there.
+    match Command::new("git").arg("gc").current_dir(repo.path()).output() {
+        Ok(out) => {
+            debug!("git-gc status: {}\n\nstdout ---\n{}\nstderr ---\n{}",
+                   out.status,
+                   String::from_utf8_lossy(&out.stdout),
+                   String::from_utf8_lossy(&out.stderr));
+            if out.status.success() {
+                let new = git2::Repository::open(repo.path())?;
+                mem::replace(repo, new);
+                return Ok(())
+            }
+        }
+        Err(e) => debug!("git-gc failed to spawn: {}", e),
+    }
+
+    // Alright all else failed, let's start over.
+    //
+    // Here we want to drop the current repository object pointed to by `repo`,
+    // so we initialize temporary repository in a sub-folder, blow away the
+    // existing git folder, and then recreate the git repo. Finally we blow away
+    // the `tmp` folder we allocated.
+    let path = repo.path().to_path_buf();
+    let tmp = path.join("tmp");
+    mem::replace(repo, git2::Repository::init(&tmp)?);
+    for entry in path.read_dir()? {
+        let entry = entry?;
+        if entry.file_name().to_str() == Some("tmp") {
+            continue
+        }
+        let path = entry.path();
+        drop(fs::remove_file(&path).or_else(|_| fs::remove_dir_all(&path)));
+    }
+    if repo.is_bare() {
+        mem::replace(repo, git2::Repository::init_bare(path)?);
+    } else {
+        mem::replace(repo, git2::Repository::init(path)?);
+    }
+    fs::remove_dir_all(&tmp).chain_err(|| {
+        format!("failed to remove {:?}", tmp)
+    })?;
+    Ok(())
+}
+
+/// Updating the index is done pretty regularly so we want it to be as fast as
+/// possible. For registries hosted on github (like the crates.io index) there's
+/// a fast path available to use [1] to tell us that there's no updates to be
+/// made.
+///
+/// This function will attempt to hit that fast path and verify that the `oid`
+/// is actually the current `master` branch of the repository. If `true` is
+/// returned then no update needs to be performed, but if `false` is returned
+/// then the standard update logic still needs to happen.
+///
+/// [1]: https://developer.github.com/v3/repos/commits/#get-the-sha-1-of-a-commit-reference
+///
+/// Note that this function should never cause an actual failure because it's
+/// just a fast path. As a result all errors are ignored in this function and we
+/// just return a `bool`. Any real errors will be reported through the normal
+/// update path above.
+fn github_up_to_date(handle: &mut Easy, url: &Url, oid: &git2::Oid) -> bool {
+    macro_rules! try {
+        ($e:expr) => (match $e {
+            Some(e) => e,
+            None => return false,
+        })
+    }
+
+    // This expects github urls in the form `github.com/user/repo` and nothing
+    // else
+    let mut pieces = try!(url.path_segments());
+    let username = try!(pieces.next());
+    let repo = try!(pieces.next());
+    if pieces.next().is_some() {
+        return false
+    }
+
+    let url = format!("https://api.github.com/repos/{}/{}/commits/master",
+                      username, repo);
+    try!(handle.get(true).ok());
+    try!(handle.url(&url).ok());
+    try!(handle.useragent("cargo").ok());
+    let mut headers = List::new();
+    try!(headers.append("Accept: application/vnd.github.3.sha").ok());
+    try!(headers.append(&format!("If-None-Match: \"{}\"", oid)).ok());
+    try!(handle.http_headers(headers).ok());
+    try!(handle.perform().ok());
+
+    try!(handle.response_code().ok()) == 304
+}
