@@ -1,14 +1,15 @@
 use std::fs::{self, File, OpenOptions};
-use std::io::*;
+use std::io::{Seek, Read, Write, SeekFrom};
 use std::io;
 use std::path::{Path, PathBuf, Display};
 
-use term::color::CYAN;
+use termcolor::Color::Cyan;
 use fs2::{FileExt, lock_contended_error};
 #[allow(unused_imports)]
 use libc;
 
-use util::{CargoResult, ChainError, Config, human};
+use util::Config;
+use util::errors::{CargoResult, CargoResultExt, CargoError};
 
 pub struct FileLock {
     f: Option<File>,
@@ -139,7 +140,7 @@ impl Filesystem {
     /// Handles errors where other Cargo processes are also attempting to
     /// concurrently create this directory.
     pub fn create_dir(&self) -> io::Result<()> {
-        return create_dir_all(&self.root);
+        create_dir_all(&self.root)
     }
 
     /// Returns an adaptor that can be used to print the path of this
@@ -211,8 +212,8 @@ impl Filesystem {
             } else {
                 Err(e)
             }
-        }).chain_error(|| {
-            human(format!("failed to open: {}", path.display()))
+        }).chain_err(|| {
+            format!("failed to open: {}", path.display())
         })?;
         match state {
             State::Exclusive => {
@@ -232,6 +233,18 @@ impl Filesystem {
     }
 }
 
+impl PartialEq<Path> for Filesystem {
+    fn eq(&self, other: &Path) -> bool {
+        self.root == other
+    }
+}
+
+impl PartialEq<Filesystem> for Path {
+    fn eq(&self, other: &Filesystem) -> bool {
+        self == other.root
+    }
+}
+
 /// Acquires a lock on a file in a "nice" manner.
 ///
 /// Almost all long-running blocking actions in Cargo have a status message
@@ -270,10 +283,11 @@ fn acquire(config: &Config,
     match try() {
         Ok(()) => return Ok(()),
 
-        // Like above, where we ignore file locking on NFS mounts on Linux, we
-        // do the same on OSX here. Note that ENOTSUP is an OSX_specific
-        // constant.
-        #[cfg(target_os = "macos")]
+        // In addition to ignoring NFS which is commonly not working we also
+        // just ignore locking on filesystems that look like they don't
+        // implement file locking. We detect that here via the return value of
+        // locking (e.g. inspecting errno).
+        #[cfg(unix)]
         Err(ref e) if e.raw_os_error() == Some(libc::ENOTSUP) => return Ok(()),
 
         #[cfg(target_os = "linux")]
@@ -281,18 +295,19 @@ fn acquire(config: &Config,
 
         Err(e) => {
             if e.raw_os_error() != lock_contended_error().raw_os_error() {
-                return Err(human(e)).chain_error(|| {
-                    human(format!("failed to lock file: {}", path.display()))
-                })
+                let e = CargoError::from(e);
+                let cx = format!("failed to lock file: {}", path.display());
+                return Err(e.context(cx).into())
             }
         }
     }
     let msg = format!("waiting for file lock on {}", msg);
-    config.shell().err().say_status("Blocking", &msg, CYAN, true)?;
+    config.shell().status_with_color("Blocking", &msg, Cyan)?;
 
-    return block().chain_error(|| {
-        human(format!("failed to lock file: {}", path.display()))
-    });
+    block().chain_err(|| {
+        format!("failed to lock file: {}", path.display())
+    })?;
+    return Ok(());
 
     #[cfg(all(target_os = "linux", not(target_env = "musl")))]
     fn is_on_nfs_mount(path: &Path) -> bool {
@@ -321,7 +336,7 @@ fn acquire(config: &Config,
 
 fn create_dir_all(path: &Path) -> io::Result<()> {
     match create_dir(path) {
-        Ok(()) => return Ok(()),
+        Ok(()) => Ok(()),
         Err(e) => {
             if e.kind() == io::ErrorKind::NotFound {
                 if let Some(p) = path.parent() {
