@@ -1,9 +1,10 @@
+#![allow(deprecated)]
+
+use hex;
 use std::hash::{Hasher, Hash, SipHasher};
 
-use rustc_serialize::hex::ToHex;
-
 pub fn to_hex(num: u64) -> String {
-    [
+    hex::encode(&[
         (num >>  0) as u8,
         (num >>  8) as u8,
         (num >> 16) as u8,
@@ -12,7 +13,7 @@ pub fn to_hex(num: u64) -> String {
         (num >> 40) as u8,
         (num >> 48) as u8,
         (num >> 56) as u8,
-    ].to_hex()
+    ])
 }
 
 pub fn hash_u64<H: Hash>(hashable: &H) -> u64 {
