@@ -34,9 +34,9 @@ fn http_auth_offered() {
         let mut conn = BufStream::new(server.accept().unwrap().0);
         let req = headers(&mut conn);
         let user_agent = if cfg!(windows) {
-            "User-Agent: git/1.0 (libgit2 0.25.0)"
+            "User-Agent: git/1.0 (libgit2 0.26.0)"
         } else {
-            "User-Agent: git/2.0 (libgit2 0.25.0)"
+            "User-Agent: git/2.0 (libgit2 0.26.0)"
         };
         conn.write_all(b"\
             HTTP/1.1 401 Unauthorized\r\n\
@@ -77,9 +77,10 @@ fn http_auth_offered() {
                 println!("username=foo");
                 println!("password=bar");
             }
-        "#);
+        "#)
+        .build();
 
-    assert_that(script.cargo_process("build").arg("-v"),
+    assert_that(script.cargo("build").arg("-v"),
                 execs().with_status(0));
     let script = script.bin("script");
 
@@ -102,9 +103,10 @@ fn http_auth_offered() {
         .file(".cargo/config","\
         [net]
         retry = 0
-        ");
+        ")
+        .build();
 
-    assert_that(p.cargo_process("build"),
+    assert_that(p.cargo("build"),
                 execs().with_status(101).with_stderr(&format!("\
 [UPDATING] git repository `http://{addr}/foo/bar`
 [ERROR] failed to load source for a dependency on `bar`
@@ -152,9 +154,10 @@ fn https_something_happens() {
         .file(".cargo/config","\
         [net]
         retry = 0
-        ");
+        ")
+        .build();
 
-    assert_that(p.cargo_process("build").arg("-v"),
+    assert_that(p.cargo("build").arg("-v"),
                 execs().with_status(101).with_stderr_contains(&format!("\
 [UPDATING] git repository `https://{addr}/foo/bar`
 ", addr = addr))
@@ -163,14 +166,14 @@ Caused by:
   {errmsg}
 ",
         errmsg = if cfg!(windows) {
-            "[[..]] failed to send request: [..]\n"
+            "[..]failed to send request: [..]"
         } else if cfg!(target_os = "macos") {
             // OSX is difficult to tests as some builds may use
             // Security.framework and others may use OpenSSL. In that case let's
             // just not verify the error message here.
             "[..]"
         } else {
-            "[..] SSL error: [..]"
+            "[..]SSL error: [..]"
         })));
 
     t.join().ok().unwrap();
@@ -195,15 +198,16 @@ fn ssh_something_happens() {
             [dependencies.bar]
             git = "ssh://127.0.0.1:{}/foo/bar"
         "#, addr.port()))
-        .file("src/main.rs", "");
+        .file("src/main.rs", "")
+        .build();
 
-    assert_that(p.cargo_process("build").arg("-v"),
+    assert_that(p.cargo("build").arg("-v"),
                 execs().with_status(101).with_stderr_contains(&format!("\
 [UPDATING] git repository `ssh://{addr}/foo/bar`
 ", addr = addr))
                     .with_stderr_contains("\
 Caused by:
-  [[..]] Failed to start SSH session: Failed getting banner
+  [..]failed to start SSH session: Failed getting banner[..]
 "));
     t.join().ok().unwrap();
 }
