@@ -1,5 +1,7 @@
 extern crate cargotest;
 extern crate libc;
+#[cfg(windows)]
+extern crate winapi;
 
 use std::fs;
 use std::io::{self, Read};
@@ -15,7 +17,7 @@ fn enabled() -> bool {
     true
 }
 
-// On Windows suport for these tests is only enabled through the usage of job
+// On Windows support for these tests is only enabled through the usage of job
 // objects. Support for nested job objects, however, was added in recent-ish
 // versions of Windows, so this test may not always be able to succeed.
 //
@@ -23,16 +25,16 @@ fn enabled() -> bool {
 // can succeed or not.
 #[cfg(windows)]
 fn enabled() -> bool {
-    extern crate kernel32;
-    extern crate winapi;
+    use winapi::um::{handleapi, jobapi, jobapi2, processthreadsapi};
+
     unsafe {
         // If we're not currently in a job, then we can definitely run these
         // tests.
-        let me = kernel32::GetCurrentProcess();
+        let me = processthreadsapi::GetCurrentProcess();
         let mut ret = 0;
-        let r = kernel32::IsProcessInJob(me, 0 as *mut _, &mut ret);
+        let r = jobapi::IsProcessInJob(me, 0 as *mut _, &mut ret);
         assert!(r != 0);
-        if ret == winapi::FALSE {
+        if ret == winapi::shared::minwindef::FALSE {
             return true
         }
 
@@ -42,10 +44,10 @@ fn enabled() -> bool {
         //
         // If we can't be added to a nested job, then these tests will
         // definitely fail, and there's not much we can do about that.
-        let job = kernel32::CreateJobObjectW(0 as *mut _, 0 as *const _);
+        let job = jobapi2::CreateJobObjectW(0 as *mut _, 0 as *const _);
         assert!(!job.is_null());
-        let r = kernel32::AssignProcessToJobObject(job, me);
-        kernel32::CloseHandle(job);
+        let r = jobapi2::AssignProcessToJobObject(job, me);
+        handleapi::CloseHandle(job);
         r != 0
     }
 }
@@ -77,8 +79,8 @@ fn ctrl_c_kills_everyone() {
                 let _ = socket.read(&mut [0; 10]);
                 panic!("that read should never return");
             }}
-        "#, addr));
-    p.build();
+        "#, addr))
+        .build();
 
     let mut cargo = p.cargo("build").build_command();
     cargo.stdin(Stdio::piped())
