@@ -1,19 +1,20 @@
 #[macro_use]
 extern crate cargotest;
 extern crate hamcrest;
-extern crate rustc_serialize;
+#[macro_use]
+extern crate serde_derive;
+extern crate serde_json;
 
 use std::collections::HashMap;
 use std::fs::{self, File};
 use std::io::prelude::*;
 use std::str;
 
-use rustc_serialize::json;
-
 use cargotest::cargo_process;
-use cargotest::support::{project, execs, ProjectBuilder};
+use cargotest::support::git;
 use cargotest::support::paths;
 use cargotest::support::registry::{Package, cksum};
+use cargotest::support::{project, execs, ProjectBuilder};
 use hamcrest::assert_that;
 
 fn setup() {
@@ -33,9 +34,9 @@ struct VendorPackage {
     cksum: Checksum,
 }
 
-#[derive(RustcEncodable)]
+#[derive(Serialize)]
 struct Checksum {
-    package: String,
+    package: Option<String>,
     files: HashMap<String, String>,
 }
 
@@ -44,7 +45,7 @@ impl VendorPackage {
         VendorPackage {
             p: Some(project(&format!("index/{}", name))),
             cksum: Checksum {
-                package: String::new(),
+                package: Some(String::new()),
                 files: HashMap::new(),
             },
         }
@@ -56,11 +57,16 @@ impl VendorPackage {
         self
     }
 
+    fn disable_checksum(&mut self) -> &mut VendorPackage {
+        self.cksum.package = None;
+        self
+    }
+
     fn build(&mut self) {
         let p = self.p.take().unwrap();
-        let json = json::encode(&self.cksum).unwrap();
+        let json = serde_json::to_string(&self.cksum).unwrap();
         let p = p.file(".cargo-checksum.json", &json);
-        p.build();
+        let _ = p.build();
     }
 }
 
@@ -94,8 +100,8 @@ fn simple() {
             pub fn bar() {
                 foo::foo();
             }
-        "#);
-    p.build();
+        "#)
+        .build();
 
     assert_that(p.cargo("build"),
                 execs().with_status(0).with_stderr("\
@@ -189,9 +195,9 @@ fn simple_install_fail() {
 error: failed to compile `bar v0.1.0`, intermediate artifacts can be found at `[..]`
 
 Caused by:
-  no matching package named `baz` found (required by `bar`)
-location searched: registry https://github.com/rust-lang/crates.io-index
-version required: ^9.8.7
+  no matching package named `baz` found
+location searched: registry `https://github.com/rust-lang/crates.io-index`
+required by package `bar v0.1.0`
 "));
 }
 
@@ -247,7 +253,7 @@ warning: be sure to add `[..]` to your PATH to be able to run the installed bina
 fn not_there() {
     setup();
 
-    project("index").build();
+    let _ = project("index").build();
 
     let p = project("bar")
         .file("Cargo.toml", r#"
@@ -265,14 +271,14 @@ fn not_there() {
             pub fn bar() {
                 foo::foo();
             }
-        "#);
-    p.build();
+        "#)
+        .build();
 
     assert_that(p.cargo("build"),
                 execs().with_status(101).with_stderr("\
-error: no matching package named `foo` found (required by `bar`)
+error: no matching package named `foo` found
 location searched: [..]
-version required: ^0.1.0
+required by package `bar v0.1.0 ([..])`
 "));
 }
 
@@ -318,8 +324,8 @@ fn multiple() {
             pub fn bar() {
                 foo::foo();
             }
-        "#);
-    p.build();
+        "#)
+        .build();
 
     assert_that(p.cargo("build"),
                 execs().with_status(0).with_stderr("\
@@ -347,8 +353,8 @@ fn crates_io_then_directory() {
             pub fn bar() {
                 foo::foo();
             }
-        "#);
-    p.build();
+        "#)
+        .build();
 
     let cksum = Package::new("foo", "0.1.0")
                         .file("src/lib.rs", "pub fn foo() -> u32 { 0 }")
@@ -373,7 +379,7 @@ fn crates_io_then_directory() {
         authors = []
     "#);
     v.file("src/lib.rs", "pub fn foo() -> u32 { 1 }");
-    v.cksum.package = cksum;
+    v.cksum.package = Some(cksum);
     v.build();
 
     assert_that(p.cargo("build"),
@@ -396,8 +402,8 @@ fn crates_io_then_bad_checksum() {
             [dependencies]
             foo = "0.1.0"
         "#)
-        .file("src/lib.rs", "");
-    p.build();
+        .file("src/lib.rs", "")
+        .build();
 
     Package::new("foo", "0.1.0").publish();
 
@@ -457,8 +463,8 @@ fn bad_file_checksum() {
             [dependencies]
             foo = "0.1.0"
         "#)
-        .file("src/lib.rs", "");
-    p.build();
+        .file("src/lib.rs", "")
+        .build();
 
     assert_that(p.cargo("build"),
                 execs().with_status(101).with_stderr("\
@@ -471,3 +477,191 @@ required then it is recommended that [replace] is used with a forked copy of \
 the source
 "));
 }
+
+#[test]
+fn only_dot_files_ok() {
+    setup();
+
+    VendorPackage::new("foo")
+        .file("Cargo.toml", r#"
+            [package]
+            name = "foo"
+            version = "0.1.0"
+            authors = []
+        "#)
+        .file("src/lib.rs", "")
+        .build();
+    VendorPackage::new("bar")
+        .file(".foo", "")
+        .build();
+
+    let p = project("bar")
+        .file("Cargo.toml", r#"
+            [package]
+            name = "bar"
+            version = "0.1.0"
+            authors = []
+
+            [dependencies]
+            foo = "0.1.0"
+        "#)
+        .file("src/lib.rs", "")
+        .build();
+
+    assert_that(p.cargo("build"), execs().with_status(0));
+}
+
+#[test]
+fn random_files_ok() {
+    setup();
+
+    VendorPackage::new("foo")
+        .file("Cargo.toml", r#"
+            [package]
+            name = "foo"
+            version = "0.1.0"
+            authors = []
+        "#)
+        .file("src/lib.rs", "")
+        .build();
+    VendorPackage::new("bar")
+        .file("foo", "")
+        .file("../test", "")
+        .build();
+
+    let p = project("bar")
+        .file("Cargo.toml", r#"
+            [package]
+            name = "bar"
+            version = "0.1.0"
+            authors = []
+
+            [dependencies]
+            foo = "0.1.0"
+        "#)
+        .file("src/lib.rs", "")
+        .build();
+
+    assert_that(p.cargo("build"), execs().with_status(0));
+}
+
+#[test]
+fn git_lock_file_doesnt_change() {
+
+    let git = git::new("git", |p| {
+        p.file("Cargo.toml", r#"
+            [project]
+            name = "git"
+            version = "0.5.0"
+            authors = []
+        "#)
+        .file("src/lib.rs", "")
+    }).unwrap();
+
+    VendorPackage::new("git")
+        .file("Cargo.toml", r#"
+            [package]
+            name = "git"
+            version = "0.5.0"
+            authors = []
+        "#)
+        .file("src/lib.rs", "")
+        .disable_checksum()
+        .build();
+
+    let p = project("bar")
+        .file("Cargo.toml", &format!(r#"
+            [package]
+            name = "foo"
+            version = "0.0.1"
+            authors = []
+
+            [dependencies]
+            git = {{ git = '{0}' }}
+        "#, git.url()))
+        .file("src/lib.rs", "")
+        .build();
+
+    assert_that(p.cargo("build"), execs().with_status(0));
+
+    let mut lock1 = String::new();
+    t!(t!(File::open(p.root().join("Cargo.lock"))).read_to_string(&mut lock1));
+
+    let root = paths::root();
+    t!(fs::create_dir(&root.join(".cargo")));
+    t!(t!(File::create(root.join(".cargo/config"))).write_all(&format!(r#"
+        [source.my-git-repo]
+        git = '{}'
+        replace-with = 'my-awesome-local-registry'
+
+        [source.my-awesome-local-registry]
+        directory = 'index'
+    "#, git.url()).as_bytes()));
+
+    assert_that(p.cargo("build"),
+                execs().with_status(0)
+                       .with_stderr("\
+[COMPILING] [..]
+[COMPILING] [..]
+[FINISHED] [..]
+"));
+
+    let mut lock2 = String::new();
+    t!(t!(File::open(p.root().join("Cargo.lock"))).read_to_string(&mut lock2));
+    assert!(lock1 == lock2, "lock files changed");
+}
+
+#[test]
+fn git_override_requires_lockfile() {
+    VendorPackage::new("git")
+        .file("Cargo.toml", r#"
+            [package]
+            name = "git"
+            version = "0.5.0"
+            authors = []
+        "#)
+        .file("src/lib.rs", "")
+        .disable_checksum()
+        .build();
+
+    let p = project("bar")
+        .file("Cargo.toml", r#"
+            [package]
+            name = "foo"
+            version = "0.0.1"
+            authors = []
+
+            [dependencies]
+            git = { git = 'https://example.com/' }
+        "#)
+        .file("src/lib.rs", "")
+        .build();
+
+    let root = paths::root();
+    t!(fs::create_dir(&root.join(".cargo")));
+    t!(t!(File::create(root.join(".cargo/config"))).write_all(br#"
+        [source.my-git-repo]
+        git = 'https://example.com/'
+        replace-with = 'my-awesome-local-registry'
+
+        [source.my-awesome-local-registry]
+        directory = 'index'
+    "#));
+
+    assert_that(p.cargo("build"),
+                execs().with_status(101)
+                       .with_stderr("\
+error: failed to load source for a dependency on `git`
+
+Caused by:
+  Unable to update [..]
+
+Caused by:
+  the source my-git-repo requires a lock file to be present first before it can be
+used against vendored source code
+
+remove the source replacement configuration, generate a lock file, and then
+restore the source replacement configuration to continue the build
+
+"));
+}
