@@ -8,10 +8,11 @@ use cargotest::support::{project, execs, basic_bin_manifest, basic_lib_manifest,
 #[test]
 fn cargo_metadata_simple() {
     let p = project("foo")
-            .file("src/foo.rs", "")
-            .file("Cargo.toml", &basic_bin_manifest("foo"));
+        .file("src/foo.rs", "")
+        .file("Cargo.toml", &basic_bin_manifest("foo"))
+        .build();
 
-    assert_that(p.cargo_process("metadata"), execs().with_json(r#"
+    assert_that(p.cargo("metadata"), execs().with_json(r#"
     {
         "packages": [
             {
@@ -49,7 +50,9 @@ fn cargo_metadata_simple() {
             ],
             "root": "foo 0.5.0 (path+file:[..]foo)"
         },
-        "version": 1
+        "target_directory": "[..]foo[/]target",
+        "version": 1,
+        "workspace_root": "[..][/]foo"
     }"#));
 }
 
@@ -57,8 +60,8 @@ fn cargo_metadata_simple() {
 fn cargo_metadata_warns_on_implicit_version() {
     let p = project("foo")
         .file("src/foo.rs", "")
-        .file("Cargo.toml", &basic_bin_manifest("foo"));
-    p.build();
+        .file("Cargo.toml", &basic_bin_manifest("foo"))
+        .build();
 
     assert_that(p.cargo("metadata"),
                 execs().with_stderr("\
@@ -71,17 +74,18 @@ fn cargo_metadata_warns_on_implicit_version() {
 #[test]
 fn library_with_several_crate_types() {
     let p = project("foo")
-            .file("src/lib.rs", "")
-            .file("Cargo.toml", r#"
+        .file("src/lib.rs", "")
+        .file("Cargo.toml", r#"
 [package]
 name = "foo"
 version = "0.5.0"
 
 [lib]
 crate-type = ["lib", "staticlib"]
-            "#);
+            "#)
+        .build();
 
-    assert_that(p.cargo_process("metadata"), execs().with_json(r#"
+    assert_that(p.cargo("metadata"), execs().with_json(r#"
     {
         "packages": [
             {
@@ -121,7 +125,9 @@ crate-type = ["lib", "staticlib"]
             ],
             "root": "foo 0.5.0 (path+file:[..]foo)"
         },
-        "version": 1
+        "target_directory": "[..]foo[/]target",
+        "version": 1,
+        "workspace_root": "[..][/]foo"
     }"#));
 }
 
@@ -142,11 +148,12 @@ fn cargo_metadata_with_deps_and_version() {
 
             [dependencies]
             bar = "*"
-        "#);
+        "#)
+        .build();
     Package::new("baz", "0.0.1").publish();
     Package::new("bar", "0.0.1").dep("baz", "0.0.1").publish();
 
-    assert_that(p.cargo_process("metadata")
+    assert_that(p.cargo("metadata")
                  .arg("-q")
                  .arg("--format-version").arg("1"),
                 execs().with_json(r#"
@@ -269,25 +276,28 @@ fn cargo_metadata_with_deps_and_version() {
             ],
             "root": "foo 0.5.0 (path+file:[..]foo)"
         },
-        "version": 1
+        "target_directory": "[..]foo[/]target",
+        "version": 1,
+        "workspace_root": "[..][/]foo"
     }"#));
 }
 
 #[test]
 fn example() {
     let p = project("foo")
-            .file("src/lib.rs", "")
-            .file("examples/ex.rs", "")
-            .file("Cargo.toml", r#"
+        .file("src/lib.rs", "")
+        .file("examples/ex.rs", "")
+        .file("Cargo.toml", r#"
 [package]
 name = "foo"
 version = "0.1.0"
 
 [[example]]
 name = "ex"
-            "#);
+            "#)
+        .build();
 
-    assert_that(p.cargo_process("metadata"), execs().with_json(r#"
+    assert_that(p.cargo("metadata"), execs().with_json(r#"
     {
         "packages": [
             {
@@ -329,16 +339,18 @@ name = "ex"
                 }
             ]
         },
-        "version": 1
+        "target_directory": "[..]foo[/]target",
+        "version": 1,
+        "workspace_root": "[..][/]foo"
     }"#));
 }
 
 #[test]
 fn example_lib() {
     let p = project("foo")
-            .file("src/lib.rs", "")
-            .file("examples/ex.rs", "")
-            .file("Cargo.toml", r#"
+        .file("src/lib.rs", "")
+        .file("examples/ex.rs", "")
+        .file("Cargo.toml", r#"
 [package]
 name = "foo"
 version = "0.1.0"
@@ -346,9 +358,10 @@ version = "0.1.0"
 [[example]]
 name = "ex"
 crate-type = ["rlib", "dylib"]
-            "#);
+            "#)
+        .build();
 
-    assert_that(p.cargo_process("metadata"), execs().with_json(r#"
+    assert_that(p.cargo("metadata"), execs().with_json(r#"
     {
         "packages": [
             {
@@ -390,7 +403,9 @@ crate-type = ["rlib", "dylib"]
                 }
             ]
         },
-        "version": 1
+        "target_directory": "[..]foo[/]target",
+        "version": 1,
+        "workspace_root": "[..][/]foo"
     }"#));
 }
 
@@ -404,9 +419,10 @@ fn workspace_metadata() {
         .file("bar/Cargo.toml", &basic_lib_manifest("bar"))
         .file("bar/src/lib.rs", "")
         .file("baz/Cargo.toml", &basic_lib_manifest("baz"))
-        .file("baz/src/lib.rs", "");
+        .file("baz/src/lib.rs", "")
+        .build();
 
-    assert_that(p.cargo_process("metadata"), execs().with_status(0).with_json(r#"
+    assert_that(p.cargo("metadata"), execs().with_status(0).with_json(r#"
     {
         "packages": [
             {
@@ -464,7 +480,9 @@ fn workspace_metadata() {
             ],
             "root": null
         },
-        "version": 1
+        "target_directory": "[..]foo[/]target",
+        "version": 1,
+        "workspace_root": "[..][/]foo"
     }"#))
 }
 
@@ -478,9 +496,10 @@ fn workspace_metadata_no_deps() {
         .file("bar/Cargo.toml", &basic_lib_manifest("bar"))
         .file("bar/src/lib.rs", "")
         .file("baz/Cargo.toml", &basic_lib_manifest("baz"))
-        .file("baz/src/lib.rs", "");
+        .file("baz/src/lib.rs", "")
+        .build();
 
-    assert_that(p.cargo_process("metadata").arg("--no-deps"), execs().with_status(0).with_json(r#"
+    assert_that(p.cargo("metadata").arg("--no-deps"), execs().with_status(0).with_json(r#"
     {
         "packages": [
             {
@@ -526,21 +545,24 @@ fn workspace_metadata_no_deps() {
         ],
         "workspace_members": ["baz 0.5.0 (path+file:[..]baz)", "bar 0.5.0 (path+file:[..]bar)"],
         "resolve": null,
-        "version": 1
+        "target_directory": "[..]foo[/]target",
+        "version": 1,
+        "workspace_root": "[..][/]foo"
     }"#))
 }
 
 #[test]
 fn cargo_metadata_with_invalid_manifest() {
     let p = project("foo")
-            .file("Cargo.toml", "");
+        .file("Cargo.toml", "")
+        .build();
 
-    assert_that(p.cargo_process("metadata").arg("--format-version").arg("1"),
+    assert_that(p.cargo("metadata").arg("--format-version").arg("1"),
                 execs().with_status(101).with_stderr("\
 [ERROR] failed to parse manifest at `[..]`
 
 Caused by:
-  no `package` or `project` section found."))
+  virtual manifests must be configured with [workspace]"))
 }
 
 const MANIFEST_OUTPUT: &'static str=
@@ -566,16 +588,19 @@ const MANIFEST_OUTPUT: &'static str=
     }],
     "workspace_members": [ "foo 0.5.0 (path+file:[..]foo)" ],
     "resolve": null,
-    "version": 1
+    "target_directory": "[..]foo[/]target",
+    "version": 1,
+    "workspace_root": "[..][/]foo"
 }"#;
 
 #[test]
 fn cargo_metadata_no_deps_path_to_cargo_toml_relative() {
     let p = project("foo")
         .file("Cargo.toml", &basic_bin_manifest("foo"))
-        .file("src/foo.rs", &main_file(r#""i am foo""#, &[]));
+        .file("src/foo.rs", &main_file(r#""i am foo""#, &[]))
+        .build();
 
-        assert_that(p.cargo_process("metadata").arg("--no-deps")
+        assert_that(p.cargo("metadata").arg("--no-deps")
                      .arg("--manifest-path").arg("foo/Cargo.toml")
                      .cwd(p.root().parent().unwrap()),
                     execs().with_status(0)
@@ -586,9 +611,10 @@ fn cargo_metadata_no_deps_path_to_cargo_toml_relative() {
 fn cargo_metadata_no_deps_path_to_cargo_toml_absolute() {
     let p = project("foo")
         .file("Cargo.toml", &basic_bin_manifest("foo"))
-        .file("src/foo.rs", &main_file(r#""i am foo""#, &[]));
+        .file("src/foo.rs", &main_file(r#""i am foo""#, &[]))
+        .build();
 
-    assert_that(p.cargo_process("metadata").arg("--no-deps")
+    assert_that(p.cargo("metadata").arg("--no-deps")
                  .arg("--manifest-path").arg(p.root().join("Cargo.toml"))
                  .cwd(p.root().parent().unwrap()),
                 execs().with_status(0)
@@ -599,9 +625,10 @@ fn cargo_metadata_no_deps_path_to_cargo_toml_absolute() {
 fn cargo_metadata_no_deps_path_to_cargo_toml_parent_relative() {
     let p = project("foo")
         .file("Cargo.toml", &basic_bin_manifest("foo"))
-        .file("src/foo.rs", &main_file(r#""i am foo""#, &[]));
+        .file("src/foo.rs", &main_file(r#""i am foo""#, &[]))
+        .build();
 
-    assert_that(p.cargo_process("metadata").arg("--no-deps")
+    assert_that(p.cargo("metadata").arg("--no-deps")
                  .arg("--manifest-path").arg("foo")
                  .cwd(p.root().parent().unwrap()),
                 execs().with_status(101)
@@ -613,9 +640,10 @@ fn cargo_metadata_no_deps_path_to_cargo_toml_parent_relative() {
 fn cargo_metadata_no_deps_path_to_cargo_toml_parent_absolute() {
     let p = project("foo")
         .file("Cargo.toml", &basic_bin_manifest("foo"))
-        .file("src/foo.rs", &main_file(r#""i am foo""#, &[]));
+        .file("src/foo.rs", &main_file(r#""i am foo""#, &[]))
+        .build();
 
-    assert_that(p.cargo_process("metadata").arg("--no-deps")
+    assert_that(p.cargo("metadata").arg("--no-deps")
                  .arg("--manifest-path").arg(p.root())
                  .cwd(p.root().parent().unwrap()),
                 execs().with_status(101)
@@ -627,9 +655,10 @@ fn cargo_metadata_no_deps_path_to_cargo_toml_parent_absolute() {
 fn cargo_metadata_no_deps_cwd() {
     let p = project("foo")
         .file("Cargo.toml", &basic_bin_manifest("foo"))
-        .file("src/foo.rs", &main_file(r#""i am foo""#, &[]));
+        .file("src/foo.rs", &main_file(r#""i am foo""#, &[]))
+        .build();
 
-    assert_that(p.cargo_process("metadata").arg("--no-deps")
+    assert_that(p.cargo("metadata").arg("--no-deps")
                  .cwd(p.root()),
                 execs().with_status(0)
                        .with_json(MANIFEST_OUTPUT));
@@ -639,9 +668,10 @@ fn cargo_metadata_no_deps_cwd() {
 fn cargo_metadata_bad_version() {
     let p = project("foo")
         .file("Cargo.toml", &basic_bin_manifest("foo"))
-        .file("src/foo.rs", &main_file(r#""i am foo""#, &[]));
+        .file("src/foo.rs", &main_file(r#""i am foo""#, &[]))
+        .build();
 
-    assert_that(p.cargo_process("metadata").arg("--no-deps")
+    assert_that(p.cargo("metadata").arg("--no-deps")
                  .arg("--format-version").arg("2")
                  .cwd(p.root()),
                 execs().with_status(101)
@@ -661,9 +691,10 @@ fn multiple_features() {
             a = []
             b = []
         "#)
-        .file("src/lib.rs", "");
+        .file("src/lib.rs", "")
+        .build();
 
-    assert_that(p.cargo_process("metadata")
+    assert_that(p.cargo("metadata")
                  .arg("--features").arg("a b"),
                 execs().with_status(0));
 }
