@@ -163,37 +163,6 @@ fn keyword_name() {
 use --name to override crate name"));
 }
 
-#[test]
-fn rust_prefix_stripped() {
-    assert_that(cargo_process("new").arg("--lib").arg("rust-foo").env("USER", "foo"),
-                execs().with_status(0)
-                       .with_stderr_contains("note: package will be named `foo`; use --name to override"));
-    let toml = paths::root().join("rust-foo/Cargo.toml");
-    let mut contents = String::new();
-    File::open(&toml).unwrap().read_to_string(&mut contents).unwrap();
-    assert!(contents.contains(r#"name = "foo""#));
-}
-
-#[test]
-fn bin_disables_stripping() {
-    assert_that(cargo_process("new").arg("rust-foo").arg("--bin").env("USER", "foo"),
-                execs().with_status(0));
-    let toml = paths::root().join("rust-foo/Cargo.toml");
-    let mut contents = String::new();
-    File::open(&toml).unwrap().read_to_string(&mut contents).unwrap();
-    assert!(contents.contains(r#"name = "rust-foo""#));
-}
-
-#[test]
-fn explicit_name_not_stripped() {
-    assert_that(cargo_process("new").arg("foo").arg("--name").arg("rust-bar").env("USER", "foo"),
-                execs().with_status(0));
-    let toml = paths::root().join("foo/Cargo.toml");
-    let mut contents = String::new();
-    File::open(&toml).unwrap().read_to_string(&mut contents).unwrap();
-    assert!(contents.contains(r#"name = "rust-bar""#));
-}
-
 #[test]
 fn finds_author_user() {
     create_empty_gitconfig();
@@ -416,3 +385,11 @@ Usage:
     cargo new [..]
 "));
 }
+
+#[test]
+fn explicit_invalid_name_not_suggested() {
+    assert_that(cargo_process("new").arg("--name").arg("10-invalid").arg("a"),
+                execs().with_status(101)
+                       .with_stderr("\
+[ERROR] Package names starting with a digit cannot be used as a crate name"));
+}
