@@ -9,8 +9,8 @@ use std::fs::File;
 use std::io::prelude::*;
 use std::path::{Path, PathBuf};
 
-use cargotest::{cargo_process, process};
-use cargotest::support::{project, execs, paths, git, path2url, cargo_exe};
+use cargotest::{cargo_process, process, ChannelChanger};
+use cargotest::support::{project, execs, paths, git, path2url, cargo_exe, registry};
 use cargotest::support::registry::Package;
 use flate2::read::GzDecoder;
 use hamcrest::{assert_that, existing_file, contains, equal_to};
@@ -330,10 +330,6 @@ See [..]
 See [..]
 [WARNING] [..] file `some_dir[/]dir_deep_3[/]some_dir[/]file` WILL be excluded [..]
 See [..]
-[WARNING] [..] file `some_dir[/]dir_deep_4[/]some_dir[/]file` WILL be excluded [..]
-See [..]
-[WARNING] [..] file `some_dir[/]dir_deep_5[/]some_dir[/]file` WILL be excluded [..]
-See [..]
 [WARNING] [..] file `some_dir[/]file_deep_1` WILL be excluded [..]
 See [..]
 [ARCHIVING] [..]
@@ -715,6 +711,8 @@ fn generated_manifest() {
     Package::new("ghi", "1.0.0").publish();
     let p = project("foo")
         .file("Cargo.toml", r#"
+            cargo-features = ["alternative-registries"]
+
             [project]
             name = "foo"
             version = "0.0.1"
@@ -730,7 +728,7 @@ fn generated_manifest() {
 
             [dependencies]
             bar = { path = "bar", version = "0.1" }
-            def = "1.0"
+            def = { version = "1.0", registry = "alternative" }
             ghi = "1.0"
             abc = "1.0"
         "#)
@@ -744,7 +742,9 @@ fn generated_manifest() {
         .file("bar/src/lib.rs", "")
         .build();
 
-    assert_that(p.cargo("package").arg("--no-verify"),
+    assert_that(p.cargo("package")
+                    .masquerade_as_nightly_cargo()
+                    .arg("--no-verify"),
                 execs().with_status(0));
 
     let f = File::open(&p.root().join("target/package/foo-0.0.1.crate")).unwrap();
@@ -761,6 +761,7 @@ fn generated_manifest() {
     // BTreeMap makes the order of dependencies in the generated file deterministic
     // by sorting alphabetically
     assert_that(&contents[..], equal_to(
+&*format!(
 r#"# THIS FILE IS AUTOMATICALLY GENERATED BY CARGO
 #
 # When uploading crates to the registry Cargo will automatically
@@ -773,6 +774,8 @@ r#"# THIS FILE IS AUTOMATICALLY GENERATED BY CARGO
 # editing this file be aware that the upstream Cargo.toml
 # will likely look very different (and much more reasonable)
 
+cargo-features = ["alternative-registries"]
+
 [package]
 name = "foo"
 version = "0.0.1"
@@ -791,10 +794,12 @@ version = "0.1"
 
 [dependencies.def]
 version = "1.0"
+registry-index = "{}"
 
 [dependencies.ghi]
 version = "1.0"
-"#));
+"#,
+    registry::alt_registry())));
 }
 
 #[test]
@@ -878,3 +883,120 @@ fn package_two_kinds_of_deps() {
     assert_that(p.cargo("package").arg("--no-verify"),
                 execs().with_status(0));
 }
+
+#[test]
+fn test_epoch() {
+    let p = project("foo")
+        .file("Cargo.toml", r#"
+            cargo-features = ["epoch"]
+            [package]
+            name = "foo"
+            version = "0.0.1"
+            authors = []
+            rust = "2018"
+        "#)
+        .file("src/lib.rs", r#" "#)
+        .build();
+
+    assert_that(p.cargo("build").arg("-v")
+                 .masquerade_as_nightly_cargo(),
+                execs()
+                // -Zepoch is still in flux and we're not passing -Zunstable-options
+                // from Cargo so it will probably error. Only partially match the output
+                // until stuff stabilizes
+                .with_stderr_contains(format!("\
+[COMPILING] foo v0.0.1 ({url})
+[RUNNING] `rustc --crate-name foo src[/]lib.rs --crate-type lib \
+        --emit=dep-info,link -Zepoch=2018 -C debuginfo=2 \
+        -C metadata=[..] \
+        --out-dir [..] \
+        -L dependency={dir}[/]target[/]debug[/]deps`
+", dir = p.root().display(), url = p.url())));
+}
+
+#[test]
+fn test_epoch_missing() {
+    // no epoch = 2015
+    let p = project("foo")
+        .file("Cargo.toml", r#"
+            cargo-features = ["epoch"]
+            [package]
+            name = "foo"
+            version = "0.0.1"
+            authors = []
+        "#)
+        .file("src/lib.rs", r#" "#)
+        .build();
+
+    assert_that(p.cargo("build").arg("-v")
+                 .masquerade_as_nightly_cargo(),
+                execs()
+                // -Zepoch is still in flux and we're not passing -Zunstable-options
+                // from Cargo so it will probably error. Only partially match the output
+                // until stuff stabilizes
+                .with_stderr_contains(format!("\
+[COMPILING] foo v0.0.1 ({url})
+[RUNNING] `rustc --crate-name foo src[/]lib.rs --crate-type lib \
+        --emit=dep-info,link -Zepoch=2015 -C debuginfo=2 \
+        -C metadata=[..] \
+        --out-dir [..] \
+        -L dependency={dir}[/]target[/]debug[/]deps`
+", dir = p.root().display(), url = p.url())));
+}
+
+#[test]
+fn test_epoch_malformed() {
+    let p = project("foo")
+        .file("Cargo.toml", r#"
+            cargo-features = ["epoch"]
+            [package]
+            name = "foo"
+            version = "0.0.1"
+            authors = []
+            rust = "chicken"
+        "#)
+        .file("src/lib.rs", r#" "#)
+        .build();
+
+    assert_that(p.cargo("build").arg("-v")
+                 .masquerade_as_nightly_cargo(),
+                execs()
+                .with_status(101)
+                .with_stderr(format!("\
+error: failed to parse manifest at `[..]`
+
+Caused by:
+  the `rust` key must be one of: `2015`, `2018`
+")));
+}
+
+
+#[test]
+fn test_epoch_nightly() {
+    let p = project("foo")
+        .file("Cargo.toml", r#"
+            [package]
+            name = "foo"
+            version = "0.0.1"
+            authors = []
+            rust = "2015"
+        "#)
+        .file("src/lib.rs", r#" "#)
+        .build();
+
+    assert_that(p.cargo("build").arg("-v")
+                 .masquerade_as_nightly_cargo(),
+                execs()
+                .with_status(101)
+                .with_stderr(format!("\
+error: failed to parse manifest at `[..]`
+
+Caused by:
+  epoches are unstable
+
+Caused by:
+  feature `epoch` is required
+
+consider adding `cargo-features = [\"epoch\"]` to the manifest
+")));
+}
