@@ -20,9 +20,10 @@ fn build_lib_for_foo() {
         .file("src/main.rs", r#"
             fn main() {}
         "#)
-        .file("src/lib.rs", r#" "#);
+        .file("src/lib.rs", r#" "#)
+        .build();
 
-    assert_that(p.cargo_process("rustc").arg("--lib").arg("-v"),
+    assert_that(p.cargo("rustc").arg("--lib").arg("-v"),
                 execs()
                 .with_status(0)
                 .with_stderr(format!("\
@@ -48,9 +49,10 @@ fn lib() {
         .file("src/main.rs", r#"
             fn main() {}
         "#)
-        .file("src/lib.rs", r#" "#);
+        .file("src/lib.rs", r#" "#)
+        .build();
 
-    assert_that(p.cargo_process("rustc").arg("--lib").arg("-v")
+    assert_that(p.cargo("rustc").arg("--lib").arg("-v")
                 .arg("--").arg("-C").arg("debug-assertions=off"),
                 execs()
                 .with_status(0)
@@ -78,9 +80,10 @@ fn build_main_and_allow_unstable_options() {
         .file("src/main.rs", r#"
             fn main() {}
         "#)
-        .file("src/lib.rs", r#" "#);
+        .file("src/lib.rs", r#" "#)
+        .build();
 
-    assert_that(p.cargo_process("rustc").arg("-v").arg("--bin").arg("foo")
+    assert_that(p.cargo("rustc").arg("-v").arg("--bin").arg("foo")
                 .arg("--").arg("-C").arg("debug-assertions"),
                 execs()
                 .with_status(0)
@@ -116,9 +119,10 @@ fn fails_when_trying_to_build_main_and_lib_with_args() {
         .file("src/main.rs", r#"
             fn main() {}
         "#)
-        .file("src/lib.rs", r#" "#);
+        .file("src/lib.rs", r#" "#)
+        .build();
 
-    assert_that(p.cargo_process("rustc").arg("-v")
+    assert_that(p.cargo("rustc").arg("-v")
                 .arg("--").arg("-C").arg("debug-assertions"),
                 execs()
                 .with_status(101)
@@ -143,9 +147,10 @@ fn build_with_args_to_one_of_multiple_binaries() {
         .file("src/bin/baz.rs", r#"
             fn main() {}
         "#)
-        .file("src/lib.rs", r#" "#);
+        .file("src/lib.rs", r#" "#)
+        .build();
 
-    assert_that(p.cargo_process("rustc").arg("-v").arg("--bin").arg("bar")
+    assert_that(p.cargo("rustc").arg("-v").arg("--bin").arg("bar")
                 .arg("--").arg("-C").arg("debug-assertions"),
                 execs()
                 .with_status(0)
@@ -178,9 +183,10 @@ fn fails_with_args_to_all_binaries() {
         .file("src/bin/baz.rs", r#"
             fn main() {}
         "#)
-        .file("src/lib.rs", r#" "#);
+        .file("src/lib.rs", r#" "#)
+        .build();
 
-    assert_that(p.cargo_process("rustc").arg("-v")
+    assert_that(p.cargo("rustc").arg("-v")
                 .arg("--").arg("-C").arg("debug-assertions"),
                 execs()
                 .with_status(101)
@@ -199,9 +205,10 @@ fn build_with_args_to_one_of_multiple_tests() {
         .file("tests/foo.rs", r#" "#)
         .file("tests/bar.rs", r#" "#)
         .file("tests/baz.rs", r#" "#)
-        .file("src/lib.rs", r#" "#);
+        .file("src/lib.rs", r#" "#)
+        .build();
 
-    assert_that(p.cargo_process("rustc").arg("-v").arg("--test").arg("bar")
+    assert_that(p.cargo("rustc").arg("-v").arg("--test").arg("bar")
                 .arg("--").arg("-C").arg("debug-assertions"),
                 execs()
                 .with_status(0)
@@ -233,8 +240,9 @@ fn build_foo_with_bar_dependency() {
             fn main() {
                 bar::baz()
             }
-        "#);
-    let bar = project("bar")
+        "#)
+        .build();
+    let _bar = project("bar")
         .file("Cargo.toml", r#"
             [package]
             name = "bar"
@@ -243,10 +251,10 @@ fn build_foo_with_bar_dependency() {
         "#)
         .file("src/lib.rs", r#"
             pub fn baz() {}
-        "#);
-    bar.build();
+        "#)
+        .build();
 
-    assert_that(foo.cargo_process("rustc").arg("-v").arg("--").arg("-C").arg("debug-assertions"),
+    assert_that(foo.cargo("rustc").arg("-v").arg("--").arg("-C").arg("debug-assertions"),
                 execs()
                 .with_status(0)
                 .with_stderr(format!("\
@@ -275,8 +283,9 @@ fn build_only_bar_dependency() {
             fn main() {
                 bar::baz()
             }
-        "#);
-    let bar = project("bar")
+        "#)
+        .build();
+    let _bar = project("bar")
         .file("Cargo.toml", r#"
             [package]
             name = "bar"
@@ -285,10 +294,10 @@ fn build_only_bar_dependency() {
         "#)
         .file("src/lib.rs", r#"
             pub fn baz() {}
-        "#);
-    bar.build();
+        "#)
+        .build();
 
-    assert_that(foo.cargo_process("rustc").arg("-v").arg("-p").arg("bar")
+    assert_that(foo.cargo("rustc").arg("-v").arg("-p").arg("bar")
                 .arg("--").arg("-C").arg("debug-assertions"),
                 execs()
                 .with_status(0)
@@ -316,10 +325,10 @@ fn fail_with_multiple_packages() {
         "#)
         .file("src/main.rs", r#"
             fn main() {}
-        "#);
-    foo.build();
+        "#)
+        .build();
 
-    let bar = project("bar")
+    let _bar = project("bar")
         .file("Cargo.toml", r#"
             [package]
             name = "bar"
@@ -330,10 +339,10 @@ fn fail_with_multiple_packages() {
             fn main() {
                 if cfg!(flag = "1") { println!("Yeah from bar!"); }
             }
-        "#);
-    bar.build();
+        "#)
+        .build();
 
-    let baz = project("baz")
+    let _baz = project("baz")
         .file("Cargo.toml", r#"
             [package]
             name = "baz"
@@ -344,8 +353,8 @@ fn fail_with_multiple_packages() {
             fn main() {
                 if cfg!(flag = "1") { println!("Yeah from baz!"); }
             }
-        "#);
-    baz.build();
+        "#)
+        .build();
 
     assert_that(foo.cargo("rustc").arg("-v").arg("-p").arg("bar")
                                           .arg("-p").arg("baz"),
@@ -358,7 +367,7 @@ Usage:
 
 #[test]
 fn rustc_with_other_profile() {
-    let foo = project("foo")
+    let p = project("foo")
         .file("Cargo.toml", r#"
             [package]
             name = "foo"
@@ -380,9 +389,9 @@ fn rustc_with_other_profile() {
             version = "0.1.0"
             authors = []
         "#)
-        .file("a/src/lib.rs", "");
-    foo.build();
+        .file("a/src/lib.rs", "")
+        .build();
 
-    assert_that(foo.cargo("rustc").arg("--profile").arg("test"),
+    assert_that(p.cargo("rustc").arg("--profile").arg("test"),
                 execs().with_status(0));
 }
