@@ -2,8 +2,9 @@
 extern crate cargotest;
 extern crate hamcrest;
 
+use std::env;
+use std::fs::{self, File};
 use std::io::{Read, Write};
-use std::fs::File;
 
 use cargotest::sleep_ms;
 use cargotest::support::{project, execs, git};
@@ -31,7 +32,7 @@ fn simple_explicit() {
             workspace = ".."
         "#)
         .file("bar/src/main.rs", "fn main() {}");
-    p.build();
+    let p = p.build();
 
     assert_that(p.cargo("build"), execs().with_status(0));
     assert_that(&p.bin("foo"), existing_file());
@@ -46,6 +47,35 @@ fn simple_explicit() {
     assert_that(&p.root().join("bar/Cargo.lock"), is_not(existing_file()));
 }
 
+#[test]
+fn simple_explicit_default_members() {
+    let p = project("foo")
+        .file("Cargo.toml", r#"
+            [project]
+            name = "foo"
+            version = "0.1.0"
+            authors = []
+
+            [workspace]
+            members = ["bar"]
+            default-members = ["bar"]
+        "#)
+        .file("src/main.rs", "fn main() {}")
+        .file("bar/Cargo.toml", r#"
+            [project]
+            name = "bar"
+            version = "0.1.0"
+            authors = []
+            workspace = ".."
+        "#)
+        .file("bar/src/main.rs", "fn main() {}");
+    let p = p.build();
+
+    assert_that(p.cargo("build"), execs().with_status(0));
+    assert_that(&p.bin("bar"), existing_file());
+    assert_that(&p.bin("foo"), is_not(existing_file()));
+}
+
 #[test]
 fn inferred_root() {
     let p = project("foo")
@@ -66,7 +96,7 @@ fn inferred_root() {
             authors = []
         "#)
         .file("bar/src/main.rs", "fn main() {}");
-    p.build();
+    let p = p.build();
 
     assert_that(p.cargo("build"), execs().with_status(0));
     assert_that(&p.bin("foo"), existing_file());
@@ -104,7 +134,7 @@ fn inferred_path_dep() {
         "#)
         .file("bar/src/main.rs", "fn main() {}")
         .file("bar/src/lib.rs", "");
-    p.build();
+    let p = p.build();
 
     assert_that(p.cargo("build"), execs().with_status(0));
     assert_that(&p.bin("foo"), existing_file());
@@ -153,7 +183,7 @@ fn transitive_path_dep() {
         "#)
         .file("baz/src/main.rs", "fn main() {}")
         .file("baz/src/lib.rs", "");
-    p.build();
+    let p = p.build();
 
     assert_that(p.cargo("build"), execs().with_status(0));
     assert_that(&p.bin("foo"), existing_file());
@@ -201,7 +231,7 @@ fn parent_pointer_works() {
         "#)
         .file("bar/src/main.rs", "fn main() {}")
         .file("bar/src/lib.rs", "");
-    p.build();
+    let p = p.build();
 
     assert_that(p.cargo("build").cwd(p.root().join("foo")),
                 execs().with_status(0));
@@ -232,7 +262,7 @@ fn same_names_in_workspace() {
             workspace = ".."
         "#)
         .file("bar/src/main.rs", "fn main() {}");
-    p.build();
+    let p = p.build();
 
     assert_that(p.cargo("build"),
                 execs().with_status(101)
@@ -262,7 +292,7 @@ fn parent_doesnt_point_to_child() {
             authors = []
         "#)
         .file("bar/src/main.rs", "fn main() {}");
-    p.build();
+    let p = p.build();
 
     assert_that(p.cargo("build").cwd(p.root().join("bar")),
                 execs().with_status(101)
@@ -286,7 +316,7 @@ fn invalid_parent_pointer() {
             workspace = "foo"
         "#)
         .file("src/main.rs", "fn main() {}");
-    p.build();
+    let p = p.build();
 
     assert_that(p.cargo("build"),
                 execs().with_status(101)
@@ -311,7 +341,7 @@ fn invalid_members() {
             members = ["foo"]
         "#)
         .file("src/main.rs", "fn main() {}");
-    p.build();
+    let p = p.build();
 
     assert_that(p.cargo("build"),
                 execs().with_status(101)
@@ -335,7 +365,7 @@ fn bare_workspace_ok() {
             [workspace]
         "#)
         .file("src/main.rs", "fn main() {}");
-    p.build();
+    let p = p.build();
 
     assert_that(p.cargo("build"), execs().with_status(0));
 }
@@ -363,7 +393,7 @@ fn two_roots() {
             members = [".."]
         "#)
         .file("bar/src/main.rs", "fn main() {}");
-    p.build();
+    let p = p.build();
 
     assert_that(p.cargo("build"),
                 execs().with_status(101)
@@ -392,7 +422,7 @@ fn workspace_isnt_root() {
             authors = []
         "#)
         .file("bar/src/main.rs", "fn main() {}");
-    p.build();
+    let p = p.build();
 
     assert_that(p.cargo("build"),
                 execs().with_status(101)
@@ -430,7 +460,7 @@ fn dangling_member() {
             workspace = "../baz"
         "#)
         .file("baz/src/main.rs", "fn main() {}");
-    p.build();
+    let p = p.build();
 
     assert_that(p.cargo("build"),
                 execs().with_status(101)
@@ -460,7 +490,7 @@ fn cycle() {
             workspace = ".."
         "#)
         .file("bar/src/main.rs", "fn main() {}");
-    p.build();
+    let p = p.build();
 
     assert_that(p.cargo("build"),
                 execs().with_status(101));
@@ -492,7 +522,7 @@ fn share_dependencies() {
             dep1 = "< 0.1.5"
         "#)
         .file("bar/src/main.rs", "fn main() {}");
-    p.build();
+    let p = p.build();
 
     Package::new("dep1", "0.1.3").publish();
     Package::new("dep1", "0.1.8").publish();
@@ -531,7 +561,7 @@ fn fetch_fetches_all() {
             dep1 = "*"
         "#)
         .file("bar/src/main.rs", "fn main() {}");
-    p.build();
+    let p = p.build();
 
     Package::new("dep1", "0.1.3").publish();
 
@@ -569,7 +599,7 @@ fn lock_works_for_everyone() {
             dep1 = "0.1"
         "#)
         .file("bar/src/main.rs", "fn main() {}");
-    p.build();
+    let p = p.build();
 
     Package::new("dep1", "0.1.0").publish();
     Package::new("dep2", "0.1.0").publish();
@@ -616,7 +646,7 @@ fn virtual_works() {
             authors = []
         "#)
         .file("bar/src/main.rs", "fn main() {}");
-    p.build();
+    let p = p.build();
     assert_that(p.cargo("build").cwd(p.root().join("bar")),
                 execs().with_status(0));
     assert_that(&p.root().join("Cargo.lock"), existing_file());
@@ -638,7 +668,7 @@ fn explicit_package_argument_works_with_virtual_manifest() {
             authors = []
         "#)
         .file("bar/src/main.rs", "fn main() {}");
-    p.build();
+    let p = p.build();
     assert_that(p.cargo("build").cwd(p.root()).args(&["--package", "bar"]),
                 execs().with_status(0));
     assert_that(&p.root().join("Cargo.lock"), existing_file());
@@ -659,7 +689,7 @@ fn virtual_misconfigure() {
             authors = []
         "#)
         .file("bar/src/main.rs", "fn main() {}");
-    p.build();
+    let p = p.build();
     assert_that(p.cargo("build").cwd(p.root().join("bar")),
                 execs().with_status(101)
                        .with_stderr("\
@@ -686,16 +716,47 @@ fn virtual_build_all_implied() {
             authors = []
         "#)
         .file("bar/src/main.rs", "fn main() {}");
-    p.build();
+    let p = p.build();
     assert_that(p.cargo("build"),
                 execs().with_status(0));
 }
 
 #[test]
-fn virtual_build_no_members() {
+fn virtual_default_members() {
     let p = project("foo")
         .file("Cargo.toml", r#"
             [workspace]
+            members = ["bar", "baz"]
+            default-members = ["bar"]
+        "#)
+        .file("bar/Cargo.toml", r#"
+            [project]
+            name = "bar"
+            version = "0.1.0"
+            authors = []
+        "#)
+        .file("baz/Cargo.toml", r#"
+            [project]
+            name = "baz"
+            version = "0.1.0"
+            authors = []
+        "#)
+        .file("bar/src/main.rs", "fn main() {}")
+        .file("baz/src/main.rs", "fn main() {}");
+    let p = p.build();
+    assert_that(p.cargo("build"),
+                execs().with_status(0));
+    assert_that(&p.bin("bar"), existing_file());
+    assert_that(&p.bin("baz"), is_not(existing_file()));
+}
+
+#[test]
+fn virtual_default_member_is_not_a_member() {
+    let p = project("foo")
+        .file("Cargo.toml", r#"
+            [workspace]
+            members = ["bar"]
+            default-members = ["something-else"]
         "#)
         .file("bar/Cargo.toml", r#"
             [project]
@@ -704,12 +765,27 @@ fn virtual_build_no_members() {
             authors = []
         "#)
         .file("bar/src/main.rs", "fn main() {}");
-    p.build();
+    let p = p.build();
     assert_that(p.cargo("build"),
                 execs().with_status(101)
                        .with_stderr("\
-error: manifest path `[..]` is a virtual manifest, but this command \
-requires running against an actual package in this workspace
+error: package `[..]something-else` is listed in workspace’s default-members \
+but is not a member.
+"));
+}
+
+#[test]
+fn virtual_build_no_members() {
+    let p = project("foo")
+        .file("Cargo.toml", r#"
+            [workspace]
+        "#);
+    let p = p.build();
+    assert_that(p.cargo("build"),
+                execs().with_status(101)
+                       .with_stderr("\
+error: manifest path `[..]` contains no package: The manifest is virtual, \
+and the workspace has no members.
 "));
 }
 
@@ -728,7 +804,7 @@ fn include_virtual() {
         .file("bar/Cargo.toml", r#"
             [workspace]
         "#);
-    p.build();
+    let p = p.build();
     assert_that(p.cargo("build"),
                 execs().with_status(101)
                        .with_stderr("\
@@ -778,7 +854,7 @@ fn members_include_path_deps() {
             authors = []
         "#)
         .file("p3/src/lib.rs", "");
-    p.build();
+    let p = p.build();
 
     assert_that(p.cargo("build").cwd(p.root().join("p1")),
                 execs().with_status(0));
@@ -807,7 +883,7 @@ fn new_warns_you_this_will_not_work() {
             [workspace]
         "#)
         .file("src/lib.rs", "");
-    p.build();
+    let p = p.build();
 
     assert_that(p.cargo("new").arg("--lib").arg("bar").env("USER", "foo"),
                 execs().with_status(0)
@@ -851,7 +927,7 @@ fn lock_doesnt_change_depending_on_crate() {
             bar = "*"
         "#)
         .file("baz/src/lib.rs", "");
-    p.build();
+    let p = p.build();
 
     Package::new("foo", "1.0.0").publish();
     Package::new("bar", "1.0.0").publish();
@@ -901,7 +977,7 @@ fn rebuild_please() {
                 assert_eq!(lib::foo(), 0);
             }
         "#);
-    p.build();
+    let p = p.build();
 
     assert_that(p.cargo("run").cwd(p.root().join("bin")),
                 execs().with_status(0));
@@ -946,7 +1022,7 @@ fn workspace_in_git() {
         .file("src/lib.rs", r#"
             pub fn foo() -> u32 { 0 }
         "#);
-    p.build();
+    let p = p.build();
 
     assert_that(p.cargo("build"),
                 execs().with_status(0));
@@ -968,7 +1044,7 @@ fn lockfile_can_specify_nonexistant_members() {
         "#)
         .file("a/src/main.rs", "fn main() {}")
         .file("Cargo.lock", r#"
-            [root]
+            [[package]]
             name = "a"
             version = "0.1.0"
 
@@ -977,7 +1053,7 @@ fn lockfile_can_specify_nonexistant_members() {
             version = "0.1.0"
         "#);
 
-    p.build();
+    let p = p.build();
 
     assert_that(p.cargo("build").cwd(p.root().join("a")), execs().with_status(0));
 }
@@ -995,7 +1071,7 @@ fn you_cannot_generate_lockfile_for_empty_workspaces() {
             authors = []
         "#)
         .file("bar/src/main.rs", "fn main() {}");
-    p.build();
+    let p = p.build();
 
     assert_that(p.cargo("update"),
                 execs().with_status(101)
@@ -1046,7 +1122,7 @@ fn workspace_with_transitive_dev_deps() {
             authors = ["mbrubeck@example.com"]
         "#)
         .file("baz/src/lib.rs", r#"pub fn do_stuff() {}"#);
-    p.build();
+    let p = p.build();
 
     assert_that(p.cargo("test").args(&["-p", "bar"]),
                 execs().with_status(0));
@@ -1063,7 +1139,7 @@ fn error_if_parent_cargo_toml_is_invalid() {
             authors = []
         "#)
         .file("bar/src/main.rs", "fn main() {}");
-    p.build();
+    let p = p.build();
 
     assert_that(p.cargo("build").cwd(p.root().join("bar")),
                 execs().with_status(101)
@@ -1092,7 +1168,7 @@ fn relative_path_for_member_works() {
         workspace = "../foo"
     "#)
         .file("bar/src/main.rs", "fn main() {}");
-    p.build();
+    let p = p.build();
 
     assert_that(p.cargo("build").cwd(p.root().join("foo")), execs().with_status(0));
     assert_that(p.cargo("build").cwd(p.root().join("bar")), execs().with_status(0));
@@ -1120,7 +1196,7 @@ fn relative_path_for_root_works() {
         authors = []
     "#)
         .file("subproj/src/main.rs", "fn main() {}");
-    p.build();
+    let p = p.build();
 
     assert_that(p.cargo("build").cwd(p.root())
                     .arg("--manifest-path").arg("./Cargo.toml"),
@@ -1153,7 +1229,7 @@ fn path_dep_outside_workspace_is_not_member() {
             authors = []
         "#)
         .file("foo/src/lib.rs", "");
-    p.build();
+    let p = p.build();
 
     assert_that(p.cargo("build").cwd(p.root().join("ws")),
                 execs().with_status(0));
@@ -1193,7 +1269,7 @@ fn test_in_and_out_of_workspace() {
             authors = []
         "#)
         .file("bar/src/lib.rs", "pub fn f() { }");
-    p.build();
+    let p = p.build();
 
     assert_that(p.cargo("build").cwd(p.root().join("ws")),
                 execs().with_status(0));
@@ -1246,7 +1322,7 @@ fn test_path_dependency_under_member() {
             authors = []
         "#)
         .file("foo/bar/src/lib.rs", "pub fn f() { }");
-    p.build();
+    let p = p.build();
 
     assert_that(p.cargo("build").cwd(p.root().join("ws")),
                 execs().with_status(0));
@@ -1281,7 +1357,7 @@ fn excluded_simple() {
             authors = []
         "#)
         .file("foo/src/lib.rs", "");
-    p.build();
+    let p = p.build();
 
     assert_that(p.cargo("build"),
                 execs().with_status(0));
@@ -1319,7 +1395,7 @@ fn exclude_members_preferred() {
             authors = []
         "#)
         .file("foo/bar/src/lib.rs", "");
-    p.build();
+    let p = p.build();
 
     assert_that(p.cargo("build"),
                 execs().with_status(0));
@@ -1362,7 +1438,7 @@ fn exclude_but_also_depend() {
             authors = []
         "#)
         .file("foo/bar/src/lib.rs", "");
-    p.build();
+    let p = p.build();
 
     assert_that(p.cargo("build"),
                 execs().with_status(0));
@@ -1412,6 +1488,71 @@ fn glob_syntax() {
             authors = []
         "#)
         .file("crates/qux/src/main.rs", "fn main() {}");
+    let p = p.build();
+
+    assert_that(p.cargo("build"), execs().with_status(0));
+    assert_that(&p.bin("foo"), existing_file());
+    assert_that(&p.bin("bar"), is_not(existing_file()));
+    assert_that(&p.bin("baz"), is_not(existing_file()));
+
+    assert_that(p.cargo("build").cwd(p.root().join("crates/bar")),
+                execs().with_status(0));
+    assert_that(&p.bin("foo"), existing_file());
+    assert_that(&p.bin("bar"), existing_file());
+
+    assert_that(p.cargo("build").cwd(p.root().join("crates/baz")),
+                execs().with_status(0));
+    assert_that(&p.bin("foo"), existing_file());
+    assert_that(&p.bin("baz"), existing_file());
+
+    assert_that(p.cargo("build").cwd(p.root().join("crates/qux")),
+                execs().with_status(0));
+    assert_that(&p.bin("qux"), is_not(existing_file()));
+
+    assert_that(&p.root().join("Cargo.lock"), existing_file());
+    assert_that(&p.root().join("crates/bar/Cargo.lock"), is_not(existing_file()));
+    assert_that(&p.root().join("crates/baz/Cargo.lock"), is_not(existing_file()));
+    assert_that(&p.root().join("crates/qux/Cargo.lock"), existing_file());
+}
+
+/*FIXME: This fails because of how workspace.exclude and workspace.members are working.
+#[test]
+fn glob_syntax_2() {
+    let p = project("foo")
+        .file("Cargo.toml", r#"
+            [project]
+            name = "foo"
+            version = "0.1.0"
+            authors = []
+
+            [workspace]
+            members = ["crates/b*"]
+            exclude = ["crates/q*"]
+        "#)
+        .file("src/main.rs", "fn main() {}")
+        .file("crates/bar/Cargo.toml", r#"
+            [project]
+            name = "bar"
+            version = "0.1.0"
+            authors = []
+            workspace = "../.."
+        "#)
+        .file("crates/bar/src/main.rs", "fn main() {}")
+        .file("crates/baz/Cargo.toml", r#"
+            [project]
+            name = "baz"
+            version = "0.1.0"
+            authors = []
+            workspace = "../.."
+        "#)
+        .file("crates/baz/src/main.rs", "fn main() {}")
+        .file("crates/qux/Cargo.toml", r#"
+            [project]
+            name = "qux"
+            version = "0.1.0"
+            authors = []
+        "#)
+        .file("crates/qux/src/main.rs", "fn main() {}");
     p.build();
 
     assert_that(p.cargo("build"), execs().with_status(0));
@@ -1438,6 +1579,7 @@ fn glob_syntax() {
     assert_that(&p.root().join("crates/baz/Cargo.lock"), is_not(existing_file()));
     assert_that(&p.root().join("crates/qux/Cargo.lock"), existing_file());
 }
+*/
 
 #[test]
 fn glob_syntax_invalid_members() {
@@ -1453,7 +1595,7 @@ fn glob_syntax_invalid_members() {
         "#)
         .file("src/main.rs", "fn main() {}")
         .file("crates/bar/src/main.rs", "fn main() {}");
-    p.build();
+    let p = p.build();
 
     assert_that(p.cargo("build"),
                 execs().with_status(101)
@@ -1465,3 +1607,252 @@ Caused by:
 "));
 }
 
+/// This is a freshness test for feature use with workspaces
+///
+/// feat_lib is used by caller1 and caller2, but with different features enabled.
+/// This test ensures that alternating building caller1, caller2 doesn't force
+/// recompile of feat_lib.
+///
+/// Ideally once we solve https://github.com/rust-lang/cargo/issues/3620, then
+/// a single cargo build at the top level will be enough.
+#[test]
+fn dep_used_with_separate_features() {
+    let p = project("foo")
+        .file("Cargo.toml", r#"
+            [workspace]
+            members = ["feat_lib", "caller1", "caller2"]
+        "#)
+        .file("feat_lib/Cargo.toml", r#"
+            [project]
+            name = "feat_lib"
+            version = "0.1.0"
+            authors = []
+
+            [features]
+            myfeature = []
+        "#)
+        .file("feat_lib/src/lib.rs", "")
+        .file("caller1/Cargo.toml", r#"
+            [project]
+            name = "caller1"
+            version = "0.1.0"
+            authors = []
+
+            [dependencies]
+            feat_lib = { path = "../feat_lib" }
+        "#)
+        .file("caller1/src/main.rs", "fn main() {}")
+        .file("caller1/src/lib.rs", "")
+        .file("caller2/Cargo.toml", r#"
+            [project]
+            name = "caller2"
+            version = "0.1.0"
+            authors = []
+
+            [dependencies]
+            feat_lib = { path = "../feat_lib", features = ["myfeature"] }
+            caller1 = { path = "../caller1" }
+        "#)
+        .file("caller2/src/main.rs", "fn main() {}")
+        .file("caller2/src/lib.rs", "");
+    let p = p.build();
+
+    // Build the entire workspace
+    assert_that(p.cargo("build").arg("--all"),
+                execs().with_status(0)
+                       .with_stderr("\
+[..]Compiling feat_lib v0.1.0 ([..])
+[..]Compiling caller1 v0.1.0 ([..])
+[..]Compiling caller2 v0.1.0 ([..])
+[FINISHED] dev [unoptimized + debuginfo] target(s) in [..]
+"));
+    assert_that(&p.bin("caller1"), existing_file());
+    assert_that(&p.bin("caller2"), existing_file());
+
+
+    // Build caller1. should build the dep library. Because the features
+    // are different than the full workspace, it rebuilds.
+    // Ideally once we solve https://github.com/rust-lang/cargo/issues/3620, then
+    // a single cargo build at the top level will be enough.
+    assert_that(p.cargo("build").cwd(p.root().join("caller1")),
+                execs().with_status(0)
+                       .with_stderr("\
+[..]Compiling feat_lib v0.1.0 ([..])
+[..]Compiling caller1 v0.1.0 ([..])
+[FINISHED] dev [unoptimized + debuginfo] target(s) in [..]
+"));
+
+    // Alternate building caller2/caller1 a few times, just to make sure
+    // features are being built separately.  Should not rebuild anything
+    assert_that(p.cargo("build").cwd(p.root().join("caller2")),
+                execs().with_status(0)
+                       .with_stderr("\
+[FINISHED] dev [unoptimized + debuginfo] target(s) in [..]
+"));
+    assert_that(p.cargo("build").cwd(p.root().join("caller1")),
+                execs().with_status(0)
+                       .with_stderr("\
+[FINISHED] dev [unoptimized + debuginfo] target(s) in [..]
+"));
+    assert_that(p.cargo("build").cwd(p.root().join("caller2")),
+                execs().with_status(0)
+                       .with_stderr("\
+[FINISHED] dev [unoptimized + debuginfo] target(s) in [..]
+"));
+}
+
+#[test]
+fn dont_recurse_out_of_cargo_home() {
+    let git_project = git::new("dep", |project| {
+        project
+            .file("Cargo.toml", r#"
+                [package]
+                name = "dep"
+                version = "0.1.0"
+            "#)
+            .file("src/lib.rs", "")
+            .file("build.rs", r#"
+                use std::env;
+                use std::path::Path;
+                use std::process::{self, Command};
+
+                fn main() {
+                    let cargo = env::var_os("CARGO").unwrap();
+                    let cargo_manifest_dir = env::var_os("CARGO_MANIFEST_DIR").unwrap();
+                    let output = Command::new(cargo)
+                        .args(&["metadata", "--format-version", "1", "--manifest-path"])
+                        .arg(&Path::new(&cargo_manifest_dir).join("Cargo.toml"))
+                        .output()
+                        .unwrap();
+                    if !output.status.success() {
+                        eprintln!("{}", String::from_utf8(output.stderr).unwrap());
+                        process::exit(1);
+                    }
+                }
+            "#)
+    }).unwrap();
+    let p = project("lib")
+        .file("Cargo.toml", &format!(r#"
+            [package]
+            name = "lib"
+            version = "0.1.0"
+
+            [dependencies.dep]
+            git = "{}"
+
+            [workspace]
+        "#, git_project.url()))
+        .file("src/lib.rs", "");
+    let p = p.build();
+
+    assert_that(p.cargo("build").env("CARGO_HOME", p.root().join(".cargo")),
+                execs().with_status(0));
+}
+
+/*FIXME: This fails because of how workspace.exclude and workspace.members are working.
+#[test]
+fn include_and_exclude() {
+    let p = project("foo")
+        .file("Cargo.toml", r#"
+            [workspace]
+            members = ["foo"]
+            exclude = ["foo/bar"]
+            "#)
+        .file("foo/Cargo.toml", r#"
+            [project]
+            name = "foo"
+            version = "0.1.0"
+            authors = []
+            "#)
+        .file("foo/src/lib.rs", "")
+        .file("foo/bar/Cargo.toml", r#"
+            [project]
+            name = "bar"
+            version = "0.1.0"
+            authors = []
+            "#)
+        .file("foo/bar/src/lib.rs", "");
+    p.build();
+
+    assert_that(p.cargo("build").cwd(p.root().join("foo")),
+    execs().with_status(0));
+    assert_that(&p.root().join("target"), existing_dir());
+    assert_that(&p.root().join("foo/target"), is_not(existing_dir()));
+    assert_that(p.cargo("build").cwd(p.root().join("foo/bar")),
+    execs().with_status(0));
+    assert_that(&p.root().join("foo/bar/target"), existing_dir());
+}
+*/
+
+#[test]
+fn cargo_home_at_root_works() {
+    let p = project("lib")
+        .file("Cargo.toml", r#"
+            [package]
+            name = "lib"
+            version = "0.1.0"
+
+            [workspace]
+            members = ["a"]
+        "#)
+        .file("src/lib.rs", "")
+        .file("a/Cargo.toml", r#"
+            [package]
+            name = "a"
+            version = "0.1.0"
+        "#)
+        .file("a/src/lib.rs", "");
+    let p = p.build();
+
+    assert_that(p.cargo("build"), execs().with_status(0));
+    assert_that(p.cargo("build").arg("--frozen").env("CARGO_HOME", p.root()),
+                execs().with_status(0));
+}
+
+#[test]
+fn relative_rustc() {
+    let p = project("the_exe")
+        .file("Cargo.toml", r#"
+            [package]
+            name = "foo"
+            version = "0.1.0"
+        "#)
+        .file("src/main.rs", r#"
+            use std::process::Command;
+            use std::env;
+
+            fn main() {
+                let mut cmd = Command::new("rustc");
+                for arg in env::args_os().skip(1) {
+                    cmd.arg(arg);
+                }
+                std::process::exit(cmd.status().unwrap().code().unwrap());
+            }
+        "#)
+        .build();
+    assert_that(p.cargo("build"), execs().with_status(0));
+
+    let src = p.root()
+        .join("target/debug/foo")
+        .with_extension(env::consts::EXE_EXTENSION);
+
+    Package::new("a", "0.1.0").publish();
+
+    let p = project("lib")
+        .file("Cargo.toml", r#"
+            [package]
+            name = "lib"
+            version = "0.1.0"
+
+            [dependencies]
+            a = "0.1"
+        "#)
+        .file("src/lib.rs", "")
+        .build();
+
+    fs::copy(&src, p.root().join(src.file_name().unwrap())).unwrap();
+
+    let file = format!("./foo{}", env::consts::EXE_SUFFIX);
+    assert_that(p.cargo("build").env("RUSTC", &file),
+                execs().with_status(0));
+}
